/***************************************************************************

  CWindow.cpp

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 1, or (at your option)
  any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

***************************************************************************/

#define __CWINDOW_CPP

#include "gambas.h"
#include "main.h"
#include "CMainWindow.h"

#include <ClanLib/display.h>
#include <ClanLib/core.h>

DECLARE_EVENT(DWINDOW_gotfocus);
DECLARE_EVENT(DWINDOW_lostfocus);
DECLARE_EVENT(DWINDOW_close);

/**************************************************************************/

BEGIN_METHOD (CLDWINDOW_new, GB_INTEGER width; GB_INTEGER height; GB_INTEGER depth)

	DWINDOW = new myWindow (VARG (width), VARG (height), VARGOPT (depth, 0));

END_METHOD

BEGIN_METHOD_VOID (CLDWINDOW_show)

	DWINDOW->show (THIS);

END_METHOD

BEGIN_METHOD (CLDWINDOW_clear, GB_INTEGER color)

	DWINDOW->clear (VARGOPT(color, 0));

END_METHOD

BEGIN_METHOD_VOID (CLDWINDOW_refresh)

	DWINDOW->refresh ();

END_METHOD


BEGIN_PROPERTY(CLDWINDOW_text)

	if (READ_PROPERTY)
	{
		GB.ReturnNewZeroString(DWINDOW->get_title().c_str());
	}
	else
	{
		DWINDOW->setTitle(GB.ToZeroString(PROP(GB_STRING)));
	}		

END_PROPERTY

BEGIN_PROPERTY(CLDWINDOW_width)

	if (READ_PROPERTY)
		GB.ReturnInteger(0);
	else
		DWINDOW->setWidth((int ) PROP(GB_INTEGER));

END_PROPERTY

BEGIN_PROPERTY(CLDWINDOW_height)

	if (READ_PROPERTY)
		GB.ReturnInteger(0);
	else
		DWINDOW->setWidth((int ) PROP(GB_INTEGER));

END_PROPERTY

BEGIN_PROPERTY(CLDWINDOW_fullscreen)

	if (READ_PROPERTY)
		GB.ReturnBoolean(DWINDOW->getFullscreen());
	else
		DWINDOW->setFullscreen(VPROP(GB_BOOLEAN));

END_PROPERTY

/**************************************************************************/
 
GB_DESC CLmainwindow[] =
{
    GB_DECLARE ("MainWindow", sizeof (CLDWINDOW)),
	
	GB_EVENT("GotFocus", NULL, NULL, &DWINDOW_gotfocus),
	GB_EVENT("LostFocus", NULL, NULL, &DWINDOW_lostfocus),
	GB_EVENT("Close", NULL, NULL, &DWINDOW_close),

    GB_METHOD ("_new", NULL, CLDWINDOW_new, "(Width)i(Height)i[(depth)i]"),

    GB_METHOD ("Show", NULL, CLDWINDOW_show, NULL),
	GB_METHOD ("Clear", NULL, CLDWINDOW_clear, "[(color)i]"),
	GB_METHOD ("Refresh", NULL, CLDWINDOW_refresh, NULL),

	GB_PROPERTY ("Text", "s", CLDWINDOW_text),
	GB_PROPERTY ("Title", "s", CLDWINDOW_text),
	GB_PROPERTY ("Caption", "s", CLDWINDOW_text),
	
	GB_PROPERTY("Width", "i", CLDWINDOW_width),
	GB_PROPERTY("W", "i", CLDWINDOW_width),
	GB_PROPERTY("Height", "i", CLDWINDOW_height),
	GB_PROPERTY("H", "i", CLDWINDOW_height),
	GB_PROPERTY("Fullscreen", "b", CLDWINDOW_fullscreen),

	GB_END_DECLARE
};

/***************************************************************************

  Window declaration class 

***************************************************************************/

myWindow::myWindow (int width, int height, int depth)
{
	this->set_size (CL_Size (width, height));
	this->set_bpp (depth);
	this->set_title (GB.Application.Name ());
}

void myWindow::setTitle (char *title)
{
	this->set_title (title);
	
	if (window)
		window->set_title (title);
}

void myWindow::show (CLDWINDOW *object)
{
	_cldwindow = object;
	window = new CL_DisplayWindow (*this);
	CL_Display::set_current_window (window);
	CL_Display::clear (CL_Color::black);
	window->flip (1);
	
	GotFocus = window->sig_got_focus().connect(this, &myWindow::gotFocus);
	LostFocus = window->sig_lost_focus().connect(this, &myWindow::lostFocus);
	MainClose = window->sig_window_close().connect(this, &myWindow::mainClose);
}

void myWindow::setWidth (int width)
{
/*	if (window)
	{
		window->set_size (width, window->get_height ());
		this->set_size 	(CL_Size(width, window->get_height ()));	
	}
	else
	{
		CL_Size size = this->get_size ();
		this->set_size 	(CL_Size(width, size.height));
	}
*/
}

int myWindow::height (int height)
{
	return 0;
}

void myWindow::refresh()
{
	window->flip();
}

void myWindow::clear(int color)
{
	if (!window)
		return;

	CL_Display::set_current_window (window);
	CL_Display::clear (CL_Color(((color >> 16) & 0xFF), ((color >> 16) & 0xFF),
				(color & 0xFF )));
}

int myWindow::getFullscreen ()
{
	if (!window)
		return (this->is_fullscreen());
	else
		return (window->is_fullscreen());
}


void myWindow::setFullscreen(bool choice)
{
	this->set_fullscreen(choice);
	
	if (!window)
		return;
	
	if (!choice && window->is_fullscreen())
	{
		window->set_windowed();
		window->flip(0);
	}
	else if (choice && !window->is_fullscreen())
	{
		window->set_fullscreen(this->get_size().width, this->get_size().height,
				this->get_bpp(), this->get_refresh_rate());
		window->flip(0);
	}
}

void myWindow::gotFocus()
{
	if (GB.CanRaise(_cldwindow, DWINDOW_gotfocus))
		GB.Raise(_cldwindow, DWINDOW_gotfocus, 0);
}

void myWindow::lostFocus()
{
	if (GB.CanRaise(_cldwindow, DWINDOW_lostfocus))
		GB.Raise(_cldwindow, DWINDOW_lostfocus, 0);
}

void myWindow::mainClose()
{
	if (GB.CanRaise(_cldwindow, DWINDOW_close))
		GB.Raise(_cldwindow, DWINDOW_close, 0);
}
