/***************************************************************************

  main.cpp

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 1, or (at your option)
  any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

***************************************************************************/

#define __MAIN_CPP

#include "gambas.h"
#include "main.h"

#include <ClanLib/core.h>
#include <ClanLib/display.h>
#include <ClanLib/gl.h>
 
#include "CMainWindow.h"
#include "CTimer.h"

CL_SetupCore *setup_core;
CL_SetupDisplay *setup_display;
CL_SetupGL *setup_GL;

extern "C" {

GB_INTERFACE GB;

} /* extern "C" */

static void cl_wait(long duration)
{ 
	CL_System::keep_alive (duration);  
}

static void cl_init(void)
{
	setup_core = new CL_SetupCore;
	setup_display = new CL_SetupDisplay;
	setup_GL = new CL_SetupGL;
	
	setup_core->init();
	setup_display->init();
	setup_GL->init();
}

static void cl_exit(void)
{
  setup_GL->deinit();
  setup_display->deinit();
  setup_core->deinit();
}

static int cl_loop(void)
{
	while(1)
	{
		CL_System::keep_alive (1);
	}
	
	return 0;
}

extern "C" {

GB_DESC *GB_CLASSES[] =
{
	CLmainwindow,
	CLtimer,
	
	NULL
};

int GB_INIT(void)
{
	GB.Hook(GB_HOOK_WAIT, (void *)cl_wait);
	GB.Hook(GB_HOOK_LOOP, (void *)cl_loop);
	
	cl_init();
	
	return true;
}

void GB_EXIT()
{
	cl_exit();
}

} /* extern "C" */
