/***************************************************************************

  CTreeView.cpp

  (c) 2004-2005 - Daniel Campos Fernández <danielcampos@netcourrier.com>
  
  GTK+ component
  
  Realizado para la Junta de Extremadura. 
  Consejería de Educación Ciencia y Tecnología. 
  Proyecto gnuLinEx

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 1, or (at your option)
  any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

***************************************************************************/


#define __CLISTVIEW_CPP

#include <stdlib.h>

#include "gambas.h"
#include "main.h"
#include "widgets.h"

#include "CWidget.h"
#include "CContainer.h"
#include "CPicture.h"
#include "CListView.h"

#include <stdio.h>

DECLARE_EVENT(EVENT_Select);    /* selection change */
DECLARE_EVENT(EVENT_Click);     /* simple click */
DECLARE_EVENT(EVENT_Activate);  /* double click */
DECLARE_EVENT(EVENT_Rename);  /* double click */
DECLARE_EVENT(EVENT_Expand);
DECLARE_EVENT(EVENT_Collapse);
DECLARE_EVENT(EVENT_ColumnClick);     /* simple click */


void listView_raise_Select(gListView *sender)
{
	CWIDGET *_ob=GetObject((gControl*)sender);
	
	if (!_ob) return;
	GB.Raise((void*)_ob,EVENT_Select,0);
}
/***************************************************************************

  TreeView

***************************************************************************/



BEGIN_METHOD(CLISTVIEW_new, GB_OBJECT parent)

	CCONTAINER *Parent=(CCONTAINER*)VPROP(GB_OBJECT);

	THIS->widget=new gListView(Parent->widget);
	InitControl(THIS->widget,(CWIDGET*)THIS);
	LISTVIEW->onSelect=listView_raise_Select;

END_METHOD



BEGIN_METHOD_VOID(CLISTVIEW_clear)

	LISTVIEW->clear();

END_METHOD



BEGIN_METHOD(CLISTVIEW_add, GB_STRING key; GB_STRING text; GB_OBJECT picture; GB_STRING after)

	gPicture *pic=NULL;
	char *after=NULL;
	
	if (!LENGTH(key)) { GB.Error("Null key"); return; }
	
	if (!MISSING(picture)) pic=((CPICTURE*)VARG(picture))->picture;
	if (!MISSING(after)) after=GB.ToZeroString(ARG(after));
	
	if (LISTVIEW->add(GB.ToZeroString(ARG(key)),GB.ToZeroString(ARG(text)),pic,after))
	{
		GB.Error ("Key already used: &1",GB.ToZeroString(ARG(key)));
	}

END_METHOD


BEGIN_METHOD(CLISTVIEW_remove, GB_STRING key)

	long index;
	
	index=LISTVIEW->keyIndex(GB.ToZeroString(ARG(key)));
	if (index==-1) { GB.Error ("Unknown item: &1",GB.ToZeroString(ARG(key))); return; }
	LISTVIEW->remove(index);
	
END_METHOD


BEGIN_METHOD(CLISTVIEW_exist, GB_STRING key)

	if ( LISTVIEW->keyIndex(GB.ToZeroString(ARG(key)))>=0)
		GB.ReturnBoolean(true);
	else
		GB.ReturnBoolean(false);

END_METHOD


BEGIN_METHOD(CLISTVIEW_find, GB_INTEGER x; GB_INTEGER y)


  
END_METHOD


BEGIN_METHOD(CLISTVIEW_get, GB_STRING key)

	THIS->index=LISTVIEW->keyIndex(GB.ToZeroString(ARG(key)));
	if (THIS->index!=-1) RETURN_SELF();
	else GB.Error("Unknown item : &1",GB.ToZeroString(ARG(key)));
	

END_METHOD


BEGIN_PROPERTY(CLISTVIEW_mode)

	if (READ_PROPERTY) { GB.ReturnInteger(LISTVIEW->mode()); return; }
	LISTVIEW->setMode(VPROP(GB_INTEGER));

END_PROPERTY


BEGIN_PROPERTY(CLISTVIEW_current)

	long ind;
	char *buf=NULL;
	
	ind=LISTVIEW->index();
	if (ind==-1)
		GB.ReturnNull();
	else
	{
		THIS->index=ind;
		RETURN_SELF();
	}

END_PROPERTY


BEGIN_PROPERTY(CLISTVIEW_key)

	long ind;
	char *buf=NULL;
	
	ind=LISTVIEW->index();
	if (ind==-1)
		GB.ReturnNull();
	else
	{
		buf=LISTVIEW->itemKey(ind);
		GB.ReturnNewString(buf,0);
		if (buf) free(buf);
	}


END_PROPERTY


BEGIN_PROPERTY(CLISTVIEW_item)

	long ind;
	char *buf=NULL;
	
	ind=LISTVIEW->index();
	if (ind==-1)
		GB.ReturnNull();
	else
	{
		THIS->index=ind;
		RETURN_SELF();
	}


END_PROPERTY


BEGIN_PROPERTY(CLISTVIEW_available)



END_PROPERTY




BEGIN_PROPERTY(CLISTVIEW_sorted)



END_PROPERTY


BEGIN_PROPERTY(CLISTVIEW_editable)



END_PROPERTY


BEGIN_PROPERTY(CLISTVIEW_count)

	GB.ReturnInteger(LISTVIEW->count());

END_PROPERTY




BEGIN_METHOD(CLISTVIEW_move_to, GB_STRING key)


END_METHOD


BEGIN_METHOD_VOID(CLISTVIEW_move_current)


END_PROPERTY


BEGIN_METHOD_VOID(CLISTVIEW_first)



END_METHOD

BEGIN_METHOD_VOID(CLISTVIEW_last)



END_METHOD


BEGIN_METHOD_VOID(CLISTVIEWITEM_previous)


END_METHOD

BEGIN_METHOD_VOID(CLISTVIEWITEM_next)



END_METHOD



BEGIN_METHOD_VOID(CLISTVIEWITEM_above)



END_METHOD


BEGIN_METHOD_VOID(CLISTVIEWITEM_below)



END_METHOD



BEGIN_PROPERTY(CLISTVIEWITEM_key)

	char *buf;

	buf=LISTVIEW->itemKey(THIS->index);
	GB.ReturnNewString(buf,0);
	if (buf) free(buf);

END_PROPERTY


BEGIN_PROPERTY(CLISTVIEWITEM_picture)

	CPICTURE *pic=NULL;
	gPicture *ret=NULL;
	
	if (READ_PROPERTY)
	{
		ret=LISTVIEW->itemPicture(THIS->index);
		if (!ret) return;
		GB.New((void **)&pic, GB.FindClass("Picture"), 0, 0);
		if (pic->picture) delete pic->picture;
		pic->picture=ret;
		GB.ReturnObject(pic);
		return;
	}
	
	pic=(CPICTURE*)VPROP(GB_OBJECT);
	if (pic) ret=pic->picture;
	LISTVIEW->setItemPicture(THIS->index,ret);

END_PROPERTY


BEGIN_PROPERTY(CLISTVIEWITEM_selected)

	if (READ_PROPERTY)
	{
		GB.ReturnBoolean(LISTVIEW->itemSelected(THIS->index));
		return;
	}
	
	LISTVIEW->setItemSelected(THIS->index,VPROP(GB_BOOLEAN));

END_PROPERTY


BEGIN_PROPERTY(CLISTVIEWITEM_text)

	char *buf;

	if (READ_PROPERTY)
	{
		buf=LISTVIEW->itemText(THIS->index);
		GB.ReturnNewString(buf,0);
		if (buf) free(buf);
		return;
	}
	
	LISTVIEW->setItemText(THIS->index,GB.ToZeroString(PROP(GB_STRING)));

END_PROPERTY


BEGIN_METHOD_VOID(CLISTVIEWITEM_ensure_visible)



END_METHOD


BEGIN_METHOD_VOID(CLISTVIEWITEM_delete)

	LISTVIEW->remove(THIS->index);

END_METHOD


BEGIN_PROPERTY(CLISTVIEWITEM_editable)



END_PROPERTY



BEGIN_PROPERTY(CLISTVIEW_client_width)



END_PROPERTY


BEGIN_PROPERTY(CLISTVIEW_client_height)



END_PROPERTY

BEGIN_PROPERTY(CLISTVIEW_border_simple)

END_PROPERTY

BEGIN_PROPERTY(CLISTVIEW_scrollbar)

END_PROPERTY

/***************************************************************************

  ListViewItem

***************************************************************************/

GB_DESC CListViewItemDesc[] =
{
  GB_DECLARE(".ListViewItem", 0), GB_VIRTUAL_CLASS(),

  GB_PROPERTY_READ("Key", "s", CLISTVIEWITEM_key),
  GB_PROPERTY("Picture", "Picture", CLISTVIEWITEM_picture),
  GB_PROPERTY("Selected", "b", CLISTVIEWITEM_selected),
  GB_PROPERTY("Text", "s", CLISTVIEWITEM_text),
  GB_METHOD("EnsureVisible", NULL, CLISTVIEWITEM_ensure_visible, NULL),
  GB_METHOD("Delete", NULL, CLISTVIEWITEM_delete, NULL),
  GB_PROPERTY("Editable", "b", CLISTVIEWITEM_editable),

  GB_METHOD("MoveNext", "b", CLISTVIEWITEM_next, NULL),
  GB_METHOD("MovePrevious", "b", CLISTVIEWITEM_previous, NULL),
  GB_METHOD("MoveAbove", "b", CLISTVIEWITEM_above, NULL),
  GB_METHOD("MoveBelow", "b", CLISTVIEWITEM_below, NULL),

  GB_END_DECLARE
};

/***************************************************************************

  ListView

***************************************************************************/

GB_DESC CListViewDesc[] =
{
  GB_DECLARE("ListView", sizeof(CLISTVIEW)), GB_INHERITS("Control"),

  GB_CONSTANT("None", "i", 3),
  GB_CONSTANT("Single", "i", 0),
  GB_CONSTANT("Multi", "i", 1),
  GB_CONSTANT("Extended", "i", 2),

  GB_METHOD("_new", NULL, CLISTVIEW_new, "(Parent)Container;"),

  GB_PROPERTY("Mode", "i<ListView,None,Single,Multi,Extended>", CLISTVIEW_mode),
  GB_PROPERTY("Sorted", "b", CLISTVIEW_sorted),
  GB_PROPERTY("Editable", "b", CLISTVIEW_editable),
  GB_PROPERTY("ScrollBar", "i<Scroll>", CLISTVIEW_scrollbar),
  GB_PROPERTY("Border", "b", CLISTVIEW_border_simple),

  GB_PROPERTY_READ("Count", "i", CLISTVIEW_count),

  GB_PROPERTY_READ("Available", "b", CLISTVIEW_available),
  GB_METHOD("MoveTo", "b", CLISTVIEW_move_to, "(Key)s"),
  GB_METHOD("MoveCurrent", "b", CLISTVIEW_move_current, NULL),
  GB_METHOD("MoveFirst", "b", CLISTVIEW_first, NULL),
  GB_METHOD("MoveLast", "b", CLISTVIEW_last, NULL),
  GB_METHOD("MovePrevious", "b", CLISTVIEWITEM_previous, NULL),
  GB_METHOD("MoveNext", "b", CLISTVIEWITEM_next, NULL),
  GB_METHOD("MoveAbove", "b", CLISTVIEWITEM_above, NULL),
  GB_METHOD("MoveBelow", "b", CLISTVIEWITEM_below, NULL),

  GB_METHOD("_get", ".ListViewItem", CLISTVIEW_get, "(Key)s"),

  GB_METHOD("Clear", NULL, CLISTVIEW_clear, NULL),
  GB_METHOD("Add", ".ListViewItem", CLISTVIEW_add, "(Key)s(Text)s[(Picture)Picture;(After)s]"),
  GB_METHOD("Remove", NULL, CLISTVIEW_remove, "(Key)s"),
  GB_METHOD("Exist", "b", CLISTVIEW_exist, "(Key)s"),
  GB_METHOD("Find", "b", CLISTVIEW_find, "(X)i(Y)i"),

  GB_PROPERTY_READ("Current", ".ListViewItem", CLISTVIEW_current),
  GB_PROPERTY_READ("Key", "s", CLISTVIEW_key),
  GB_PROPERTY_READ("Item", ".ListViewItem", CLISTVIEW_item),

  GB_CONSTANT("_Properties", "s", CLISTVIEW_PROPERTIES),
  GB_CONSTANT("_DefaultEvent", "s", "Click"),

  GB_PROPERTY_READ("ClientWidth", "i", CLISTVIEW_client_width),
  GB_PROPERTY_READ("ClientW", "i",  CLISTVIEW_client_width),
  GB_PROPERTY_READ("ClientHeight", "i", CLISTVIEW_client_height),
  GB_PROPERTY_READ("ClientH", "i", CLISTVIEW_client_height),

  GB_EVENT("Select", NULL, NULL, &EVENT_Select),
  GB_EVENT("Activate", NULL, NULL, &EVENT_Activate),
  GB_EVENT("Click", NULL, NULL, &EVENT_Click),
  GB_EVENT("Rename", NULL, NULL, &EVENT_Rename),

  GB_END_DECLARE
};






