/***************************************************************************

  CPicture.cpp

  (c) 2004-2005 - Daniel Campos Fernández <danielcampos@netcourrier.com>
  
  GTK+ component
  
  Realizado para la Junta de Extremadura. 
  Consejería de Educación Ciencia y Tecnología. 
  Proyecto gnuLinEx
  
  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 1, or (at your option)
  any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

***************************************************************************/

#define __CPICTURE_CPP

#include "main.h"
#include "gambas.h"
#include "widgets.h"
#include "CPicture.h"
#include "CImage.h"

/*******************************************************************************

  class Picture

*******************************************************************************/

BEGIN_METHOD(CPICTURE_get, GB_STRING path)

	char *addr;
	long len;
	CPICTURE *pic=NULL;
	gPicture *buf;
	gPicture *img;
	
	img=gPictureCache::load( GB.ToZeroString(ARG(path)) );
	if (img)
	{
		GB.New((void **)&pic, GB.FindClass("Picture"), 0, 0);
		if (pic->picture) delete pic->picture;
		pic->picture=img;
		GB.ReturnObject(pic);
		return;
	}
	
	
	if (GB.LoadFile (STRING(path),LENGTH(path),&addr,&len)) 
	{
		buf=gPicture::fromStock( GB.ToZeroString(ARG(path)) );
		if (buf)
		{
			GB.New((void **)&pic, GB.FindClass("Picture"), 0, 0);
			if (pic->picture) delete pic->picture;
			pic->picture=buf;
			GB.ReturnObject(pic);
			return;
			
		}
		
		buf=gPicture::fromNamedIcon( GB.ToZeroString(ARG(path)) );
		if (buf)
		{
			GB.New((void **)&pic, GB.FindClass("Picture"), 0, 0);
			if (pic->picture) delete pic->picture;
			pic->picture=buf;
			GB.ReturnObject(pic);
			return;
			
		}
		
		GB.Error("Unable to load image");
		return;
	}
	
	GB.New((void **)&pic, GB.FindClass("Picture"), 0, 0);
	pic->picture->fromMemory(addr,len);
	GB.ReleaseFile (&addr,len);
	
	if (pic) gPictureCache::save(STRING(path),pic->picture);
	
	GB.ReturnObject(pic);

END_METHOD


BEGIN_METHOD(CPICTURE_put, GB_OBJECT picture; GB_STRING path)

	CPICTURE *picture=(CPICTURE*)VARG(picture);
	
	if (picture) gPictureCache::save(STRING(path),picture->picture);

END_METHOD



BEGIN_METHOD_VOID(CPICTURE_flush)

	gPictureCache::flush();

END_METHOD


BEGIN_METHOD(CPICTURE_new, GB_INTEGER w; GB_INTEGER h; GB_BOOLEAN trans)

	gint myw=0,myh=0;
	bool mytrans=false;
	
	if (!MISSING(w)) myw=(gint)VARG(w);
	if (!MISSING(h)) myh=(gint)VARG(h);
	if (!MISSING(trans)) mytrans=VARG(trans);
	
	PICTURE=new gPicture(myw,myh,mytrans);

END_METHOD


BEGIN_METHOD_VOID(CPICTURE_free)

	if (PICTURE) delete PICTURE;

END_METHOD


BEGIN_METHOD(CPICTURE_resize, GB_INTEGER width; GB_INTEGER height)

	PICTURE->resize(VARG(width),VARG(height));

END_METHOD


BEGIN_PROPERTY(CPICTURE_width)

	GB.ReturnInteger(PICTURE->width());

END_PROPERTY


BEGIN_PROPERTY(CPICTURE_height)

	GB.ReturnInteger(PICTURE->height());

END_PROPERTY


BEGIN_PROPERTY(CPICTURE_depth)

	GB.ReturnInteger(PICTURE->depth());

END_PROPERTY


BEGIN_METHOD(CPICTURE_load, GB_STRING path)

	char *addr;
	long len;
	
	if (GB.LoadFile (STRING(path),LENGTH(path),&addr,&len)) 
	{
		GB.Error("Unable to load image");
		return;
	}
	
	PICTURE->fromMemory(addr,len);
	GB.ReleaseFile (&addr,len);


END_METHOD

BEGIN_METHOD(CPICTURE_fromMemory,GB_STRING data;)

	CPICTURE *pic=NULL;
	
	if (!LENGTH(data)) return;	
	
	GB.New((void **)&pic, GB.FindClass("Picture"), 0, 0);
	pic->picture->fromMemory(STRING(data),LENGTH(data));
	GB.ReturnObject(pic);


END_METHOD


BEGIN_METHOD(CPICTURE_save, GB_STRING path)

	char *path=NULL;
	
	if (LENGTH(path)) path=STRING(path);
	switch (PICTURE->save(path))
	{
		case 0: break;
		case -1: GB.Error("Unknown format"); break;
		case -2: GB.Error("Unable to save picture"); break;
	}



END_METHOD


BEGIN_METHOD_VOID(CPICTURE_clear)

	bool trans;

	if (PICTURE) 
	{ 
		trans=PICTURE->transparent();
		delete PICTURE; 
		PICTURE=new gPicture(0,0,trans); 
	}

END_METHOD


BEGIN_METHOD(CPICTURE_fill, GB_INTEGER col)

	PICTURE->Fill(VARG(col));

END_METHOD


BEGIN_METHOD(CPICTURE_copy, GB_INTEGER x; GB_INTEGER y; GB_INTEGER w; GB_INTEGER h)



END_METHOD


BEGIN_PROPERTY(CPICTURE_image)

	CIMAGE *img;

	GB.New((void **)&img, GB.FindClass("Image"), 0, 0);
	if (img->image) delete img->image;
	img->image=PICTURE->getImage();
	GB.ReturnObject((void*)img);
	
END_PROPERTY


BEGIN_PROPERTY(CPICTURE_transparent)

	if (READ_PROPERTY) { GB.ReturnBoolean(PICTURE->transparent()); return; }
	PICTURE->setTransparent(VPROP(GB_INTEGER));	
	
END_PROPERTY


GB_DESC CPictureDesc[] =
{
  GB_DECLARE("Picture", sizeof(CPICTURE)),

  GB_STATIC_METHOD("_exit", NULL, CPICTURE_flush, NULL),

  GB_METHOD("_new", NULL, CPICTURE_new, "[(Width)i(Height)i(Transparent)b]"),
  GB_METHOD("_free", NULL, CPICTURE_free, NULL),
  GB_STATIC_METHOD("_exit",NULL,CPICTURE_flush,NULL),

  GB_STATIC_METHOD("_get", "Picture", CPICTURE_get, "(Path)s"),
  GB_STATIC_METHOD("_put", NULL, CPICTURE_put, "(Picture)Picture;(Path)s"),
  GB_STATIC_METHOD("Flush", NULL, CPICTURE_flush, NULL),

  GB_PROPERTY_READ("Width", "i", CPICTURE_width),
  GB_PROPERTY_READ("Height", "i", CPICTURE_height),
  GB_PROPERTY_READ("Depth", "i", CPICTURE_depth),

  GB_STATIC_METHOD("FromString", "Picture", CPICTURE_fromMemory, "(Data)s"),
  GB_METHOD("Load", NULL, CPICTURE_load, "(Path)s"),
  GB_METHOD("Save", NULL, CPICTURE_save, "(Path)s"),
  GB_METHOD("Resize", NULL, CPICTURE_resize, "(Width)i(Height)i"),

  GB_METHOD("Clear", NULL, CPICTURE_clear, NULL),
  GB_METHOD("Fill", NULL, CPICTURE_fill, "(Color)i"),

  GB_PROPERTY("Transparent", "b", CPICTURE_transparent),

  GB_METHOD("Copy", "Picture", CPICTURE_copy, "[(X)i(Y)i(Width)i(Height)i]"),
  GB_PROPERTY_READ("Image", "Image", CPICTURE_image),

  GB_END_DECLARE
};

