/***************************************************************************

  gcombobox.cpp

  (c) 2004-2005 - Daniel Campos Fernández <danielcampos@netcourrier.com>
  
  Gtkmae "GTK+ made easy" classes
  
  Realizado para la Junta de Extremadura. 
  Consejería de Educación Ciencia y Tecnología. 
  Proyecto gnuLinEx
  
  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 1, or (at your option)
  any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

***************************************************************************/
#include "widgets.h"
#include "widgets_private.h"
#include <gtk/gtk.h>

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#define STORE_LIST ((GtkListStore*)gtk_combo_box_get_model(GTK_COMBO_BOX(border)))

void combo_changed(GtkComboBox *widget,gComboBox *data)
{
	if (data->entry)
		if (data->count()) 
			gtk_entry_set_text(GTK_ENTRY(data->entry),data->itemText(data->index()));

	if (data->onChange) data->onChange(data);
}

gComboBox::gComboBox(gControl *parent) : gControl(parent)
{
	GtkCellRenderer *cell;
	GtkListStore *store;

	onChange=NULL;
	sort=false;
	g_typ=Type_gComboBox;
	entry=NULL;
	store=gtk_list_store_new (1, G_TYPE_STRING);
	border=gtk_combo_box_new_with_model(GTK_TREE_MODEL(store));
	g_object_unref (store);
	cell = gtk_cell_renderer_text_new ();
	gtk_cell_layout_pack_start (GTK_CELL_LAYOUT (border), cell, TRUE);
	gtk_cell_layout_set_attributes (GTK_CELL_LAYOUT (border),cell,"text",0,NULL);
	widget=border;	
	connectParent();
	initSignals();
	
	g_signal_connect(G_OBJECT(border),"changed",G_CALLBACK(combo_changed),(gpointer)this);
	setReadOnly(false);

}

long gComboBox::count()
{
	return gStoreList_Count(STORE_LIST);

}

long gComboBox::index()
{
	GtkTreeModel *store;
	GtkTreeIter iter;
	
	store = gtk_combo_box_get_model(GTK_COMBO_BOX(border));
	if (!gtk_tree_model_get_iter_first(store,&iter)) return 0;

	return gtk_combo_box_get_active (GTK_COMBO_BOX(border));
}

char* gComboBox::itemText(long ind)
{
	GtkTreeModel *store;
	GtkTreeIter iter;
	char *ret=NULL;
	long len=0;
	
	if (ind<0) return NULL;
	if (ind>=count()) return NULL;
	
	store = gtk_combo_box_get_model(GTK_COMBO_BOX(border));
	gtk_tree_model_get_iter_first(store,&iter);
	while (len<ind)
	{
		len++;
		gtk_tree_model_iter_next(store,&iter);
	}
	gtk_tree_model_get (GTK_TREE_MODEL(store),&iter,0,&ret,-1);
	return ret;
}

long gComboBox::length()
{
	const gchar *buf;
	long len;
	
	if (!entry)
	{
		if (!count()) return 0;
		buf=itemText(index());
		if (!buf) return 0;
		
	}
	else
	{
		buf=gtk_entry_get_text(GTK_ENTRY(entry));
		if (!buf) return 0;
	}
	len=g_utf8_strlen(buf,-1);
	g_free((void*)buf);
	return len;
}

char* gComboBox::list()
{
	char *ret=NULL;

	gStoreList_List(STORE_LIST,&ret);
	return ret;
}



bool gComboBox::readOnly()
{
	return !entry;
}

bool gComboBox::sorted()
{
	return sort;
}

char* gComboBox::text()
{
	if (!entry)
	{
		if (count()) return itemText(index());
		return NULL;
	}
	else
	{
		return (char*)gtk_entry_get_text(GTK_ENTRY(entry));
	}
}

void gComboBox::setIndex(long vl)
{
	if (vl<0) return;
	if (vl>=count()) return;
	gtk_combo_box_set_active(GTK_COMBO_BOX(border),vl);
}

void gComboBox::setItemText(long ind,char *txt)
{
	long buf;
	
	if (ind<0) return;
	if (ind>=count()) return;
	buf=index();
	add(txt,ind);
	remove(ind+1);
	setIndex(buf);
	if (sort) gStoreList_sortData(STORE_LIST);
}

void gComboBox::setList(char *vl)
{
	GtkTreeIter iter;

	gStoreList_setList(STORE_LIST,vl);
	
	if (sort) gStoreList_sortData(STORE_LIST);
	if(gtk_tree_model_get_iter_first(GTK_TREE_MODEL(STORE_LIST),&iter))
		gtk_combo_box_set_active_iter(GTK_COMBO_BOX(border),&iter);

}

void gComboBox::setReadOnly(bool vl)
{
	GtkListStore *store;
	gint x,y;
	
	if ( (!vl) && (!entry) )
	{
		entry=gtk_entry_new();
		gtk_widget_show(entry);
		gtk_container_add(GTK_CONTAINER(border),entry);
		if (count()) gtk_entry_set_text(GTK_ENTRY(entry),itemText(index()));
		gtk_widget_get_size_request(border,&x,&y);
		gtk_widget_set_size_request(entry,x,y);
	}
	
	if ( vl && entry )
	{
		gtk_widget_destroy(entry);
		entry=NULL;
	}
}

void gComboBox::setSorted(bool vl)
{
	sort=vl;
	if (sort) gStoreList_sortData(STORE_LIST);
}

void gComboBox::setText(char *vl)
{
	if (!entry) return;
	gtk_entry_set_text(GTK_ENTRY(entry),(const gchar*)vl);
}

void gComboBox::add(char *vl,long pos)
{
	GtkTreeIter iter,curr;
	GtkListStore *store;
	bool set;

	if (pos<0) return;
	if (pos>count()) return;
	
	store = (GtkListStore*)gtk_combo_box_get_model(GTK_COMBO_BOX(border));
	
	if (!gtk_tree_model_get_iter_first(GTK_TREE_MODEL(store),&iter))
	{
		gtk_list_store_append (store, &iter);
		gtk_list_store_set (store, &iter,0,vl, -1);
		gtk_combo_box_set_active_iter(GTK_COMBO_BOX(border),&iter);
		return;
	}
	
	set=gtk_combo_box_get_active_iter(GTK_COMBO_BOX(border),&curr);
	gtk_list_store_insert(store,&iter,pos);
	gtk_list_store_set (store, &iter,0,vl, -1);
	if (set) gtk_combo_box_set_active_iter(GTK_COMBO_BOX(border),&curr); 
	if (sort) gStoreList_sortData(STORE_LIST);
}

void gComboBox::clear()
{
	gStoreList_Clear(STORE_LIST);
}

long gComboBox::find(char *ptr)
{
	return gStoreList_Find(STORE_LIST,ptr);
}

void gComboBox::remove(long pos)
{
	gStoreList_Remove(STORE_LIST,pos);
	if (sort) gStoreList_sortData(STORE_LIST);
}


