/***************************************************************************

  gcontainer.cpp

  (c) 2004-2005 - Daniel Campos Fernández <danielcampos@netcourrier.com>
  
  Gtkmae "GTK+ made easy" classes
  
  Realizado para la Junta de Extremadura. 
  Consejería de Educación Ciencia y Tecnología. 
  Proyecto gnuLinEx
  
  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 1, or (at your option)
  any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

***************************************************************************/
#include <gtk/gtk.h>
#include "widgets.h"

#include <stdio.h>

extern long ncontrols;
extern gControl** controls;

void gContainer::performArrange()
{
	gControl *wid;
	int x, y, w, h, i;
	int xc, yc, wc, hc;
	int sexp, nexp;
	bool swap,autoresize;
	void *ob;
	bool redo;
	bool first;
	
	long chCount;
	long bucle;

	if (design()) return;
	if (locked)  return;
	if (arrange() == ARRANGE_NONE)  return;

	chCount=childCount();
	if (!chCount) return;


	locked = true;

	swap = ((arrange() & 1) == 0);
	autoresize = autoSize() && !expand();

	for (i = 0; i < 3; i++)
	{
		
		redo = false;
		xc = padding();
		yc = padding();
		wc = width() + padding();
		hc = height() + padding();
    
		if (wc <= 0 || hc <= 0) break;
  
		x = xc;
		y = yc;
		w = h = 0;
		wid = 0;
	
		switch (arrange())
		{
			case ARRANGE_HORIZONTAL:
			case ARRANGE_VERTICAL:
		
				sexp = 0;
				nexp = 0;
				
				for(bucle=0;bucle<chCount;bucle++)
				{
					wid = child(bucle);
					
					if (wid->expand()) nexp++;
					else sexp += (swap ? wid->height() : wid->width());
					
					sexp += spacing();
				}
				
				sexp -= spacing();
				sexp = (swap ? hc : wc) - sexp;
				wid = 0;
				
				for(bucle=0;bucle<chCount;bucle++)
				{
					first = wid == 0;
					
					wid = child(bucle);
					
					if (!first)
					{
						if (swap) y += spacing();
						else      x += spacing();
					}
		
					if (swap)
					{
						if (wid->expand())
						{
							h = sexp / nexp;
							sexp -= h;
							nexp--;
						}
						else
							h = wid->height();
						
						if (h > 0)
						{
							if (wc != wid->width() || h != wid->height())
							redo = true;
							
							wid->move(x, y);
							wid->resize(wc, h);
							y += h;
						}
					}
					else
					{
						if (wid->expand())
						{
							w = sexp / nexp;
							sexp -= w;
							nexp--;
						}
						else
							w = wid->width();
						
						if (w > 0)
						{
							if (w != wid->width() || hc != wid->height())
							redo = true;
							
							wid->move(x, y);
							wid->resize(w,hc);
							x += w;
						}
					}
				}
				break;
			
			case ARRANGE_LEFT_RIGHT:
			case ARRANGE_TOP_BOTTOM:
		
				for(bucle=0;bucle<chCount;bucle++)
				{
					wid = child(bucle);
					
					if (swap)
					{
					if ((y > yc) && ((y + wid->height()) > hc))
					{
						y = yc;
						x += w + spacing();
						w = 0;
					}
			
					wid->move(x, y);
					y += wid->height() + spacing();
			
					if (wid->width() > w)
						w = wid->width();
					}
					else
					{
					if ((x > xc) && ((x + wid->width()) > wc))
					{
						x = xc;
						y += h + spacing();
						h = 0;
					}
			
					wid->move(x, y);
					x += wid->width() + spacing();
			
					if (wid->height() > h)
						h = wid->height();
					}
				}
				break;
			
			case ARRANGE_FILL:
			
				for(bucle=0;bucle<chCount;bucle++)
				{
					wid = child(bucle);
					wid->move(xc, yc);
					wid->resize(wc,hc);    
				}
				break;
		}
		
		if (autoresize)
		{
			switch(arrange())
			{
				case ARRANGE_HORIZONTAL: resize(x + width() - wc - xc, height()); break;
				case ARRANGE_TOP_BOTTOM: resize(x + w + width() - wc - xc, height()); break;
				case ARRANGE_VERTICAL: resize(width(), y + height() - hc - yc); break;
				case ARRANGE_LEFT_RIGHT: resize(width(), y + h + height() - hc - yc); break;
			}
		}
		
		if (!redo)  break;
  }

  locked = false;        
}

gContainer::gContainer() 
{
	offX=0;
	offY=0;
	arr=0;
	spc=0;
	pad=0;
	autosz=false;
	locked=false;
	radiogroup=NULL;
}

gContainer::gContainer(gControl *parent) : gControl(parent)
{
	offX=0;
	offY=0;
	arr=0;
	spc=0;
	pad=0;
	locked=false;
	radiogroup=NULL;
}

gContainer::~gContainer()
{
		if (radiogroup) { g_object_unref(G_OBJECT(radiogroup)); radiogroup=NULL; }
}

long gContainer::childCount()
{
	long ret=0;

	GList *chd=gtk_container_get_children(GTK_CONTAINER(widget));
	
	if (!chd) return 0;
	
	chd=g_list_first(chd);
	while (chd)
	{
		ret++;
		chd=g_list_next(chd);
	}
	
	return ret;
}

gControl* gContainer::child(long index)
{
	long bucle;
	long ct=0;
	
	if ( (index<0) || (index>=childCount()) ) return NULL;
	
	for(bucle=0;bucle<ncontrols;bucle++)
	{
		if (controls[bucle]->pr == this)
		{
			if (ct==index) return controls[bucle]; 
			ct++;
		}
	}

	return NULL;
}

long gContainer::arrange()
{
	return arr;
}

void gContainer::setArrange(long vl)
{
	switch(vl)
	{
		case ARRANGE_NONE:
		case ARRANGE_HORIZONTAL:
		case ARRANGE_VERTICAL:
		case ARRANGE_LEFT_RIGHT:
		case ARRANGE_TOP_BOTTOM:
		case ARRANGE_FILL: 
			arr=vl;
			performArrange();
		default: break;
	}
}

long gContainer::padding()
{
	return pad;
}

void gContainer::setPadding(long vl)
{
	if (vl>=0 && vl<=32768) 
	{
		pad=vl;
		performArrange();
	}
}

long gContainer::spacing()
{
	return spc;
}

void gContainer::setSpacing(long vl)
{
	if (vl>=0 && vl<=32768) 
	{
		spc=vl;
		performArrange();
	}
}

bool gContainer::autoSize()
{
	return autosz;
}

void gContainer::setAutoSize(bool vl)
{
	autosz=vl;
	performArrange();
}

long gContainer::clientX()
{
	GtkWidget *chd;
	gint x,y;
	
	if (getClass()==Type_gTabStrip)
	{
		chd=gtk_notebook_get_nth_page(GTK_NOTEBOOK(border),0);
		gtk_widget_translate_coordinates(chd,border,0,0,&x,&y);
		return x;
	}
	
	return 0;
}

long gContainer::clientY()
{
	GtkWidget *chd;
	gint x,y;
	
	if (getClass()==Type_gTabStrip)
	{
		chd=gtk_notebook_get_nth_page(GTK_NOTEBOOK(border),0);
		gtk_widget_translate_coordinates(chd,border,0,0,&x,&y);
		return y;
	}
	
	return 0;
}

long gContainer::clientWidth()
{
	GtkWidget *chd;
	gint x,y;
	
	if (getClass()==Type_gTabStrip)
	{
		chd=gtk_notebook_get_nth_page(GTK_NOTEBOOK(border),0);
		gtk_widget_translate_coordinates(chd,border,0,0,&x,&y);
		return width()-x;
	}
	
	return width();
}

long gContainer::clientHeight()
{
	GtkRequisition req;
	GtkWidget *chd;
	gint x,y;
	
	if (getClass()==Type_gMainWindow)
	{
		if ( ((gMainWindow*)this)->menuBar)
		{
			if ( GTK_WIDGET_VISIBLE( ((gMainWindow*)this)->menuBar ) )
			{
				gtk_widget_size_request( GTK_WIDGET(((gMainWindow*)this)->menuBar),&req);
				return height()-req.height;
			}
		}
	}
	
	if (getClass()==Type_gTabStrip)
	{
		chd=gtk_notebook_get_nth_page(GTK_NOTEBOOK(border),0);
		gtk_widget_translate_coordinates(chd,border,0,0,&x,&y);
		return height()-y;
	}
	return height();
}


