/***************************************************************************

  gdesktop.cpp

  (c) 2004-2005 - Daniel Campos Fernández <danielcampos@netcourrier.com>
  
  Gtkmae "GTK+ made easy" classes
  
  Realizado para la Junta de Extremadura. 
  Consejería de Educación Ciencia y Tecnología. 
  Proyecto gnuLinEx
  
  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 1, or (at your option)
  any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

***************************************************************************/
#include "widgets.h"
#include "widgets_private.h"
#include <gtk/gtk.h>
#include <gdk/gdkx.h>
#include <X11/X.h>
#include <X11/Xlib.h>
#include <string.h>
#include <stdio.h>
#include <stdlib.h>

#if GTK_MAJOR_VERSION == 2
#if GTK_MINOR_VERSION <= 4
#define GTK_2_4
#endif
#endif

static bool gDrag_Enabled=false;
static long gDrag_X=0;
static long gDrag_Y=0;
static int gDrag_Action=0;
static int gDrag_Type=0;
static char *gDrag_Format=NULL;
static char *gDrag_Text=NULL;
static GdkPixbuf *gDrag_Pic=NULL;


static GtkClipboard *clipBoard=NULL;
/***********************************************************************

Clipboard

************************************************************************/
void clipBoard_Init()
{
		if (!clipBoard) clipBoard=gtk_clipboard_get_for_display(gdk_display_get_default         (),GDK_SELECTION_CLIPBOARD);

}



void clipBoard_Clear()
{
	gtk_clipboard_clear(clipBoard);
}

char* clipBoard_getText()
{
	gchar *buf=NULL;
	char *bufOut;
	
	if ( gtk_clipboard_wait_is_text_available(clipBoard) )
		buf=gtk_clipboard_wait_for_text(clipBoard);
		
	if (!buf) return NULL;
	
	bufOut=(char*)malloc(sizeof(char)*(strlen(buf)+1));
	strcpy(bufOut,buf);
	g_free(buf);
	return bufOut;
}

char* clipBoard_Format()
{
	gint n_tg;
	gchar *buf;
	char *bufOut;
	bool ok;
	GdkAtom *targets;
	long bucle,b2,b3;
	long pos=-1;

	if (gtk_clipboard_wait_for_targets  (clipBoard,&targets,&n_tg))
	{
		for (bucle=0;bucle<n_tg;bucle++)
		{
			buf=gdk_atom_name(targets[bucle]);
			for (b2=0;b2<strlen(buf);b2++)
			{
				if ( buf[b2]=='/' )
				{
					if (pos==-1) pos=bucle;
					
					ok=true;
					for (b3=0;b3<strlen(buf);b3++)	
						if (buf[b3]==';') { ok=false; break; }
					if (ok) { pos=bucle; bucle=n_tg; }
					
					break;
				}
			}
			g_free(buf);
		}
		
		if (pos==-1) pos=0;
		
		buf=gdk_atom_name(targets[pos]);
		bufOut=(char*)malloc(sizeof(char)*(strlen(buf)+1));
		strcpy(bufOut,buf);
		g_free(buf);
		return bufOut;
	}
	
	return NULL;
}

void cB_clearFunc(GtkClipboard *clipboard, gpointer clipBoardText)
{
	if (clipBoardText) g_free(clipBoardText);
}

void cB_getFunc(GtkClipboard *clip,GtkSelectionData *selection,guint info,gpointer clipBoardText)
{
	gtk_selection_data_set_text(selection,(const char*)clipBoardText,-1);
}

#ifdef GTK_2_4
void cB_clearPicFunc(GtkClipboard *clipboard, gpointer clipBoardPic)
{
	if (clipBoardPic) g_object_unref(G_OBJECT(clipBoardPic));
}

void cB_getPicFunc(GtkClipboard *clip,GtkSelectionData *selection,guint info,gpointer clipBoardPic)
{
	gchar *buffer=NULL;
	gsize len;

	gdk_pixbuf_save_to_buffer((GdkPixbuf*)clipBoardPic,&buffer,&len,"png",NULL,NULL);
	gtk_selection_data_set (selection,gdk_atom_intern("image/png",false), 8, (guchar *)buffer, len);

}

int clipBoard_Type()
{
	GdkAtom *tg;
  	gint bucle, n_tg;
	
	if (gtk_clipboard_wait_is_text_available(clipBoard)) return 1;
	
	if (gtk_clipboard_wait_for_targets (clipBoard, &tg, &n_tg))
	{
		for (bucle = 0; bucle < n_tg; bucle++)
    	{
      		if (tg[bucle] == gdk_atom_intern("image/png",false)) { g_free(tg); return 2; }
    		if (tg[bucle] == gdk_atom_intern("image/jpeg",false)) { g_free(tg); return 2; }
			if (tg[bucle] == gdk_atom_intern("image/jpg",false)) { g_free(tg); return 2; }
			if (tg[bucle] == gdk_atom_intern("image/gif",false)) { g_free(tg); return 2; }
		}
		g_free(tg);
	}
	
	return 0;
	
}

void clipBoard_setImage(gImage *image)
{
	GtkTargetList *list;
	GList *l;
	GtkTargetEntry *targets;
	gint n_t, i;
	gpointer clipBoardPic=NULL;
	
	gtk_clipboard_clear(clipBoard);
	
	if (!image) return;
	if (!image->image) return;
	
	clipBoardPic=(gpointer)gdk_pixbuf_copy(image->image);
	
	list = gtk_target_list_new (NULL, 0);
	
	gtk_target_list_add (list,gdk_atom_intern("image/png",false),0,0);
	gtk_target_list_add (list,gdk_atom_intern("image/jpg",false),0,0);
	gtk_target_list_add (list,gdk_atom_intern("image/jpeg",false),0,0);
	gtk_target_list_add (list,gdk_atom_intern("image/gif",false),0,0);
	
	n_t = g_list_length (list->list);
	targets = g_new0 (GtkTargetEntry, n_t);
	for (l = list->list, i = 0; l; l = l->next, i++)
	{
		GtkTargetPair *pair = (GtkTargetPair *)l->data;
		targets[i].target = gdk_atom_name (pair->target);
	}
	
	gtk_clipboard_set_with_data (clipBoard,targets, n_t,cB_getPicFunc, cB_clearPicFunc,clipBoardPic);
	
	g_free (targets);
	gtk_target_list_unref (list);	
}

gImage* clipBoard_getImage()
{
	gImage *buf=new gImage(0,0);
	char *atoms[4]={"image/png","image/jpeg","image/jpg","image/gif"};
	long bucle=0;
	GtkSelectionData *sel=NULL;
	GdkPixbufLoader *ld;
	
	do
	{
		sel=gtk_clipboard_wait_for_contents(clipBoard,gdk_atom_intern(atoms[bucle],false));
		bucle++;
		
	} while ( (!sel) && (bucle<4) );
	
	if (sel) 
	{
		ld = gdk_pixbuf_loader_new ();
		if (gdk_pixbuf_loader_write (ld,sel->data,sel->length, NULL))
		{
			buf->image=gdk_pixbuf_loader_get_pixbuf (ld);
			if (buf->image) g_object_ref(buf->image);
		}
		gdk_pixbuf_loader_close (ld, NULL);
  		g_object_unref (ld);
		
	}
	
	return buf;
}

#else
int clipBoard_Type()
{
	if (gtk_clipboard_wait_is_text_available(clipBoard)) return 1;
	if (gtk_clipboard_wait_is_image_available(clipBoard)) return 2;
	
	return 0;
	
}

void clipBoard_setImage(gImage *image)
{
	gtk_clipboard_set_image(clipBoard,image->image);
}

gImage* clipBoard_getImage()
{
	gImage *buf=new gImage(0,0);
	
	buf->image=gtk_clipboard_wait_for_image(clipBoard);

	return buf;
}

#endif 

void clipBoard_setText(char *text,char *format)
{
  GtkTargetList *list;
  GList *l;
  GtkTargetEntry *targets;
  gint n_t, i;
  gpointer clipBoardText;

  gtk_clipboard_clear(clipBoard);
  
  clipBoardText=(gpointer)malloc(sizeof(char)*(strlen(text)+1));
  strcpy((char*)clipBoardText,text);

  list = gtk_target_list_new (NULL, 0);
  gtk_target_list_add (list,gdk_atom_intern(format,false),0,0);
  #ifndef GTK_2_4
  gtk_target_list_add_text_targets (list, 0);
  #else
  gtk_target_list_add (list,gdk_atom_intern("UTF8_STRING",false),0,0);
  gtk_target_list_add (list,gdk_atom_intern("COMPOUND_TEXT",false),0,0);
  gtk_target_list_add (list,gdk_atom_intern("TEXT",false),0,0);
  gtk_target_list_add (list,GDK_TARGET_STRING,0,0);
  gtk_target_list_add (list,gdk_atom_intern("text/plain;charset=utf-8",false),0,0);
  gtk_target_list_add (list,gdk_atom_intern("text/plain",false),0,0);
  #endif
  
  n_t = g_list_length (list->list);
  targets = g_new0 (GtkTargetEntry, n_t);
  for (l = list->list, i = 0; l; l = l->next, i++)
  {
  	GtkTargetPair *pair = (GtkTargetPair *)l->data;
    targets[i].target = gdk_atom_name (pair->target);
  }
  

  gtk_clipboard_set_with_data (clipBoard,targets, n_t,cB_getFunc, cB_clearFunc,clipBoardText);
 
  #ifndef GTK_2_4
  gtk_clipboard_set_can_store (clipBoard,NULL,0);
  #endif
  
  g_free (targets);
  gtk_target_list_unref (list);

}
/***********************************************************************
Drag & Drop
************************************************************************/
//"Private"
void gDrag_Enable(long x,long y,int action)
{
	gDrag_Enabled=true;
	gDrag_X=x;
	gDrag_Y=y;
	gDrag_Action=action;
}

void gDrag_Disable()
{
	gDrag_Enabled=false;
}

void gDrag_setTarget(int type,char *buf)
{
	gDrag_Type=type;
	if (gDrag_Format) { free(gDrag_Format); gDrag_Format=NULL;}
	if (buf)
	{
		gDrag_Format=(char*)malloc(sizeof(char)*(strlen(buf)+1));
		strcpy(gDrag_Format,buf);
	}
	
}

void gDrag_setText(char *buf)
{
	if (gDrag_Text) { free(gDrag_Text); gDrag_Text=NULL; }
	if (!buf) return;
	gDrag_Text=(char*)malloc(sizeof(char)*(strlen(buf)+1));
	strcpy(gDrag_Text,buf);
}

void gDrag_setImage(char *buf,long len)
{
	GdkPixbufLoader *ld;
	
	if (gDrag_Pic) { g_object_unref(G_OBJECT(gDrag_Pic)); gDrag_Pic=NULL; }
	if (!buf) return;
	
	ld = gdk_pixbuf_loader_new ();
	if (gdk_pixbuf_loader_write (ld,(const guchar*)buf,len, NULL))
	{
		gDrag_Pic=gdk_pixbuf_loader_get_pixbuf (ld);
		if (gDrag_Pic) g_object_ref(G_OBJECT(gDrag_Pic));
	}
	gdk_pixbuf_loader_close (ld, NULL);
	g_object_unref (G_OBJECT(ld));
	
}

void gDrag_Clear()
{
	if (gDrag_Format) { free(gDrag_Format); gDrag_Format=NULL; }
	if (gDrag_Text) { free(gDrag_Text); gDrag_Text=NULL; }
}

//"Public"
int drag_Action()
{
	return gDrag_Action;
}

int drag_Type()
{
	return gDrag_Type;
}

char* drag_Format()
{
	return gDrag_Format;
}

char* drag_Text()
{
	return gDrag_Text;
}

gImage* drag_Image()
{
	gImage *ret;

	if (!gDrag_Pic) return NULL;
	ret=new gImage(0,0);
	ret->image=gDrag_Pic;
	g_object_ref(G_OBJECT(gDrag_Pic));
	return ret;
}

bool drag_IsEnabled()
{
	return gDrag_Enabled;
}

long drag_X()
{
	if (!gDrag_Enabled) return -1;
	return gDrag_X;
}

long drag_Y()
{
	if (!gDrag_Enabled) return -1;
	return gDrag_Y;
}


/***********************************************************************

gCursor

************************************************************************/
gCursor::gCursor(gPicture *pic,long px,long py)
{
	GdkPixbuf *buf;
	GdkDisplay *dp=gdk_display_get_default();
	
	x=px;
	y=py;
	
	cur=NULL;
	if (!pic) return;
	if (!pic->pic) return;
	
	if (pic->width()<=x) x=pic->width()-1;
	if (pic->height()<=y) y=pic->height()-1;
	
	buf=pic->getPixbuf();
	cur=gdk_cursor_new_from_pixbuf(dp,buf,x,y);
	g_object_unref(buf);
	
}

gCursor::gCursor(gCursor *cursor)
{
	cur=NULL;
	if (!cursor) return;
	if (!cursor->cur) return;
	
	cur=cursor->cur;
	x=cursor->x;
	y=cursor->y;
	if (cur) gdk_cursor_ref(cur);
}

gCursor::~gCursor()
{
	if (cur) gdk_cursor_unref(cur);
}

long gCursor::left()
{
	return x;
}

long gCursor::top()
{
	return y;
}

/***********************************************************************

gMouse

************************************************************************/
//"Properties"
bool gMouse_isValid=false;
long gMouse_x=0;
long gMouse_y=0;
long gMouse_button=0;
long gMouse_genButton=0;
long gMouse_state=0;

void gMouse::move(long x,long y)
{
	GdkDisplay* dpy;
	GdkWindow*  win=gdk_get_default_root_window();
	
	dpy=gdk_display_get_default();
	XWarpPointer(GDK_DISPLAY_XDISPLAY(dpy),GDK_WINDOW_XID(win),GDK_WINDOW_XID(win),0, 0,0, 0,x, y);
}

long gMouse::button()
{
	if (!gMouse_isValid) return false;
	return gMouse_genButton; 
}

bool gMouse::left()
{
	if (!gMouse_isValid) return false;
	return gMouse_genButton & 1;
}

bool gMouse::right()
{
	if (!gMouse_isValid) return false;
	return gMouse_genButton & 2;
}

bool gMouse::middle()
{
	if (!gMouse_isValid) return false;
	return gMouse_genButton & 4;
}

bool gMouse::shift()
{
	if (!gMouse_isValid) return false;
	return gMouse_state & GDK_SHIFT_MASK;
}

bool gMouse::control()
{
	if (!gMouse_isValid) return false;
	return gMouse_state & GDK_CONTROL_MASK;
}

bool gMouse::alt()
{
	if (!gMouse_isValid) return false;
	return gMouse_state & GDK_MOD1_MASK;
}

bool gMouse::meta()
{
	if (!gMouse_isValid) return false;
	return gMouse_state & GDK_MOD2_MASK;
}

bool gMouse::normal()
{
	if (!gMouse_isValid) return false;
	return !(gMouse_state & 0xFF);
}

long gMouse::x()
{
	if (!gMouse_isValid) return -1;
	return gMouse_x;
}

long gMouse::y()
{
	if (!gMouse_isValid) return -1;
	return gMouse_y;
}

bool gMouse::valid()
{
	return gMouse_isValid;
}



//"Private"
void gMouse::setValid(bool vl,long xval,long yval,long button_val,long state_val)
{
	gMouse_isValid=vl;
	if (vl) { 
		gMouse_x=xval; 
		gMouse_y=yval; 
		gMouse_button=button_val; 
		gMouse_state=state_val; 
		gMouse_genButton=0;
		switch(gMouse_button)
		{
			case 1: gMouse_genButton=1; break;
			case 2: gMouse_genButton=4; break;
			case 3: gMouse_genButton=2; break;
		}
		if (gMouse_state & GDK_BUTTON1_MASK) gMouse_genButton+=1;
		if (gMouse_state & GDK_BUTTON2_MASK) gMouse_genButton+=4;
		if (gMouse_state & GDK_BUTTON3_MASK) gMouse_genButton+=2;
	}
		
}
	
/***********************************************************************

Desktop

************************************************************************/

long gDesktop::height()
{
	return gdk_screen_get_height(gdk_screen_get_default ());
}

long gDesktop::width()
{
	return gdk_screen_get_width(gdk_screen_get_default ());
}

long gDesktop::resolution()
{
	long d_pix=gdk_screen_get_width(gdk_screen_get_default());
	long d_mm=gdk_screen_get_width_mm(gdk_screen_get_default());
	
	return (d_pix*25.4)/d_mm;

}

gPicture* gDesktop::grab()
{
	return Grab_gdkWindow(gdk_get_default_root_window());	
	
}


