/***************************************************************************

  glistview.cpp

  (c) 2004-2005 - Daniel Campos Fernández <danielcampos@netcourrier.com>
  
  Gtkmae "GTK+ made easy" classes
  
  Realizado para la Junta de Extremadura. 
  Consejería de Educación Ciencia y Tecnología. 
  Proyecto gnuLinEx
  
  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 1, or (at your option)
  any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

***************************************************************************/
#include "widgets.h"
#include "widgets_private.h"
#include <gtk/gtk.h>
#include <string.h>
#include <stdlib.h>

#include <stdio.h>

#define STORE_LIST ((GtkListStore*)gtk_tree_view_get_model(GTK_TREE_VIEW(widget)))
/**************************************************************************

gListView

***************************************************************************/
void lv_activated(GtkTreeView *treeview,gListView *data)
{
	if (data->onSelect) data->onSelect(data);
}

gListView::gListView(gControl *parent) : gControl(parent)
{
	GtkListStore *store;
	GtkCellRenderer *renderer;
	GtkTreeViewColumn *column;

	g_typ=Type_gListView;
	sort=false;
	border=gtk_scrolled_window_new (NULL,NULL);
	gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(border),GTK_POLICY_AUTOMATIC,GTK_POLICY_AUTOMATIC);
	gtk_scrolled_window_set_shadow_type(GTK_SCROLLED_WINDOW(border),GTK_SHADOW_IN);
	
	store=gtk_list_store_new (3,GDK_TYPE_PIXBUF,G_TYPE_STRING,G_TYPE_STRING,NULL);
	widget=gtk_tree_view_new_with_model(GTK_TREE_MODEL(store));
	gtk_tree_view_set_headers_visible(GTK_TREE_VIEW(widget),false);
	gtk_container_add(GTK_CONTAINER(border),widget);
	g_object_unref (store);
	
	renderer = gtk_cell_renderer_pixbuf_new ();
	column = gtk_tree_view_column_new_with_attributes ("",renderer,"pixbuf",0,NULL);
	gtk_tree_view_column_set_alignment(column,0);
	gtk_tree_view_append_column (GTK_TREE_VIEW (widget), column);
	renderer = gtk_cell_renderer_text_new ();
	column = gtk_tree_view_column_new_with_attributes ("",renderer,"text",1,NULL);
	gtk_tree_view_column_set_alignment(column,1);
	gtk_tree_view_append_column (GTK_TREE_VIEW (widget), column);
	renderer = gtk_cell_renderer_text_new (); // key
	column = gtk_tree_view_column_new_with_attributes ("",renderer,"text",1,NULL);
	gtk_tree_view_column_set_alignment(column,1);
	gtk_tree_view_append_column (GTK_TREE_VIEW (widget), column);
	gtk_tree_view_column_set_visible(column,false);
		
	connectParent();
	initSignals();

	onSelect=NULL;
	g_signal_connect(G_OBJECT(widget),"cursor-changed",G_CALLBACK(lv_activated),(gpointer)this);
}


long gListView::backGround()
{
	return get_gdk_base_color(widget);
}

void gListView::setBackGround(long color)
{
	GtkTreeViewColumn *column;
	GtkCellRenderer *renderer;
	GList *rend;
	GdkColor gcol;
	
	set_gdk_base_color(widget,color);	
	
	column=gtk_tree_view_get_column(GTK_TREE_VIEW(widget),0);
	rend=gtk_tree_view_column_get_cell_renderers(column);
	renderer=(GtkCellRenderer*)rend->data;
	fill_gdk_color(&gcol,color);
	g_object_set(G_OBJECT(renderer),"cell-background-gdk",&gcol,0);
	g_list_free(rend);
	
	column=gtk_tree_view_get_column(GTK_TREE_VIEW(widget),1);
	rend=gtk_tree_view_column_get_cell_renderers(column);
	renderer=(GtkCellRenderer*)rend->data;
	fill_gdk_color(&gcol,color);
	g_object_set(G_OBJECT(renderer),"cell-background-gdk",&gcol,0);
	g_list_free(rend);
	
	if (!border->window) gtk_widget_realize(border);
	gdk_window_process_updates(border->window,true);
}

long gListView::foreGround()
{
	return get_gdk_text_color(widget);
}

void gListView::setForeGround(long color)
{	
	set_gdk_text_color(widget,color);
	if (!border->window) gtk_widget_realize(border);
	gdk_window_process_updates(border->window,true);
}

int gListView::add(char *key,char *vl,gPicture *pic,char *pos)
{
	GdkPixbuf *buf=NULL;
	GtkTreeIter iter,curr;
	GtkListStore *store;
	bool set;
	char *currkey;
	
	if (!key) return -1;
	if (pic) buf=pic->getPixbuf();
	
	store = (GtkListStore*)gtk_tree_view_get_model(GTK_TREE_VIEW(widget));
	
	if (gtk_tree_model_get_iter_first(GTK_TREE_MODEL(store),&curr))
	{
		do
		{
			gtk_tree_model_get (GTK_TREE_MODEL(store),&curr,2,&currkey,-1);
			if ( !strcmp ( currkey, key ) )
			{
				if (buf) g_object_unref(G_OBJECT(buf));
				g_free(currkey);
				return -1;
			}
			g_free(currkey);
		
		} while ( gtk_tree_model_iter_next(GTK_TREE_MODEL(store),&curr) );
	}
	
	gtk_list_store_append (store, &iter);
	gtk_list_store_set (store, &iter,0,buf,1,vl,2,key,-1);
	return 0;
}

void gListView::clear()
{
	gStoreList_Clear(STORE_LIST);
}

void gListView::remove(long pos)
{
	gStoreList_Remove(STORE_LIST,pos);
}


long gListView::count()
{
	return gStoreList_Count(STORE_LIST);
}

long gListView::index()
{
	GtkTreePath *path;
	GtkTreeViewColumn *col;
	gint *indices;
	
	if (selmode==3) return -1;
	
	gtk_tree_view_get_cursor(GTK_TREE_VIEW(widget),&path,&col);
	if (!path) return -1;
	indices=gtk_tree_path_get_indices(path);
	return indices[0];
}

gPicture* gListView::itemPicture(long ind)
{
	GtkTreeIter iter;
	GtkListStore *store;
	gPicture *ret=NULL;
	GdkPixbuf *buf;
	
	store = (GtkListStore*)gtk_tree_view_get_model(GTK_TREE_VIEW(widget));
	if (gStoreList_Iter(store,&iter,ind)) return NULL;
	
	gtk_tree_model_get (GTK_TREE_MODEL(store),&iter,0,&buf,-1);
	if (buf) ret=gPicture::fromPixbuf(buf);
	g_object_unref(G_OBJECT(buf));
	return ret;
}

bool gListView::itemSelected(long ind)
{
	GtkTreeSelection *sel;
	
	sel=gtk_tree_view_get_selection(GTK_TREE_VIEW(widget));
	return gStoreList_itemSelected(sel,ind);
}

char* gListView::itemText(long ind)
{
	GtkTreeIter iter;
	GtkListStore *store;
	char *ret=NULL;
	long len=0;
	
	store = (GtkListStore*)gtk_tree_view_get_model(GTK_TREE_VIEW(widget));
	if (gStoreList_Iter(store,&iter,ind)) return NULL;
	gtk_tree_model_get (GTK_TREE_MODEL(store),&iter,1,&ret,-1);
	return ret;
}

char* gListView::itemKey(long ind)
{
	GtkTreeIter iter;
	GtkListStore *store;
	char *ret=NULL;
	long len=0;
	
	store = (GtkListStore*)gtk_tree_view_get_model(GTK_TREE_VIEW(widget));
	if (gStoreList_Iter(store,&iter,ind)) return NULL;
	gtk_tree_model_get (GTK_TREE_MODEL(store),&iter,2,&ret,-1);
	return ret;
}

long gListView::keyIndex(char *key)
{
	long ret=0;
	GtkTreeIter curr;
	GtkListStore *store;
	char *currkey;
	
	if (!key) return -1;
	store = (GtkListStore*)gtk_tree_view_get_model(GTK_TREE_VIEW(widget));
	
	if (gtk_tree_model_get_iter_first(GTK_TREE_MODEL(store),&curr))
	{
		do
		{
			gtk_tree_model_get (GTK_TREE_MODEL(store),&curr,2,&currkey,-1);
			if ( !strcmp ( currkey, key ) )
			{
				g_free(currkey);
				return ret;
			}
			g_free(currkey);
			ret++;
		
		} while ( gtk_tree_model_iter_next(GTK_TREE_MODEL(store),&curr) );
	}
	
	return -1;
}

long gListView::mode()
{
	return selmode;
}

void gListView::setItemPicture(long ind,gPicture *pic)
{
	GtkTreeIter iter;
	GtkListStore *store;
	GdkPixbuf *buf=NULL;
	
	store = (GtkListStore*)gtk_tree_view_get_model(GTK_TREE_VIEW(widget));
	if (gStoreList_Iter(store,&iter,ind)) return;
	if (pic) buf=pic->getPixbuf();
	gtk_list_store_set (store,&iter,0,buf,-1);
	if (buf) g_object_unref(buf);
}

void gListView::setItemSelected(long ind,bool vl)
{
	GtkTreeSelection *sel=gtk_tree_view_get_selection(GTK_TREE_VIEW(widget));
	gStoreList_setItemSelected(STORE_LIST,sel,ind,vl);
}

void gListView::setItemText(long ind,char *vl)
{
	GtkTreeIter iter;
	GtkListStore *store;
	
	store = (GtkListStore*)gtk_tree_view_get_model(GTK_TREE_VIEW(widget));
	if (gStoreList_Iter(store,&iter,ind)) return;
	gtk_list_store_set (store,&iter,1,vl,-1);
}

void gListView::setMode(long vl)
{
	GtkTreeSelection* sel=gtk_tree_view_get_selection(GTK_TREE_VIEW(widget));
	
	switch (vl)
	{
		case 0:	//SINGLE
			gtk_tree_selection_set_mode (sel,GTK_SELECTION_SINGLE); 
			selmode=vl; 
			break;
			
		case 1:	//TODO: MULTIPLE
			stub("gListView::setMode - MULTIPLE");
			gtk_tree_selection_set_mode (sel,GTK_SELECTION_MULTIPLE); 
			selmode=vl; 
			break;
			
		case 2:	//EXTENDED
			gtk_tree_selection_set_mode (sel,GTK_SELECTION_MULTIPLE); 
			selmode=vl; 
			break;
			
		case 3:	// None
			gtk_tree_selection_set_mode (sel,GTK_SELECTION_NONE); 
			selmode=vl; 
			break;

	}
}
