/***************************************************************************

  gmainwindow.cpp

  (c) 2004-2005 - Daniel Campos Fernández <danielcampos@netcourrier.com>
  
  Gtkmae "GTK+ made easy" classes
  
  Realizado para la Junta de Extremadura. 
  Consejería de Educación Ciencia y Tecnología. 
  Proyecto gnuLinEx
  
  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 1, or (at your option)
  any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

***************************************************************************/
#include <gtk/gtk.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <ctype.h>
#include <time.h>
#include "widgets.h"
#include "widgets_private.h"


static long nwindows;
static gMainWindow **windows;
extern long ncontrols;
extern gControl** controls;

void stub(char *function)
{
	printf("WARNING: %s not yet implemented\n",function);
}


void win_open (GtkWidget *widget, gMainWindow *data)
{
	if (data->onOpen) data->onOpen(data);
}

void win_show (GtkWidget *widget, gMainWindow *data)
{
	if (data->onShow) data->onShow(data);
}

void win_hide (GtkWidget *widget, gMainWindow *data)
{
	if (data->onHide) data->onHide(data);
}

gboolean win_close(GtkWidget *widget,GdkEvent  *event,gMainWindow *data)
{
	bool ret=false;
		
	if (data->onClose) ret=data->onClose(data);
	return ret; 
}

gboolean win_motion(GtkWidget *widget,GdkEventConfigure *event,gMainWindow *data)
{
	data->performArrange();

	if ( (event->width!=data->bufW) || (event->height!=data->bufH) || (data->resize_flag) )
	{
		data->resize_flag=false;
		data->bufW=event->width;
		data->bufH=event->height;
		if (data->onResize) data->onResize(data);
	}
	
	if ( (event->x!=data->bufX) || (event->y!=data->bufY) )
	{
		data->bufX=event->x;
		data->bufY=event->y;
		if (data->onMove) data->onMove(data);
	}
	
	return false;
}

gMainWindow::gMainWindow(bool plug) 
{
	resize_flag=false;
	bufW=0;
	bufH=0;
	bufX=0;
	bufY=0;
	esc_button=NULL;
	menuBar=NULL;
	win_style=NULL;
	g_typ=Type_gMainWindow;
	top_only=false;
	nwindows++;
	if (nwindows==1)
		windows=(gMainWindow**)malloc(sizeof(gMainWindow*));
	else
		windows=(gMainWindow**)realloc(windows,nwindows*sizeof(gMainWindow*));
	windows[nwindows-1]=this;

	if (plug)
		border=gtk_plug_new(0);
	else
		border=gtk_window_new(GTK_WINDOW_TOPLEVEL);
	
	accel=gtk_accel_group_new();
	gtk_window_add_accel_group(GTK_WINDOW(border), accel);
	
	widget=gtk_layout_new(0,0);
	gtk_container_add(GTK_CONTAINER(border),widget);
	gtk_widget_realize(border);
	gtk_widget_show(widget);
	gtk_widget_set_size_request(border,1,1);
	resize(200,150);
	initSignals();
	onOpen=NULL;
	onShow=NULL;
	onHide=NULL;
	onMove=NULL;
	onResize=NULL;
	g_signal_connect(G_OBJECT(widget),"map",G_CALLBACK(win_open),(gpointer)this);
	g_signal_connect(G_OBJECT(border),"show",G_CALLBACK(win_show),(gpointer)this);
	g_signal_connect(G_OBJECT(border),"hide",G_CALLBACK(win_hide),(gpointer)this);
	g_signal_connect(G_OBJECT(border),"configure-event",G_CALLBACK(win_motion),(gpointer)this);
	g_signal_connect(G_OBJECT(border),"delete-event",G_CALLBACK(win_close),(gpointer)this);
}

gMainWindow::gMainWindow(gControl *parent)
{
	initAll(parent);
	
	resize_flag=false;
	accel=NULL;
	esc_button=NULL;
	menuBar=NULL;
	win_style=NULL;
	g_typ=Type_gEmbedWindow;
	top_only=false;
	
	border=gtk_event_box_new();
	widget=gtk_layout_new(0,0);
	gtk_container_add(GTK_CONTAINER(border),widget);
	gtk_widget_show_all(border);
	pr=parent;
	connectParent();
	initSignals();
	onOpen=NULL;
	onShow=NULL;
	onHide=NULL;
	onMove=NULL;
	onResize=NULL;
	//g_signal_connect(G_OBJECT(widget),"map",G_CALLBACK(win_open),(gpointer)this);
	//g_signal_connect(G_OBJECT(border),"show",G_CALLBACK(win_show),(gpointer)this);
	//g_signal_connect(G_OBJECT(border),"hide",G_CALLBACK(win_hide),(gpointer)this);
	//g_signal_connect(G_OBJECT(border),"configure-event",G_CALLBACK(win_motion),(gpointer)this);
	//g_signal_connect(G_OBJECT(border),"delete-event",G_CALLBACK(win_close),(gpointer)this);
}

gMainWindow::~gMainWindow()
{
	long bucle,b2;
	
	if (pr) return;
	
	for(bucle=0;bucle<nwindows;bucle++)
		if (windows[bucle]==this)
		{
			if (nwindows==1)
			{
				free(windows);
				windows=NULL;
				nwindows=0;
			}
			else
			{
				for(b2=bucle;b2<(nwindows-1);b2++)
					windows[b2]=windows[b2+1];
				nwindows--;
				windows=(gMainWindow**)realloc(windows,nwindows*sizeof(gMainWindow*));
				
			}
		}


}

void gMainWindow::center()
{
	long myx,myy;
	
	if (pr) return;
	
	myx=(gDesktop::width()/2)-(width()/2);
	myy=(gDesktop::height()/2)-(height()/2);
	
	move(myx,myy);
	
	
}

bool gMainWindow::modal()
{
	if (pr) return false;

	return gtk_window_get_modal (GTK_WINDOW(border));
}

void gMainWindow::showModal()
{
	if (pr) return;
	
	gtk_window_set_modal(GTK_WINDOW(border),true);
	gtk_widget_show(border);
}

void gMainWindow::raise()
{
	gtk_window_present(GTK_WINDOW(border));
}

const char* gMainWindow::text()
{
	if (pr) return NULL;

	return gtk_window_get_title(GTK_WINDOW(border));
}

bool gMainWindow::skipTaskBar()
{
	if (pr) return false;
	
	return gtk_window_get_skip_taskbar_hint (GTK_WINDOW(border));
}


void gMainWindow::setText(const char *txt)
{
	if (pr) return;

	gtk_window_set_title(GTK_WINDOW(border),txt);
}

int gMainWindow::_border()
{
	if (pr) return 0;
	
	if (!gtk_window_get_decorated(GTK_WINDOW(border))) return 0;
	if (!gtk_window_get_resizable(GTK_WINDOW(border))) return 1;
	return 2;
}

void gMainWindow::setBorder(int b)
{
	 gint w,h;

	 if (pr) return;
	 
	switch (b)
	{
		case 0: // None
			gtk_window_set_decorated(GTK_WINDOW(border),false);
			break;
		case 1: // Fixed
			gtk_window_set_decorated(GTK_WINDOW(border),true);
			if (gtk_window_get_resizable(GTK_WINDOW(border)))
			{
				gtk_window_get_size(GTK_WINDOW(border),&w,&h);
				gtk_window_set_resizable(GTK_WINDOW(border),false);
				gtk_widget_set_size_request(border,w,h);
			}
			break;
		case 2: // Resizable
			gtk_window_set_decorated(GTK_WINDOW(border),true);
			if (!gtk_window_get_resizable(GTK_WINDOW(border)))
			{
				gtk_window_set_resizable(GTK_WINDOW(border),true);
				gtk_widget_set_size_request(border,1,1);
			}
			break;
	}
}

void gMainWindow::setSkipTaskBar(bool b)
{
	if (pr) return;

	gtk_window_set_skip_taskbar_hint (GTK_WINDOW(border),b);
}


gPicture* gMainWindow::icon()
{
	GdkScreen *scr;
	gint depth;
	gint myw,myh;
	GdkPixbuf *buf;
	gPicture *pic;
	
	if (pr) return NULL;
	
	buf=gtk_window_get_icon(GTK_WINDOW(border));
	if (!buf) return NULL;
	
	pic=gPicture::fromPixbuf(buf);
	
	return pic;
}

void gMainWindow::setIcon(gPicture *pic)
{
	GdkPixbuf *buf;
	
	if (pr) return;

	if (!pic) { gtk_window_set_icon(GTK_WINDOW(border),NULL); return; }
	
	buf=pic->getPixbuf();
	gtk_window_set_icon(GTK_WINDOW(border),buf);
	if (buf) g_object_unref(G_OBJECT(buf));
	
}

bool gMainWindow::topOnly()
{
	if (pr) return false;
	
	return top_only;
}

void gMainWindow::setTopOnly(bool vl)
{
	if (pr) return;

	gtk_window_set_keep_above (GTK_WINDOW(border),vl);
	top_only=vl;
}


gPicture* gMainWindow::mask()
{
	gPicture *pic=new gPicture(0,0,0);
	return pic;
}

void gMainWindow::setMask(gPicture *pic)
{
	GdkBitmap *map=NULL;
	GdkPixbuf *buf;
	
	if (pr) return;
	
	if (pic)
		if (pic->pic)
		{
			buf=pic->getPixbuf();
			map=gdk_pixmap_new(border->window,pic->width(),pic->height(),1);
			gdk_pixbuf_render_threshold_alpha(buf,map,0,0,0,0,-1,-1,255);
			g_object_unref(G_OBJECT(buf));
		}
		
	gtk_widget_shape_combine_mask(border,map,0,0);
	
	if (win_style) { g_object_unref(win_style); win_style=NULL; }
	
	if (map)
	{
		win_style=gtk_style_copy(border->style);
		gtk_widget_set_style(border,win_style);
		g_object_ref(pic->pic);
		border->style->bg_pixmap[GTK_STATE_NORMAL]=pic->pic;
		g_object_unref(G_OBJECT(map));
		gdk_window_invalidate_rect(border->window,NULL,true);
	}
	else
	{
		gtk_widget_set_style(border,NULL);
		gtk_widget_reset_shapes(border);
		gdk_window_invalidate_rect(border->window,NULL,true);
	}
}

long gMainWindow::menuCount()
{
	if (!menuBar) return 0;
	return gMenu::winChildCount(this);
}



/*************************************************************************

gKey

**************************************************************************/
bool gKey_Flags=false;
guint gKey_Key=0;
guint gKey_State=0;


bool gKey::valid()
{
	return gKey_Flags;
}

char* gKey::text()
{
	if (!gKey_Flags) return 0;
	return gdk_keyval_name(gKey_Key);
	
}

int gKey::code()
{
	return gKey_Key;
}

int gKey::state()
{
	return gKey_State;
}

bool gKey::alt()
{
	return gKey_State & GDK_MOD1_MASK;
}

bool gKey::control()
{
	return gKey_State & GDK_CONTROL_MASK;
}

bool gKey::meta()
{
	return gKey_State & GDK_MOD2_MASK;
}

bool gKey::normal()
{
	guint myval=gKey_State & 0xFF;
	if (myval) return false;
	return true;
}

bool gKey::shift()
{
	return gKey_State & GDK_SHIFT_MASK;
}

int gKey::fromString(char* str)
{
	long bucle;
	
	if (!str) return 0;
	for (bucle=0;bucle<strlen(str);bucle++)
		str[bucle]=tolower(str[bucle]);
	return gdk_keyval_from_name (str);
}

void gKey::disable()
{
	gKey_Flags=false;
	gKey_Key=0;
	gKey_State=0;
}

void gKey::enable(GdkEventKey *e)
{
	gKey_Flags=true;
	gKey_Key=e->keyval;
	gKey_State=e->state;
	
}

/********************************************************************

gApplication

*********************************************************************/

long appEvents;

void gApplication::suspendEvents(bool vl)
{
	if (!vl) appEvents=3; //all
	else appEvents=1;     //user
}

void gApplication::enableEvents()
{
	appEvents=0;
}

bool gApplication::userEvents()
{
	if (appEvents) return false;
	return true;
}

bool gApplication::allEvents()
{
	if (appEvents & 2) return false;
	return true;
}

void gApplication::init(int *argc,char ***argv)
{
	//gDrag_Enabled=false;
	appEvents=0;
	
	ncontrols=0;
	controls=NULL;
	
	nwindows=0;
	windows=NULL;
	
	gtk_init(argc,argv);
	
	clipBoard_Init();
}

void gApplication::iteration()
{
	struct timespec mywait;
	
	if (gtk_events_pending ())
	{
	  gtk_main_iteration_do (false);
	}
	else
	{
		mywait.tv_sec=0;
		mywait.tv_nsec=100000;
		nanosleep(&mywait,NULL);
	}
}

long gApplication::controlCount()
{
	return ncontrols;
}

long gApplication::winCount()
{
	return nwindows;
}

gControl* gApplication::controlItem(GtkWidget *wid)
{
	long bucle;
	
	for (bucle=0;bucle<ncontrols;bucle++)
		if ( controls[bucle]->border==wid )
			return controls[bucle];
	
	return NULL;
}

gControl* gApplication::controlItem(long index)
{
	if ( (index<0) || (index>=ncontrols) ) return NULL;
	return controls[index];
}

gMainWindow* gApplication::winItem(long index)
{
	if ( (index<0) || (index>=nwindows) ) return NULL;
	return windows[index];
}
