/***************************************************************************

  gpicture.cpp

  (c) 2004-2005 - Daniel Campos Fernández <danielcampos@netcourrier.com>
  
  Gtkmae "GTK+ made easy" classes
  
  Realizado para la Junta de Extremadura. 
  Consejería de Educación Ciencia y Tecnología. 
  Proyecto gnuLinEx
  
  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 1, or (at your option)
  any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

***************************************************************************/
#include <gtk/gtk.h>
#include <gdk-pixbuf/gdk-pixbuf.h>
#include <stdlib.h>
#include <string.h>
#include <errno.h>
#include "widgets.h"

#include <stdio.h>
#include <math.h>

#if GTK_MAJOR_VERSION == 2
#if GTK_MINOR_VERSION <= 4
#define GTK_2_4
#endif
#endif

int pixbufFromMemory(GdkPixbuf **img,char *addr,unsigned int len)
{
	GdkPixbufLoader* loader;
	
	loader=gdk_pixbuf_loader_new();
	if (!gdk_pixbuf_loader_write(loader,(guchar*)addr,(gsize)len,NULL)){
		 g_object_unref(G_OBJECT(loader));
		 return 0;
	}
	gdk_pixbuf_loader_close(loader,NULL);
	(*img)=gdk_pixbuf_loader_get_pixbuf(loader);
	g_object_ref(G_OBJECT(*img));
	g_object_unref(G_OBJECT(loader));
	return 1;
}
/*****************************************************************

gPicture

******************************************************************/
void gPicture::freeMask()
{
	gint x,y;
	gint b;
	
	if (!mask) return;
	if (!pic) return;

	gdk_drawable_get_size (pic, &x,&y);
	for (b=0;b<y;b++) free(mask[b]);
	free(mask);
	mask=NULL;
}

void gPicture::setMask(GdkPixbuf *pixbuf)
{
	gint w,h;
	gint x,y;
	gint rowstride,nchannels;
	guchar *pixels, *p;

	freeMask();
	
	if (gdk_pixbuf_get_colorspace (pixbuf) != GDK_COLORSPACE_RGB ) return;
	if (gdk_pixbuf_get_bits_per_sample (pixbuf) != 8 ) return;
	if ((nchannels=gdk_pixbuf_get_n_channels (pixbuf)) != 4) return;

	w = gdk_pixbuf_get_width (pixbuf);
  	h = gdk_pixbuf_get_height (pixbuf);
	
	mask=(char**)malloc(h*sizeof(char*));
	for(y=0;y<h;y++) mask[y]=(char*)malloc(w*sizeof(char));
	
	rowstride = gdk_pixbuf_get_rowstride (pixbuf);
  	pixels = gdk_pixbuf_get_pixels (pixbuf);
	for (y=0;y<h;y++)
	{
		for (x=0;x<w;x++)
		{
  			p = pixels + y * rowstride + x * nchannels;
			mask[y][x]=p[3];
		}
	}

	
}

GdkPixbuf* gPicture::getPixbuf()
{
	GdkPixbuf* pixbuf;
	gint w,h;
	gint x,y;
	gint rowstride,nchannels;
	guchar *pixels, *p;
	
	if (!pic) return NULL;
	
	gdk_drawable_get_size (pic, &w,&h);
	
	pixbuf=gdk_pixbuf_new(GDK_COLORSPACE_RGB,TRUE,8,w,h);
	pixbuf=gdk_pixbuf_get_from_drawable(pixbuf,pic,NULL,0,0,0,0,w,h);
	
	if (!mask) return pixbuf;
	
	if (gdk_pixbuf_get_colorspace (pixbuf) != GDK_COLORSPACE_RGB ) return NULL;
	if (gdk_pixbuf_get_bits_per_sample (pixbuf) != 8 ) return NULL; 
	if ((nchannels=gdk_pixbuf_get_n_channels (pixbuf)) != 4) return NULL;
	
	rowstride = gdk_pixbuf_get_rowstride (pixbuf);
  	pixels = gdk_pixbuf_get_pixels (pixbuf);
	for (y=0;y<h;y++)
	{
		for (x=0;x<w;x++)
		{
  			p = pixels + y*rowstride  + x*nchannels;
			p[3]=mask[y][x];
			
		}
	}
	
	return pixbuf;
	
}

gPicture::gPicture(long w,long h,bool trans)
{
	GdkScreen*  scr;
	gint depth;

	pic=NULL;
	mask=NULL;
	_transparent=trans;
	
	if (w<=0)
	{
		if (h>0) w=h;
		else     w=0;	
	}
	if (h<=0)
	{
		if (w>0) h=w;
		else     h=0;	
	}
	
	if ( (!w) && (!h) ) return;
	
	scr=gdk_screen_get_default();
	depth=(gdk_screen_get_system_visual(scr))->depth;
	
	pic=gdk_pixmap_new (NULL,w,h,depth);
	gdk_drawable_set_colormap(GDK_DRAWABLE(pic),gdk_colormap_get_system());
}

gPicture::~gPicture()
{
	freeMask();
	if (pic) {
		g_object_unref(G_OBJECT(pic));
		pic=NULL;
	}
}

long gPicture::width()
{
	gint myw=0;
		
	if (pic) gdk_drawable_get_size(GDK_DRAWABLE(pic),&myw,NULL);
	return myw;
}

long gPicture::height()
{
	gint myh=0;
		
	if (pic) gdk_drawable_get_size(GDK_DRAWABLE(pic),NULL,&myh);
	return myh;
}

long gPicture::depth()
{
	gint depth=0;
	
	if (pic) depth=gdk_drawable_get_depth(GDK_DRAWABLE(pic));
	return depth;
}

bool gPicture::transparent()
{
	return _transparent;
}

void gPicture::setTransparent(bool vl)
{
	_transparent=vl;
}

void gPicture::Fill(long col)
{
	GdkColormap* map;
	GdkColor color;
	GdkGC*   gc;  
	gint myw,myh; 
	
	if (!pic) return;
	map=gdk_colormap_get_system();
	
	color.red=0xFF + ((col & 0xFF0000)>>8);
	color.green=0xFF + (col & 0x00FF00);
	color.blue=0xFF + ((col & 0x0000FF)<<8);

	if(gdk_colormap_alloc_color(map,&color,false,true))
	{
		gdk_drawable_get_size(GDK_DRAWABLE(pic),&myw,&myh);
		gc=gdk_gc_new(pic);
		gdk_gc_set_foreground(gc,&color);
		gdk_gc_set_background(gc,&color);
		gdk_draw_rectangle(pic,gc,true,0,0,myw,myh);
		g_free(G_OBJECT(gc));
	}
}

gImage* gPicture::getImage()
{
	gImage *img;
	
	img=new gImage(0,0);
	
	if (pic) img->image=getPixbuf();
	
	return img;
}

void gPicture::fromMemory(char *addr,unsigned int len)
{
	GdkPixbuf *img;
	GdkScreen* scr;
	gint depth;
	gint myw,myh;
	
	if (!pixbufFromMemory(&img,addr,len)) return;
	
	_transparent=false;
	freeMask();
	if (pic) g_object_unref(pic);

	myw=gdk_pixbuf_get_width (img);
	myh=gdk_pixbuf_get_height(img);
	if (gdk_pixbuf_get_has_alpha(img)) _transparent=true;
	
	scr=gdk_screen_get_default();
	depth=(gdk_screen_get_system_visual(scr))->depth;
	pic=gdk_pixmap_new (NULL,myw,myh,depth);
	gdk_drawable_set_colormap(GDK_DRAWABLE(pic),gdk_colormap_get_system());
	gdk_draw_pixbuf(pic,NULL,img,0,0,0,0,-1,-1,GDK_RGB_DITHER_MAX,0,0);                         
	setMask(img);
	g_object_unref(G_OBJECT(img));
}

gPicture* gPicture::fromPixbuf(GdkPixbuf *buf)
{	
	gPicture *pic;
	GdkScreen *scr;
	gint depth;
	gint myw,myh;
	
	if (!buf) return NULL;

	pic=new gPicture(0,0,false);
	pic->setMask(buf);
	if (gdk_pixbuf_get_has_alpha(buf)) pic->setTransparent(true);
	scr=gdk_screen_get_default();
	depth=(gdk_screen_get_system_visual(scr))->depth;
	myw=gdk_pixbuf_get_width(buf);
	myh=gdk_pixbuf_get_height(buf);
	pic->pic=gdk_pixmap_new (NULL,myw,myh,depth);
	gdk_drawable_set_colormap(GDK_DRAWABLE(pic->pic),gdk_colormap_get_system());
	gdk_draw_pixbuf(pic->pic,NULL,buf,0,0,0,0,-1,-1,GDK_RGB_DITHER_MAX,0,0); 

	return pic;
}

/***********************************************************************
 The following function tries to load an icon from predefined or "stock"
 items. It accepts the format: StockSize/IconName, where StockSize can be:
 
 "Menu", "SmallToolBar","LargeToolBar","Button","Dnd","Dialog"
 
 And IconName can be:
 
 "Add","Apply","Bold","Cancel",
 "CDRom","Clear","Close","ColorPicker",
 "Convert","Copy","Cut","Delete",
 "DialogAuthentication""DialogError","DialogInfo","DialogQuestion",
 "DialogWarning","Dnd","DndMultiple", "Execute",
 "Find","FindAndReplace","Floppy","GotoBottom",
 "GotoFirst", "GotoLast","GotoTop","GoBack",
 "GoDown","GoForward","GoUp","HardDisk"
 "Help","Home","Indent","Index",
 "Italic","JumpTo","JustifyCenter", "JustifyFill",
 "JustifyLeft","JustifyRight","MissingImage","Network",
 "New","No","Ok","Open",
 "Paste","Preferences","Print","PrintPreview",
 "Properties","Quit","Redo","Refresh",
 "Remove","RevertToSaved","Save","SaveAs",
 "SelectColor","SelectFont","SortAscending","SortDescending",
 "SpellCheck","Stop","StrikeThrough","Undelete",
 "Underline","Undo","Unindent","Yes",
 "Zoom100","ZoomFit","ZoomIn","ZoomOut"
*************************************************************************/
gPicture* gPicture::fromStock(char *name)
{
	GtkIconSet *item;
	GdkPixbuf *buf;
	GtkStyle *st;
	long r_type=GTK_ICON_SIZE_BUTTON;
	char *r_name=NULL;	
		
	r_name=strchr(name,'/');
	
	if (!r_name) { r_name=name; }
	else 
	{
		r_name[0]=0; r_name++;
		if      (!strcasecmp(name,"menu"))         r_type=GTK_ICON_SIZE_MENU;
		else if (!strcasecmp(name,"smalltoolbar")) r_type=GTK_ICON_SIZE_SMALL_TOOLBAR;
		else if (!strcasecmp(name,"largetoolbar")) r_type=GTK_ICON_SIZE_LARGE_TOOLBAR;
		else if (!strcasecmp(name,"button"))       r_type=GTK_ICON_SIZE_BUTTON;
		else if (!strcasecmp(name,"dnd"))          r_type=GTK_ICON_SIZE_DND;
		else if (!strcasecmp(name,"dialog"))       r_type=GTK_ICON_SIZE_DIALOG;
		else { r_name--; r_name[0]='/'; return NULL; }
	}
	
	if      (!strcasecmp(r_name,"Add"))                  r_name=GTK_STOCK_ADD;
	else if (!strcasecmp(r_name,"Apply"))                r_name=GTK_STOCK_APPLY;
	else if (!strcasecmp(r_name,"Bold"))                 r_name=GTK_STOCK_BOLD;
	else if (!strcasecmp(r_name,"Cancel"))               r_name=GTK_STOCK_CANCEL;
	else if (!strcasecmp(r_name,"CDRom"))                r_name=GTK_STOCK_CDROM;
	else if (!strcasecmp(r_name,"Clear"))                r_name=GTK_STOCK_CLEAR;
	else if (!strcasecmp(r_name,"Close"))                r_name=GTK_STOCK_CLOSE;
	else if (!strcasecmp(r_name,"ColorPicker"))          r_name=GTK_STOCK_COLOR_PICKER;
	else if (!strcasecmp(r_name,"Convert"))              r_name=GTK_STOCK_CONVERT;
	else if (!strcasecmp(r_name,"Copy"))                 r_name=GTK_STOCK_COPY;
	else if (!strcasecmp(r_name,"Cut"))                  r_name=GTK_STOCK_CUT;
	else if (!strcasecmp(r_name,"Delete"))               r_name=GTK_STOCK_DELETE;
	else if (!strcasecmp(r_name,"DialogAuthentication")) r_name=GTK_STOCK_DIALOG_AUTHENTICATION;
	else if (!strcasecmp(r_name,"DialogError"))          r_name=GTK_STOCK_DIALOG_ERROR;
	else if (!strcasecmp(r_name,"DialogInfo"))           r_name=GTK_STOCK_DIALOG_INFO;
	else if (!strcasecmp(r_name,"DialogQuestion"))       r_name=GTK_STOCK_DIALOG_QUESTION;
	else if (!strcasecmp(r_name,"DialogWarning"))        r_name=GTK_STOCK_DIALOG_WARNING;
	else if (!strcasecmp(r_name,"Dnd"))                  r_name=GTK_STOCK_DND;
	else if (!strcasecmp(r_name,"DndMultiple"))          r_name=GTK_STOCK_DND_MULTIPLE;
	else if (!strcasecmp(r_name,"Execute"))              r_name=GTK_STOCK_EXECUTE;
	else if (!strcasecmp(r_name,"Find"))                 r_name=GTK_STOCK_FIND;
	else if (!strcasecmp(r_name,"FindAndReplace"))       r_name=GTK_STOCK_FIND_AND_REPLACE;
	else if (!strcasecmp(r_name,"Floppy"))               r_name=GTK_STOCK_FLOPPY;
	else if (!strcasecmp(r_name,"GotoBottom"))           r_name=GTK_STOCK_GOTO_BOTTOM;
	else if (!strcasecmp(r_name,"GotoFirst"))            r_name=GTK_STOCK_GOTO_FIRST;
	else if (!strcasecmp(r_name,"GotoLast"))             r_name=GTK_STOCK_GOTO_LAST;
	else if (!strcasecmp(r_name,"GotoTop"))              r_name=GTK_STOCK_GOTO_TOP;
	else if (!strcasecmp(r_name,"GoBack"))               r_name=GTK_STOCK_GO_BACK;
	else if (!strcasecmp(r_name,"GoDown"))               r_name=GTK_STOCK_GO_DOWN;
	else if (!strcasecmp(r_name,"GoForward"))            r_name=GTK_STOCK_GO_FORWARD;
	else if (!strcasecmp(r_name,"GoUp"))                 r_name=GTK_STOCK_GO_UP;
	else if (!strcasecmp(r_name,"HardDisk"))             r_name=GTK_STOCK_HARDDISK;
	else if (!strcasecmp(r_name,"Help"))                 r_name=GTK_STOCK_HELP;
	else if (!strcasecmp(r_name,"Home"))                 r_name=GTK_STOCK_HOME;
	else if (!strcasecmp(r_name,"Indent"))               r_name=GTK_STOCK_INDENT;
	else if (!strcasecmp(r_name,"Index"))                r_name=GTK_STOCK_INDEX;
	else if (!strcasecmp(r_name,"Italic"))               r_name=GTK_STOCK_ITALIC;
	else if (!strcasecmp(r_name,"JumpTo"))               r_name=GTK_STOCK_JUMP_TO;
	else if (!strcasecmp(r_name,"JustifyCenter"))        r_name=GTK_STOCK_JUSTIFY_CENTER;
	else if (!strcasecmp(r_name,"JustifyFill"))          r_name=GTK_STOCK_JUSTIFY_FILL;
	else if (!strcasecmp(r_name,"JustifyLeft"))          r_name=GTK_STOCK_JUSTIFY_LEFT;
	else if (!strcasecmp(r_name,"JustifyRight"))         r_name=GTK_STOCK_JUSTIFY_RIGHT;
	else if (!strcasecmp(r_name,"MissingImage"))         r_name=GTK_STOCK_MISSING_IMAGE;
	else if (!strcasecmp(r_name,"Network"))              r_name=GTK_STOCK_NETWORK;
	else if (!strcasecmp(r_name,"New"))                  r_name=GTK_STOCK_NEW;
	else if (!strcasecmp(r_name,"No"))                   r_name=GTK_STOCK_NO;
	else if (!strcasecmp(r_name,"Ok"))                   r_name=GTK_STOCK_OK;
	else if (!strcasecmp(r_name,"Open"))                 r_name=GTK_STOCK_OPEN;
	else if (!strcasecmp(r_name,"Paste"))                r_name=GTK_STOCK_PASTE;
	else if (!strcasecmp(r_name,"Preferences"))          r_name=GTK_STOCK_PREFERENCES;
	else if (!strcasecmp(r_name,"Print"))                r_name=GTK_STOCK_PRINT;
	else if (!strcasecmp(r_name,"PrintPreview"))         r_name=GTK_STOCK_PRINT_PREVIEW;
	else if (!strcasecmp(r_name,"Properties"))           r_name=GTK_STOCK_PROPERTIES;
	else if (!strcasecmp(r_name,"Quit"))                 r_name=GTK_STOCK_QUIT;
	else if (!strcasecmp(r_name,"Redo"))                 r_name=GTK_STOCK_REDO;
	else if (!strcasecmp(r_name,"Refresh"))              r_name=GTK_STOCK_REFRESH;
	else if (!strcasecmp(r_name,"Remove"))               r_name=GTK_STOCK_REMOVE;
	else if (!strcasecmp(r_name,"RevertToSaved"))        r_name=GTK_STOCK_REVERT_TO_SAVED;
	else if (!strcasecmp(r_name,"Save"))                 r_name=GTK_STOCK_SAVE;
	else if (!strcasecmp(r_name,"SaveAs"))               r_name=GTK_STOCK_SAVE_AS;
	else if (!strcasecmp(r_name,"SelectColor"))          r_name=GTK_STOCK_SELECT_COLOR;
	else if (!strcasecmp(r_name,"SelectFont"))           r_name=GTK_STOCK_SELECT_FONT;
	else if (!strcasecmp(r_name,"SortAscending"))        r_name=GTK_STOCK_SORT_ASCENDING;
	else if (!strcasecmp(r_name,"SortDescending"))       r_name=GTK_STOCK_SORT_DESCENDING;
	else if (!strcasecmp(r_name,"SpellCheck"))           r_name=GTK_STOCK_SPELL_CHECK;
	else if (!strcasecmp(r_name,"Stop"))                 r_name=GTK_STOCK_STOP;
	else if (!strcasecmp(r_name,"StrikeThrough"))        r_name=GTK_STOCK_STRIKETHROUGH;
	else if (!strcasecmp(r_name,"Undelete"))             r_name=GTK_STOCK_UNDELETE;
	else if (!strcasecmp(r_name,"Underline"))            r_name=GTK_STOCK_UNDERLINE;
	else if (!strcasecmp(r_name,"Undo"))                 r_name=GTK_STOCK_UNDO;
	else if (!strcasecmp(r_name,"Unindent"))             r_name=GTK_STOCK_UNINDENT;
	else if (!strcasecmp(r_name,"Yes"))                  r_name=GTK_STOCK_YES;
	else if (!strcasecmp(r_name,"Zoom100"))              r_name=GTK_STOCK_ZOOM_100;
	else if (!strcasecmp(r_name,"ZoomFit"))              r_name=GTK_STOCK_ZOOM_FIT;
	else if (!strcasecmp(r_name,"ZoomIn"))               r_name=GTK_STOCK_ZOOM_IN;
	else if (!strcasecmp(r_name,"ZoomOut"))              r_name=GTK_STOCK_ZOOM_OUT;
	
	item=gtk_icon_factory_lookup_default (r_name);
	if (!item) return NULL;
	
	st=gtk_widget_get_default_style();
	buf=gtk_icon_set_render_icon(item,st,GTK_TEXT_DIR_NONE,GTK_STATE_NORMAL,(GtkIconSize)r_type,NULL,NULL);
	return gPicture::fromPixbuf(buf);	
}

/***********************************************************************
 The following function tries to load an icon from predefined system
 paths
 ***********************************************************************/
gPicture* gPicture::fromNamedIcon(char* name)
{
	GtkIconTheme* theme;
	GdkPixbuf *buf;
	gPicture *pic=NULL;
	long r_type=32;
	char *r_name=NULL;	
		
	r_name=strchr(name,'/');
	
	if (!r_name) { r_name=name; }
	else 
	{
		r_name[0]=0; r_name++;
		if      (!strcasecmp(name,"menu"))         r_type=8;
		else if (!strcasecmp(name,"smalltoolbar")) r_type=16;
		else if (!strcasecmp(name,"largetoolbar")) r_type=32;
		else if (!strcasecmp(name,"button"))       r_type=16;
		else if (!strcasecmp(name,"dnd"))          r_type=32;
		else if (!strcasecmp(name,"dialog"))       r_type=48;
		else { r_name--; r_name[0]='/'; return NULL; }
	}
	
	
	theme=gtk_icon_theme_get_default();
	buf=gtk_icon_theme_load_icon(theme,r_name,r_type,GTK_ICON_LOOKUP_USE_BUILTIN,NULL);
	if (!buf) return NULL;
	
	pic=gPicture::fromPixbuf(buf);
	g_object_unref(buf);
	
	return pic;
}

void  gPicture::resize(int width,int height)
{
	GdkPixbuf *pixbuf;
	GdkScreen*  scr;
	gint depth;
	
	if (width<0) width=height;
	if (height<0) height=width;
	if ( (width<=0) || (height<=0) ) return;
	
		
	pixbuf=getPixbuf();
	freeMask();
	if (pic) g_object_unref(G_OBJECT(pic));
	pic=NULL;
	if (!pixbuf) return;
	
	scr=gdk_screen_get_default();
	depth=(gdk_screen_get_system_visual(scr))->depth;
	pic=gdk_pixmap_new (NULL,width,height,depth);
	gdk_drawable_set_colormap(GDK_DRAWABLE(pic),gdk_colormap_get_system());
	gdk_draw_pixbuf(pic,NULL,pixbuf,0,0,0,0,width,height,GDK_RGB_DITHER_MAX,0,0);
	setMask(pixbuf);
	g_object_unref(G_OBJECT(pixbuf));	
}

/*****************************************************************

gImage

******************************************************************/

gImage::gImage(long w,long h)
{
	image=NULL;
	
	if (w<=0)
	{
		if (h>0) w=h;
		else     w=0;	
	}
	if (h<=0)
	{
		if (w>0) h=w;
		else     h=0;	
	}
	
	if ( (!w) && (!h) ) return;
	
	image=gdk_pixbuf_new(GDK_COLORSPACE_RGB,TRUE,8,w,h);
}

gImage::~gImage()
{
	if (image) g_object_unref(G_OBJECT(image));
}

long gImage::width()
{
	if (!image) return 0;
	return gdk_pixbuf_get_width (image);
}

long gImage::height()
{
	if (!image) return 0;
	return gdk_pixbuf_get_height (image);
}


gPicture* gImage::getPicture()
{
	return gPicture::fromPixbuf(image);
}

void gImage::fromMemory(char *addr,unsigned int len)
{
	GdkPixbuf *img;
	
	if (!pixbufFromMemory(&img,addr,len)) return;
	
	if (image) g_object_unref(G_OBJECT(image));
	image=img;
}

int gPicture::save(char *path)
{
	gImage *img=getImage();
	int ret;
	
	ret=img->save(path);
	delete img;
	return ret;
}

int gImage::save(char *path)
{
	// returns -> 0, OK / -1, Bad format / -2 invalid path
	bool ok=false;
	long b;
	char *type;
	char *buf=NULL;
	GSList *formats = gdk_pixbuf_get_formats();
	GSList *iter=formats;
	
	for (b=strlen(path)-1;b>=0;b--)
		if (path[b]=='.') { buf=path+b+1; break; }
	
	if (!buf) return -1;

	while (iter && (!ok) )
	{
		if (gdk_pixbuf_format_is_writable ((GdkPixbufFormat*)iter->data))
		{
			type=gdk_pixbuf_format_get_name((GdkPixbufFormat*)iter->data);
			if (!strcasecmp(type,buf)) 
			{
				ok=true;
				break;
			}
			else
				g_free(type);
		}
		iter=iter->next;
	}
	
	if (!ok)
	{
		g_slist_free(formats);
		if (!strcasecmp("jpg",buf))
			type="jpeg";
		else
			return -1;
	}
	
	b=gdk_pixbuf_save (image, path,type, NULL, NULL);
	
	
	if (ok) {
		g_free(type);
		g_slist_free(formats);
	}
	
	if (!b) return -2;
	return 0;
}

#ifndef GTK_2_4
gImage* gImage::flip()
{
	gImage *ret;
	
	if (!image) return 0;
	ret=new gImage(0,0);
	ret->image=gdk_pixbuf_flip(image,true);
	return ret;
	
}

gImage* gImage::mirror()
{
	gImage *ret;
	
	if (!image) return 0;
	ret=new gImage(0,0);
	ret->image=gdk_pixbuf_flip(image,false);
	return ret;
}
#else
gImage* gImage::flip()
{
	gImage *ret=NULL;
	GdkPixbuf *dest;
	guchar *p, *q;
	gint bx, by;
	
	if (!image) return 0;
	ret=new gImage(0,0);

	ret->image=gdk_pixbuf_new(gdk_pixbuf_get_colorspace(image),\ 
	                          gdk_pixbuf_get_has_alpha(image),\
	                          gdk_pixbuf_get_bits_per_sample(image),\ 
	                          width(),height());
  
	if (!ret->image) return ret;

  
	for (by=0;by<ret->height();by++)
		for (bx=0;bx<ret->width();bx++)
		{
			p = gdk_pixbuf_get_pixels(image) +\
			                         (bx * gdk_pixbuf_get_n_channels(image)\
			                          + by * gdk_pixbuf_get_rowstride(image));
			
			
			q = gdk_pixbuf_get_pixels(ret->image) +\
			                         ((ret->width()-bx-1) * gdk_pixbuf_get_n_channels(ret->image)\
			                          + by * gdk_pixbuf_get_rowstride(ret->image));
			
			
			memcpy (q, p, gdk_pixbuf_get_n_channels(ret->image));
		}
    

  return ret;
	
}

gImage* gImage::mirror()
{

	gImage *ret=NULL;
	GdkPixbuf *dest;
	guchar *p, *q;
	gint by;
	
	if (!image) return 0;
	ret=new gImage(0,0);

	ret->image=gdk_pixbuf_new(gdk_pixbuf_get_colorspace(image),\ 
	                          gdk_pixbuf_get_has_alpha(image),\
	                          gdk_pixbuf_get_bits_per_sample(image),\ 
	                          width(),height());
  
	if (!ret->image) return ret;

	for (by=0;by<ret->height();by++)
	{
		p = gdk_pixbuf_get_pixels(image)+(by * gdk_pixbuf_get_rowstride(image));
		q = gdk_pixbuf_get_pixels(ret->image)+((ret->height()-by-1)*gdk_pixbuf_get_rowstride(image));
		memcpy (q, p,gdk_pixbuf_get_rowstride(ret->image));
	}
    
	return ret;
  
}
#endif

gImage* gImage::rotate(double ang)
{
	double myang=ang;
	long myw=0,myh=0;
	long bx,by;
	long dx,dy;
	long offx,offy;
	double hypo;
	guchar *p, *q;
	gImage *ret=NULL;
	
	if (!image) return 0;
	
	if (myang<0) myang*=(-1);
	while (myang>90) myang-=90;
	myang=myang*5.1415927/360;
	
	myw=width()*cos(myang)+height()*sin(myang);
	myh=height()*cos(myang)+width()*sin(myang);

	ret=new gImage(myw,myh);
	gdk_pixbuf_fill(ret->image,0);
		
	while (ang<0) ang+=360;
	while (ang>360) ang-=360;
	ang=ang*6.2891854/360;
	
	hypo=sqrt(width()*width()+height()*height());
	offx=height()*sin(ang);
	offy=0;
	
	for (by=0;by<height();by++)
		for (bx=0;bx<width();bx++)
		{
			hypo=sqrt(bx*bx+by*by);
			myang=acos(bx/hypo);
			myang+=ang;
			dy=hypo*sin(myang)+offy;
			dx=hypo*cos(myang)+offx;
			if (dy<0) dy=0;
			if (dx<0) dx=0;
			if (dy>=ret->height()) dy=ret->height()-1;
			if (dx>=ret->width()) dx=ret->width()-1;
			
			p = gdk_pixbuf_get_pixels(image) +\
			                         (bx * gdk_pixbuf_get_n_channels(image)\
			                          + by * gdk_pixbuf_get_rowstride(image));
			
			
			q = gdk_pixbuf_get_pixels(ret->image) +\
			                         (dx * gdk_pixbuf_get_n_channels(ret->image)\
			                          + dy * gdk_pixbuf_get_rowstride(ret->image));
			
			
			memcpy (q, p, gdk_pixbuf_get_n_channels(ret->image));
		}
	

	return ret;
	
	
}

gImage* gImage::stretch(long w,long h,bool smooth)
{
	gImage *buf;

	if (w<0) w=h;
	if (h<0) h=w;
	if ( (w<=0) || (h<=0) ) return NULL;	
	
	buf=new gImage(0,0);
	
	if (!smooth)
		buf->image=gdk_pixbuf_scale_simple(image,w,h,GDK_INTERP_NEAREST);
	else
		buf->image=gdk_pixbuf_scale_simple(image,w,h,GDK_INTERP_HYPER);
		
	return buf;
	
}

void  gImage::resize(long w,long h)
{
	GdkPixbuf*  buf;
	
	if (w<0) w=h;
	if (h<0) h=w;
	if ( (w<=0) || (h<=0) ) return;
	
	if ( (w==width()) && (h==height()) ) return; 

	if (image)
	{
		if ( (w>width()) || (h>height()) )
		{
			buf=gdk_pixbuf_new(GDK_COLORSPACE_RGB,TRUE,8,w,h);
			if (w>width()) w=width();
			if (h>height()) h=height();
			gdk_pixbuf_copy_area(image,0,0,w,h,buf,0,0);
		}
		else
		{
			buf=gdk_pixbuf_new_subpixbuf(image,0,0,w,h);
		}
		g_object_unref(G_OBJECT(image));
		image=buf;
	}
}

/*****************************************************************

gImageCache

******************************************************************/
GdkPixbuf **buf_img_cache=NULL;
char **buf_txt_cache=NULL;
long buf_img_count=0;

void gImageCache::save(char *key,gImage *img)
{
	long b;
	long pos=-1;
	
	if (!key) return;

	for (b=0;b<buf_img_count;b++)
	{
		if (!strcmp(buf_txt_cache[b],key)) { pos=b; break; }
	}
	
	if (pos==-1)
	{
	
	  if (!buf_img_count)
	  {
	    buf_img_cache=(GdkPixbuf**)malloc(sizeof(GdkPixbuf*)*(buf_img_count+1));
		buf_txt_cache=(char**)malloc(sizeof(char*)*(buf_img_count+1));
	  }
	  else
	  {
	    buf_img_cache=(GdkPixbuf**)realloc((void*)buf_img_cache,sizeof(GdkPixbuf*)*(buf_img_count+1));
	    buf_txt_cache=(char**)realloc((void*)buf_txt_cache,sizeof(char*)*(buf_img_count+1));
	  }
	  pos=buf_img_count;
	  buf_img_cache[pos]=NULL;
	  buf_txt_cache[pos]=(char*)malloc(sizeof(char)*(strlen(key)+1));
	  strcpy(buf_txt_cache[pos],key);
	}
	
	buf_img_count++;
	
	if (buf_img_cache[pos]) { 
		g_object_unref(G_OBJECT(buf_img_cache[pos])); 
		buf_img_cache[pos]=NULL;
	}
	
	if (!img) return;
	if (!img->image) return;
	buf_img_cache[pos]=gdk_pixbuf_new_subpixbuf(img->image,0,0,img->width(),img->height());
	
	
}

gImage* gImageCache::load(char *key)
{
	long b;
	long w,h;
	long pos=-1;
	gImage *ret;
	
	for (b=0;b<buf_img_count;b++)
	{
		if (!strcmp(buf_txt_cache[b],key)) { pos=b; break; }
	}
	
	if (pos==-1) return NULL;
	
	ret=new gImage(0,0);
	if (buf_img_cache[pos])
	{
		w=gdk_pixbuf_get_width(buf_img_cache[pos]);
		h=gdk_pixbuf_get_height(buf_img_cache[pos]);
		ret->image=gdk_pixbuf_new_subpixbuf(buf_img_cache[pos],0,0,w,h);
		
	}
	
	return ret;
}

void gImageCache::flush()
{
	long b;
	
	if (!buf_img_count) return;
	
	for (b=0; b<buf_img_count;b++)
	{
		free( buf_txt_cache[b] );
		if (buf_img_cache[b]) g_object_unref(G_OBJECT(buf_img_cache[b]));
	}
	
	free((void*)buf_txt_cache);
	free((void*)buf_img_cache);
	buf_img_count=0;
	
	
}
	
/*****************************************************************

gPictureCache

******************************************************************/
GdkDrawable **pic_img_cache=NULL;
char ***pic_mask_cache=NULL;
char **pic_txt_cache=NULL;
long pic_img_count=0;

void gPictureCache::save(char *key,gPicture *img)
{
	long b,b2;
	long pos=-1;
	gint x,y;
	
	if (!key) return;

	for (b=0;b<pic_img_count;b++)
	{
		if (!strcmp(pic_txt_cache[b],key)) { pos=b; break; }
	}
	
	if (pos==-1)
	{
	
	  if (!pic_img_count)
	  {
	    pic_img_cache=(GdkDrawable**)malloc(sizeof(GdkPixbuf*)*(pic_img_count+1));
		pic_mask_cache=(char***)malloc(sizeof(char**)*(pic_img_count+1));
		pic_txt_cache=(char**)malloc(sizeof(char*)*(pic_img_count+1));
	  }
	  else
	  {
	    pic_img_cache=(GdkDrawable**)realloc((void*)pic_img_cache,sizeof(GdkPixbuf*)*(pic_img_count+1));
		pic_mask_cache=(char***)realloc((void*)pic_mask_cache,sizeof(char**)*(pic_img_count+1));
	    pic_txt_cache=(char**)realloc((void*)pic_txt_cache,sizeof(char*)*(pic_img_count+1));
	  }
	  pos=pic_img_count;
	  pic_mask_cache[pos]=NULL;
	  pic_img_cache[pos]=NULL;
	  pic_txt_cache[pos]=(char*)malloc(sizeof(char)*(strlen(key)+1));
	  strcpy(pic_txt_cache[pos],key);
	}
	
	pic_img_count++;
	
	if (pic_img_cache[pos]) 
	{ 
		if (pic_mask_cache[pos])
		{
			gdk_drawable_get_size (pic_img_cache[pos], &x,&y);
			for (b=0;b<y;b++) free(pic_mask_cache[pos][b]);
			free(pic_mask_cache[pos]);
		}
		g_object_unref(G_OBJECT(pic_img_cache[pos])); 
		pic_img_cache[pos]=NULL;
		pic_mask_cache[pos]=NULL;
	}
	
	if (!img) return;
	if (!img->pic) return;
	
	pic_img_cache[pos]=img->pic;
	g_object_ref(G_OBJECT(img->pic));
	if (img->mask)
	{
		gdk_drawable_get_size (pic_img_cache[pos], &x,&y);
		pic_mask_cache[pos]=(char**)malloc(sizeof(char*)*y);
		for (b=0;b<y;b++)
		{
			pic_mask_cache[pos][b]=(char*)malloc(sizeof(char)*x);
			memcpy(pic_mask_cache[pos][b],img->mask[b],x);
		}
	}
	
	
}

gPicture* gPictureCache::load(char *key)
{
	long b;
	long pos=-1;
	gPicture *ret;
	
	for (b=0;b<pic_img_count;b++)
	{
		if (!strcmp(pic_txt_cache[b],key)) { pos=b; break; }
	}
	
	if (pos==-1) return NULL;
	
	ret=new gPicture(0,0,(bool)pic_mask_cache[pos]);
	if (pic_img_cache[pos])
	{
		ret->pic=pic_img_cache[pos];
		g_object_ref(G_OBJECT(ret->pic));
		
	}
	
	return ret;
}

void gPictureCache::flush()
{
	long b,b2;
	gint x,y;
	
	if (!pic_img_count) return;
	
	for (b=0; b<pic_img_count;b++)
	{
		free( pic_txt_cache[b] );
		if (pic_img_cache[b]) 
		{
			if (pic_mask_cache[b])
			{
				gdk_drawable_get_size (pic_img_cache[b], &x,&y);
				for (b2=0;b2<y;b2++) free(pic_mask_cache[b][b2]);
				free(pic_mask_cache[b]);
			}
			g_object_unref(G_OBJECT(pic_img_cache[b]));
		}
	}
	
	free((void*)pic_mask_cache);
	free((void*)pic_txt_cache);
	free((void*)pic_img_cache);
	pic_img_count=0;
	
	
}
	
