/***************************************************************************

  gsignals.cpp

  (c) 2004-2005 - Daniel Campos Fernández <danielcampos@netcourrier.com>
  
  Gtkmae "GTK+ made easy" classes
  
  Realizado para la Junta de Extremadura. 
  Consejería de Educación Ciencia y Tecnología. 
  Proyecto gnuLinEx
  
  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 1, or (at your option)
  any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

***************************************************************************/
#include <stdio.h>

#include "widgets.h"
#include "widgets_private.h"
#include <gtk/gtk.h>
#include <gdk/gdk.h>
#include <gdk/gdkkeysyms.h>
#include <string.h>


guint32 drag_time=0;




void sg_destroy (GtkWidget *object,gControl *data)
{
	if (data->onFinish) data->onFinish(data);
}

gboolean sg_configure(GtkWidget *widget,GdkEventConfigure *event,gControl *data)
{
	if ( (event->width!=data->bufW) || (event->height!=data->bufH)  )
	{
		data->bufW=event->width;
		data->bufH=event->height;
	}
	
	if ( (event->x!=data->bufX) || (event->y!=data->bufY) )
	{
		data->bufX=event->x;
		data->bufY=event->y;
	}
	
	
	return false;
}

gboolean sg_key_Press (GtkWidget *widget, GdkEventKey *event, gControl *data)
{
	bool vl=false;
	
	if (!gApplication::userEvents()) return false;
	
	gKey::enable(event);
	if (data->onKeyPress) vl=data->onKeyPress(data);
	gKey::disable();
	
	if (!vl)
	{
		if (data->getClass()==Type_gMainWindow)
		{
			if ( (event->keyval==GDK_Escape) && ( ((gMainWindow*)data)->esc_button!=NULL) ) 
				gtk_button_clicked(GTK_BUTTON(((gMainWindow*)data)->esc_button));
		}
	}
	
	return vl;
}

gboolean sg_key_Release (GtkWidget *widget, GdkEventKey *event, gControl *data)
{
	if (!gApplication::userEvents()) return false;
	
	gKey::enable(event);
	if (data->onKeyRelease) data->onKeyRelease(data);
	gKey::disable();
	return false;
}

gboolean sg_motion(GtkWidget *widget,GdkEventMotion *event,gControl *data)
{
	long bt=0;
	
	if (!(data->getClass() & 0x1000) )
		if ( !(event->state & 0x1F00) ) return true;
	

	if (data->onMouseMove)
	{
		gMouse::setValid(true,(long)event->x,(long)event->y,bt,event->state);
		data->onMouseMove(data);
		gMouse::setValid(false,0,0,0,0);
		return true;
	}
	return true;
}

gboolean sg_button_Press (GtkWidget *widget,GdkEventButton *event,gControl *data)
{
	if (!gApplication::userEvents()) return false;

	if (data->onMousePress)
	{
		gMouse::setValid(true,(long)event->x,(long)event->y,event->button,event->state);
		data->onMousePress(data);
		gMouse::setValid(false,0,0,0,0);
	}
	
	if (event->button==3) data->onMenu(data);
	return false;
}

gboolean sg_button_Release (GtkWidget *widget,GdkEventButton *event,gControl *data)
{
	if (!gApplication::userEvents()) return false;

	if (data->onMouseRelease)
	{
		gMouse::setValid(true,(long)event->x,(long)event->y,event->button,event->state);
		data->onMouseRelease(data);
		gMouse::setValid(false,0,0,0,0);
	}
	
	return false;
}

gboolean sg_focus_In(GtkWidget *widget,GdkEventFocus *event,gControl *data)
{
	if (!gApplication::allEvents()) return false;

	if (data->onFocusEnter) data->onFocusEnter(data);
	return false;
}

gboolean sg_focus_Out(GtkWidget *widget,GdkEventFocus *event,gControl *data)
{
	if (!gApplication::allEvents()) return false;
	
	if (data->onFocusLeave) data->onFocusLeave(data);
	return false;
}

gboolean sg_event(GtkWidget *widget, GdkEvent *event,gControl *data)
{
	if (!gApplication::userEvents()) return false;
	
	if (event->type==GDK_2BUTTON_PRESS)
	{
		if (data->onDoubleClick) data->onDoubleClick(data);
		return false;
	}

	
	return false;
}


void sg_drag_rec(GtkWidget *widget,GdkDragContext *context,gint x,gint y, \
                 GtkSelectionData *sel,guint info,guint time,gControl *data)
{

	if (!gApplication::allEvents()) return;

	if (data->drops & 4)
	{
		if (drag_Type()==1)
		{
			if (sel->length != -1)
				gDrag_setText((char*)sel->data);
			else
				gDrag_setText(NULL);
		}
		
		if (drag_Type()==2)
		{
			if (sel->length != -1)
				gDrag_setImage((char*)sel->data,sel->length);
			else
				gDrag_setImage(NULL,0);
		}
		
		data->drops |= 8;
	}
}

gboolean sg_drag(GtkWidget *widget,GdkDragContext *context,gint x,gint y,guint time,gControl *data)
{
	GList *tg;
	gchar *buf;
	bool retval=true;

	if (!gApplication::allEvents()) return true;
	
	if (drag_time != context->start_time) { gDrag_Clear(); data->drops=1; }
	
	if (!(data->drops & 4))
	{
		data->drops |= 4;
		context->start_time=time;
		drag_time=time;
		gDrag_setTarget(0,NULL);
		tg=g_list_first(context->targets);
		while (tg)
		{
			buf=gdk_atom_name((GdkAtom)tg->data);
			if (strlen(buf)>=4)
				if ( (!strncmp(buf,"text",4)) || (!strncmp(buf,"TEXT",4)) ) 
				{ 
					gtk_drag_get_data (widget,context,(GdkAtom)tg->data,time);
					g_free(buf);
					tg=g_list_first(tg);
					buf=gdk_atom_name((GdkAtom)tg->data);
					gDrag_setTarget(1,buf); 
					g_free(buf); 
					break; 
				}
				
			if (strlen(buf)>=5)
				if (!strncmp(buf,"image",5)) 
				{ 
					gtk_drag_get_data (widget,context,(GdkAtom)tg->data,time);
					g_free(buf);
					tg=g_list_first(tg);
					buf=gdk_atom_name((GdkAtom)tg->data);
					gDrag_setTarget(2,buf); 
					g_free(buf); 
					break; 
				}
			
			g_free(buf);
			tg=g_list_next(tg);
		}

		return false;
	}
	
	//if (!(data->drops & 8)) { gdk_drag_status(context,context->suggested_action,time); return false; }
	
	if (!(data->drops & 2)) { x=0; y=0; }

	switch (context->suggested_action)
	{
		case GDK_ACTION_COPY: gDrag_Enable(x,y,0); break;
  		case GDK_ACTION_MOVE: gDrag_Enable(x,y,1); break;
  		case GDK_ACTION_LINK: gDrag_Enable(x,y,2); break;
		default: gDrag_Enable(x,y,0); break;
	}

	
	if (!(data->drops & 2))
	{
		if (data->onDrag) data->onDrag(data);
		data->drops |= 2;
	}
	else
		if (data->onDragMove) retval=!data->onDragMove(data);
	
	
	gDrag_Disable();
	if (retval) {
		gdk_drag_status(context,context->suggested_action,time);
		return true;
	}
	
	return false;
}

/*
void sg_drag_lv(GtkWidget *widget,GdkDragContext *context,guint time,gControl *data)
{
	
	data->drops=1;
	drag_time=0;
}
*/


gboolean sg_drop(GtkWidget *widget,GdkDragContext *context,gint x,gint y,guint time,gControl *data)
{
	if (!gApplication::allEvents()) return true;
	
	gDrag_Enable(x,y,drag_Action());
	if (data->onDrop) data->onDrop(data);
	gDrag_Disable();
	gDrag_Clear();
	gtk_drag_finish (context,true,false,time);
	data->drops=1;
	drag_time=0;
	return true;
}

void gControl::widgetSignals()
{
	
	g_signal_connect_after(G_OBJECT(widget),"motion-notify-event",G_CALLBACK(sg_motion),(gpointer)this);
	g_signal_connect(G_OBJECT(widget),"button-release-event",G_CALLBACK(sg_button_Release),(gpointer)this);
	g_signal_connect(G_OBJECT(widget),"button-press-event",G_CALLBACK(sg_button_Press),(gpointer)this);
	g_signal_connect(G_OBJECT(widget),"focus-in-event",G_CALLBACK(sg_focus_In),(gpointer)this);
	g_signal_connect(G_OBJECT(widget),"focus-out-event",G_CALLBACK(sg_focus_Out),(gpointer)this);
	g_signal_connect(G_OBJECT(widget),"event",G_CALLBACK(sg_event),(gpointer)this);
	
}

void gControl::initSignals()
{
	GtkRequisition req;

	onFinish=NULL;
	onFocusEnter=NULL;
	onFocusLeave=NULL;
	onKeyRelease=NULL;
	onKeyPress=NULL;
	onMenu=NULL;
	onMouseRelease=NULL;
	onMousePress=NULL;
	onMouseMove=NULL;
	onDoubleClick=NULL;
	onDrag=NULL;
	onDragMove=NULL;
	onDrop=NULL;

	resize(100,30);
	
	if (pr) g_signal_connect(G_OBJECT(border),"configure-event",G_CALLBACK(sg_configure),(gpointer)this);
	g_signal_connect(G_OBJECT(border),"destroy",G_CALLBACK(sg_destroy),(gpointer)this);
	g_signal_connect(G_OBJECT(border),"key-press-event",G_CALLBACK(sg_key_Press),(gpointer)this);
	g_signal_connect(G_OBJECT(border),"key-release-event",G_CALLBACK(sg_key_Release),(gpointer)this);
	g_signal_connect(G_OBJECT(border),"drag-data-received",G_CALLBACK(sg_drag_rec),(gpointer)this);
	g_signal_connect(G_OBJECT(border),"drag-motion",G_CALLBACK(sg_drag),(gpointer)this);
	g_signal_connect(G_OBJECT(border),"drag-drop",G_CALLBACK(sg_drop),(gpointer)this);
	
	widgetSignals();



}
