/***************************************************************************

  gtrayicon.cpp

  (c) 2004-2005 - Daniel Campos Fernández <danielcampos@netcourrier.com>
  
  Gtkmae "GTK+ made easy" classes
  
  Realizado para la Junta de Extremadura. 
  Consejería de Educación Ciencia y Tecnología. 
  Proyecto gnuLinEx
  
  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 1, or (at your option)
  any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

***************************************************************************/
#include "widgets.h"
#include <gtk/gtk.h>
#include <gdk/gdkx.h>
#include <X11/Xlib.h>
#include <X11/Xatom.h>
#include <string.h>
#include <stdio.h>
#include <stdlib.h>

#define SYSTEM_TRAY_REQUEST_DOCK    0
#define SYSTEM_TRAY_BEGIN_MESSAGE   1
#define SYSTEM_TRAY_CANCEL_MESSAGE  2

#define OPCODE "_NET_SYSTEM_TRAY_OPCODE"

/*****************************************************************************

Low level stuff

******************************************************************************/

GdkFilterReturn gTrayIcon_managerFilter (GdkXEvent *xevent, GdkEvent *event, gpointer data);

void gTrayIcon_updateManager (gTrayIcon *icon)
{
	XClientMessageEvent ev;
	Display *display;
	GdkWindow *gdkwin;
	long d1;
		
	if (icon->manager_window != None)
	{
		gdkwin = gdk_window_lookup (icon->manager_window);
		gdk_window_remove_filter (gdkwin, gTrayIcon_managerFilter,icon);
	}
  
	XGrabServer (icon->xdisplay);

	icon->manager_window = XGetSelectionOwner (icon->xdisplay,icon->selection_atom);
	if (icon->manager_window != None)
		XSelectInput (icon->xdisplay,icon->manager_window, StructureNotifyMask);
		
	XUngrabServer (icon->xdisplay);
	XFlush (icon->xdisplay);
  
	if (icon->manager_window != None)
	{
		d1=gtk_plug_get_id(GTK_PLUG(icon->plug));
		gdkwin = gdk_window_lookup (icon->manager_window);
		gdk_window_add_filter (gdkwin, gTrayIcon_managerFilter, icon);
		
		ev.type = ClientMessage;
		ev.window = icon->manager_window;
		ev.message_type = icon->system_tray_opcode_atom;
		ev.format = 32;
		ev.data.l[0] = gdk_x11_get_server_time (icon->plug->window);
		ev.data.l[1] = SYSTEM_TRAY_REQUEST_DOCK;
		ev.data.l[2] = d1;
		ev.data.l[3] = 0;
		ev.data.l[4] = 0;
	
		display = gdk_display;
	
		gdk_error_trap_push ();
		XSendEvent (display,icon->manager_window, 0, NoEventMask, (XEvent *)&ev);
		XSync (display, 0);
		gdk_error_trap_pop ();
		
	}
}

GdkFilterReturn gTrayIcon_managerFilter (GdkXEvent *xevent, GdkEvent *event, gpointer data)
{
	gTrayIcon *icon = (gTrayIcon*)data;
	XEvent *xev = (XEvent *)xevent;
	
	if (xev->xany.type==ClientMessage && xev->xclient.message_type == icon->manager_atom &&  xev->xclient.data.l[1] == icon->selection_atom)
	{
		gTrayIcon_updateManager (icon);
	}
	else if (xev->xany.window == icon->manager_window)
	{
		if (xev->xany.type == DestroyNotify)
		{
			gTrayIcon_updateManager (icon);
		}
		
	}
	
	return GDK_FILTER_CONTINUE;
}

/*************************************************************************

gTrayIcon

**************************************************************************/



void tray_end (GtkWidget *object,gTrayIcon *data)
{
	if (data->onDestroy) data->onDestroy(data);	
	delete data;
}


gboolean tray_down (GtkWidget *widget,GdkEventButton *event,gTrayIcon *data)
{
	
	if (!gApplication::userEvents()) return false;

	if (data->onMousePress)
	{
		gMouse::setValid(true,(long)event->x,(long)event->y,event->button,event->state);
		data->onMousePress(data);
		gMouse::setValid(false,0,0,0,0);
	}
	
	if (event->button==3) 
		if (data->onMenu) 
			data->onMenu(data);
	
	return false;
}

gboolean tray_up (GtkWidget *widget,GdkEventButton *event,gTrayIcon *data)
{
	if (!gApplication::userEvents()) return false;

	if (data->onMouseRelease)
	{
		gMouse::setValid(true,(long)event->x,(long)event->y,event->button,event->state);
		data->onMouseRelease(data);
		gMouse::setValid(false,0,0,0,0);
	}
	
	return false;
}


void gTrayIcon::setPicture(gPicture *pic)
{
	GdkPixbuf *buf;

	if (!pic) return;
	if (!pic->pic) return;
	
	buf=pic->getPixbuf();
	gtk_image_set_from_pixbuf(GTK_IMAGE(img),buf);
	g_object_unref(G_OBJECT(buf));

}

gPicture* gTrayIcon::picture()
{
	gPicture *pic=NULL;
	GdkPixbuf *buf;

	buf=gtk_image_get_pixbuf(GTK_IMAGE(img));
	pic=gPicture::fromPixbuf(buf);
	return pic;

}

char* gTrayIcon::toolTip()
{
	GtkTooltipsData* data;
	
	if (!tip) return NULL;
	data=gtk_tooltips_data_get(plug);
	if (!data) return NULL;
	return data->tip_text;
	
}

void gTrayIcon::setToolTip(char* vl)
{	
	if (!vl) vl="";
	if (!tip) tip=gtk_tooltips_new();
	
	gtk_tooltips_set_tip(tip,plug,vl,NULL);
	if (strlen(vl))
		gtk_tooltips_enable(tip);
	else
		gtk_tooltips_disable(tip);

	
}

void gTrayIcon::destroy()
{
	gtk_widget_destroy(plug);
}

gTrayIcon::gTrayIcon(char *s_tip,gPicture *pic)
{
	GtkWidget *ebox;
	GdkPixbuf *pbuf=NULL;
	char buf[256];
	long bucle;
		
	onMousePress=NULL;
	onMouseRelease=NULL;
	onMenu=NULL;
	
	buf[0]=0;
	
	tip=NULL;

	if (pic)
		if (pic->pic)
			pbuf=pic->getPixbuf();
	
	img=gtk_image_new_from_pixbuf(pbuf);
	if (pbuf) g_object_unref(G_OBJECT(pbuf));
	
	plug=gtk_plug_new(0);
	ebox=gtk_event_box_new();
	gtk_container_add(GTK_CONTAINER(plug),ebox);
	gtk_container_add(GTK_CONTAINER(ebox),img);
	gtk_widget_show_all(plug);
	
	g_signal_connect(G_OBJECT(plug),"destroy",G_CALLBACK(tray_end),(gpointer)this);
	g_signal_connect(G_OBJECT(ebox),"button-release-event",G_CALLBACK(tray_up),(gpointer)this);
	g_signal_connect(G_OBJECT(ebox),"button-press-event",G_CALLBACK(tray_down),(gpointer)this);
	
	manager_window=None;
	xdisplay=gdk_display;
	xscreen=DefaultScreenOfDisplay(gdk_display);
	sprintf(buf,"_NET_SYSTEM_TRAY_S%d",XScreenNumberOfScreen (xscreen));
	selection_atom = XInternAtom (DisplayOfScreen (xscreen),buf,0);
	manager_atom = XInternAtom (DisplayOfScreen (xscreen),"MANAGER",0);
	system_tray_opcode_atom=XInternAtom(DisplayOfScreen(xscreen),OPCODE,0);
	gTrayIcon_updateManager(this);
	
	if (s_tip) setToolTip(s_tip);
	if (pic) setPicture(pic);
}
