/***************************************************************************

  widgets.h

  (c) 2004-2005 - Daniel Campos Fernández <danielcampos@netcourrier.com>
  
  Gtkmae "GTK+ made easy" classes
  
  Realizado para la Junta de Extremadura. 
  Consejería de Educación Ciencia y Tecnología. 
  Proyecto gnuLinEx
  
  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 1, or (at your option)
  any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

***************************************************************************/

#include <gtk/gtk.h>
#include <gdk/gdkx.h>
#ifndef __widgets_h
#define __widgets_h

#define AlignVCenter 0x0040
#define AlignAuto    0x0000
#define AlignTop     0x0010
#define AlignBottom  0x0020
#define AlignHCenter 0x0004
#define AlignLeft    0x0001
#define AlignRight   0x0002

#define alignBottom        0x0024
#define alignBottomLeft    0x0021
#define alignBottomNormal  0x0020
#define alignBottomRight   0x0022
#define alignCenter        0x0044
#define alignLeft          0x0041
#define alignNormal        0x0040
#define alignRight         0x0042
#define alignTop           0x0014
#define alignTopLeft       0x0011
#define alignTopNormal     0x0010
#define alignTopRight      0x0012

#define FillNoBrush            0
#define FillSolidPattern       1
#define FillDense1Pattern      2
#define FillDense2Pattern      3
#define FillDense3Pattern      4
#define FillDense4Pattern      5
#define FillDense5Pattern      6
#define FillDense6Pattern      7
#define FillDense7Pattern      8
#define FillHorPattern         9
#define FillVerPattern        10
#define FillCrossPattern      11
#define FillBDiagPattern      12
#define FillFDiagPattern      13
#define FillDiagCrossPattern  14

#define Line_None         0
#define Line_Solid        1
#define Line_Dash         2
#define Line_Dot          3
#define Line_DashDot      4
#define Line_DashDotDot   5

#define BORDER_NONE   0
#define BORDER_PLAIN  1
#define BORDER_SUNKEN 2
#define BORDER_RAISED 3
#define BORDER_ETCHED 4

#define ARRANGE_NONE        0
#define ARRANGE_HORIZONTAL  1
#define ARRANGE_VERTICAL    2
#define ARRANGE_LEFT_RIGHT  3
#define ARRANGE_TOP_BOTTOM  4
#define ARRANGE_FILL        5

#define Type_gLabel       0x0001
#define Type_gTextLabel   0x0002
#define Type_gButton      0x0003
#define Type_gFrame       0x0105
#define Type_gMainWindow  0x0106
#define Type_gPictureBox  0x0007
#define Type_gPanel       0x0108
#define Type_gDrawingArea 0x0109
#define Type_gProgressBar 0x000A
#define Type_gSlider      0x000E 
#define Type_gScrollBar   0x000F
#define Type_gTabStrip    0x0011
#define Type_gPlugin      0x0013
#define Type_gMovieBox    0x0014
#define Type_gScrollView  0x0115
#define Type_gSpinBox     0x0016
#define Type_gTextBox     0x1004
#define Type_gTextArea    0x100B
#define Type_gComboBox    0x100C
#define Type_gListBox     0x100D
#define Type_gListView    0x1010
#define Type_gColumnView  0x1012
#define Type_gEmbedWindow 0x0113

#if GTK_MAJOR_VERSION == 2
#if GTK_MINOR_VERSION > 6
#define GTK_2_8
#endif
#endif


void stub(char *function);

class gImage;
class gPicture;
class gControl;
class gMainWindow;
class gFont;
class gDrawingArea;

int clipBoard_Type();
char* clipBoard_Format();
void clipBoard_Clear();
char* clipBoard_getText();
gImage* clipBoard_getImage();
void clipBoard_setText(char *text,char *format);
void clipBoard_setImage(gImage *image);

bool drag_IsEnabled();
int drag_Action();
int drag_Type();
char* drag_Format();
char* drag_Text();
gImage* drag_Image();
long drag_X();
long drag_Y();

class gTimer
{
public:
	gTimer();
	~gTimer();
	
	long delay();
	bool enabled(); 
	
	void setDelay(long vl);
	void setEnabled(bool vl);
	
	void (*onTimer)(void *data);
	void* data;
	
//"Private"
	long timer;
	long interval;
	bool val;

		
};

class gKey
{
public:
	static bool valid();
    static char *text();
    static int code();
    static int state();
	
	static bool alt();
	static bool control();
	static bool meta();
	static bool normal();
	static bool shift();
	
	static int fromString(char* str);
	
//"Private"
	static void disable();
	static void enable(GdkEventKey *e);
};


class gCursor
{
public:
	gCursor(gPicture *pic,long x,long y);
	gCursor(gCursor *cursor);
	~gCursor();
	
//"Properties"
	long left();
	long top();
	
//"Private"
	GdkCursor *cur;
	long x;
	long y;
};


class gMouse
{
public:

//"Properties"
	static long button();
	static bool left();
	static bool right();
	static bool middle();
	static bool shift();
	static bool control();
	static bool alt();
	static bool meta();
	static bool normal();
	static long x();
	static long y();
	static bool valid();

//"Methods"
	static void move(long x,long y);

//"Private"
	static void setValid(bool vl,long xval,long yval,long bt_val,long state_val);
};

class gDesktop
{
public:
	static long height();
	static long width();
	static long resolution();
	static gPicture* grab();
};

#ifdef GTK_2_8
class gVectorial
{
public:
	gVectorial();
	~gVectorial();
	
	void connect(gControl *wid);
	void connect(gPicture *wid);
	void disconnect();
	
//"Private"
	cairo_t *cr;
};
#endif

class gDraw
{
public:
	gDraw();
	~gDraw();
	void connect(gControl *wid);
	void connect(gPicture *wid);
	#ifdef GTK_2_8
	void connectVectorial(gControl *wid);
	void connectVectorial(gPicture *wid);
	#endif
	void disconnect();
	
//"Properties"
	long foreGround();
	gFont* font();
	long backGround();
	long fillColor();
	long lineWidth();
	long lineStyle();
	long fillX();
	long fillY();
	long fillStyle();
	bool invert();
	long clipX();
	long clipY();
	long clipWidth();
	long clipHeight();
	bool clipEnabled();
	long textWidth(char *txt);
	long textHeight(char *txt);
	
	void setForeGround(long vl);
	void setFont(gFont *f);
	void setBackGround(long vl);
	void setFillColor(long vl);
	void setLineWidth(long vl);
	void setLineStyle(long vl);
	void setFillX(long vl);
	void setFillY(long vl);
	void setFillStyle(long vl);
	void setInvert(bool vl);
	void setClipEnabled(bool vl);

//"Methods"
	#ifdef GTK_2_8
	void scale(double x,double y);
	void rotate(double angle);
	#endif
	void point(long x,long y);
	void line(long x1,long y1,long x2,long y2);
	void rect(long x,long y,long width,long height);
	void ellipse(long x,long y,long w,long h,long start,long end);
	void polyline (long *vl,long nvl);
	void polygon (long *vl,long nvl);
	void startClip(long x,long y,long w,long h);
	void text(char *txt,long x,long y,long w,long h,long align);
	
	void image(gImage *img,long x,long y,long Sx,long Sy,long Sw,long Sh);
	void picture(gPicture *pic,long x,long y,long Sx,long Sy,long Sw,long Sh);
	
//"Private"
	void pClear();
	gDrawingArea *dArea;
	gFont *ft;
	GdkRectangle clip;
	bool clip_enabled;
	GdkDrawable *dr;
	GdkPixmap *stipple;
	GdkGC *gc;
	int fill;
	long fillCol;
	long line_style;
	#ifdef GTK_2_8
	cairo_t *cr;
	#endif
};


class gImage
{
public:
	gImage(long w,long h);
	~gImage();
	
	long width();
	long height();
	gPicture* getPicture();
	
	void fromMemory(char *addr,unsigned int len);
	int save(char *path);
	gImage* flip();
	gImage* mirror();
	gImage* rotate(double ang);
	void resize(long w,long h);
	gImage* stretch(long w,long h,bool smooth);
//"Private"
	GdkPixbuf *image;
};


class gPicture
{
public:
	gPicture(long w,long h,bool trans);
	~gPicture();
	
	long width();
	long height();
	long depth();
	bool transparent();
	
	void setTransparent(bool vl);
	
	void resize(int width,int height);
	int  save(char *path);
	void fromMemory(char *addr,unsigned int len);
	static gPicture* fromStock(char *name);
	static gPicture* fromNamedIcon(char* name);
	void Fill(long col);
	gImage* getImage();

//"Private"
	GdkDrawable *pic;
    char **mask;
    int _transparent;
	GdkPixbuf* gPicture::getPixbuf();
	void freeMask();
	void setMask(GdkPixbuf *pixbuf);
	static gPicture* fromPixbuf(GdkPixbuf *buf);
};

class gImageCache
{
public:
	static void save(char *key,gImage *img);
	static gImage* load(char *key);
	static void flush();
};

class gPictureCache
{
public:
	static void save(char *key,gPicture *img);
	static gPicture* load(char *key);
	static void flush();
};

class gFont
{
public:
	gFont();
	gFont(char *name);
	~gFont();
	
	long ascent();
	long descent();
	bool fixed();
	char **styles();
	
	bool bold();
	bool italic();
	const char* name();
	long resolution();
	double size();
	bool strikeOut();
	bool underline();
	
	void setBold(bool vl);
	void setItalic(bool vl);
	void setName(char *nm);
	void setResolution(long vl);
	void setSize(double sz);
	void setStrikeOut(bool vl);
	void setUnderline(bool vl);
	
	const char * toString();
	long width(const char *text);
	long height(const char *text);
	
	void ref();
	void unref();

//"Private"
	gFont(GtkWidget *wg);
	void updateWidget();
	GtkWidget *wid;
	PangoContext* ct;
	bool uline;
	bool strike;
	long nref;
};


class gMessage
{
public:
	static int showDelete(char *msg,char *btn1,char *btn2,char *btn3);
	static int showError(char *msg,char *btn1,char *btn2,char *btn3);
	static int showInfo(char *msg,char *btn1);
	static int showQuestion(char *msg,char *btn1,char *btn2,char *btn3);
	static int showWarning(char *msg,char *btn1,char *btn2,char *btn3);
	
	static void setTitle(char *title);
};

class gDialog
{
public:
	static long color();
	static char** filter(long *nfilter);
	static char* path();
	static char* title();
	static gFont* font();
	
	static void setColor(long col);
	static void setFilter(char **filter,long nfilter);
	static void setPath(char *vl);
	static void setTitle(char *title);
	static void setFont(gFont *ft);
	
	static bool selectColor();
	static bool selectFolder();
	static bool selectFont();
	static bool openFile();
	static bool saveFile();
};

class gControl
{
public:
	gControl();
	gControl(gControl* parent);
	~gControl();
	
	void *hFree;

// "Properties"
	int getClass();
	virtual long backGround();
	gCursor* cursor();
	bool design();
	bool enabled();
	bool expand();
	virtual gFont* font();
	virtual long foreGround();
	long handle();
	virtual long height();
	long left();
	int mouse();
	gControl *next();
	gControl *parent();
	gControl *previous();
	long screenX();
	long screenY();
	char *toolTip();
	long top();
	virtual long width();
	virtual bool visible();
	bool acceptDrops();
	gControl *window();
	
	void setCursor(gCursor *vl);
	void setAcceptDrops(bool vl);
	void setDesign(bool vl);
	void setEnabled(bool vl);
	virtual void setBackGround(long color);
	void setExpand (bool vl);
	virtual void setFont(gFont *ft);
	virtual void setForeGround(long color);
	virtual void setHeight(long h);
	void setLeft(long l);
	void setMouse(int m);
	void setToolTip(char *vl);
	void setTop(long t);
	virtual void setVisible(bool v);
	virtual void setWidth(long w);

// "Methods"
	void hide();
	void lower();
	virtual void move(long x,long y);
	void raise();
	virtual void resize(long w,long h);
	void setFocus();
	virtual void show();
	void refresh(long x,long y,long w,long h);
	gPicture *grab();
	void destroy();
	
// "Signals"
	void (*onFinish)(gControl *sender); // Special 
	void (*onFocusEnter)(gControl *sender);
	void (*onFocusLeave)(gControl *sender);
	void (*onKeyRelease)(gControl *sender);
	bool (*onKeyPress)(gControl *sender);
	void (*onMouseRelease)(gControl *sender);
	void (*onMousePress)(gControl *sender);
	void (*onMouseMove)(gControl *sender);
	void (*onMenu)(gControl *sender);
	void (*onDoubleClick)(gControl *sender);
	void (*onDrag)(gControl *sender);
	bool (*onDragMove)(gControl *sender);
	void (*onDrop)(gControl *sender);
	
// "Private"
	gint bufW,bufH,bufX,bufY;
	gControl *pr;
	gCursor *curs;
	GtkWidget *widget;
	GtkWidget *border;
	GtkTooltips *tip;
	int g_typ;
	int mous;
	int drops;
	bool dsg;
	bool expa;
	void initSignals();
	void widgetSignals();
	void connectParent();
	void initAll(gControl *pr);
};

class gTrayIcon 
{
public:
//"Properties"
	gTrayIcon(char *s_tip,gPicture *pic);
	
	void *hFree;
	
	char *key();
	gPicture* picture();
	void setPicture(gPicture *pic);
	char* toolTip();
	void setToolTip(char *txt);
//"Methods"
	void destroy();
	
//"Events"
	void (*onMousePress)(gTrayIcon *sender);
	void (*onMouseRelease)(gTrayIcon *sender);
	void (*onMenu)(gTrayIcon *sender);
	void (*onDestroy)(gTrayIcon *sender);
	
//"Private"
	GtkWidget *plug;
	GtkWidget *img;
	GtkTooltips *tip;
	
	Screen *xscreen;
	Display *xdisplay;
	Atom selection_atom;
	Atom manager_atom;
	Atom system_tray_opcode_atom;
	Window manager_window;
};

class gPlugin : public gControl
{
public:
	gPlugin(gControl *parent);
	void plug(long id);
//"Properties"
	int getBorder();
	long foreGround();
	long backGround();
	
	void setBorder(int vl);
	void setForeGround(long vl);
	void setBackGround(long vl);
//"Signals"
	void (*onPlug)(gControl *sender);
	void (*onUnplug)(gControl *sender);
	
};


class gProgressBar : public gControl
{
public:
	gProgressBar(gControl *parent);
	
	bool label();
	double value();
	
	void setLabel(bool vl);
	void setValue(double vl);
	
	void reset();

//"Private"
	bool lbl;
};

class gLabel : public gControl
{
public:
	gLabel(gControl *parent);
	
	int alignment();
	int getBorder();
	const char* text();
	
	void setAlignment(int al);
	void setBorder(int vl);
	void setText(const char *st);
	
//"Private"
	void fontModified();
};

class gTextLabel : public gLabel
{
public:
	gTextLabel(gControl *parent);
	
	const char* text();
	void setText(const char *st);
};

class gButton : public gControl
{
public:
	gButton(gControl *parent,int type);
	
	long backGround();
	bool _border();
	bool isCancel();
	bool isDefault();
	gFont* font();
	long foreGround();
	const char* text();
	gPicture* picture();
	bool value();
	bool getToggle();
	
	void setBackGround(long vl);
	void setBorder(bool vl);
	void setCancel(bool vl);
	void setDefault(bool vl);
	void setFont(gFont *ft);
	void setForeGround(long vl);
	void setText(const char *st);
	void setPicture(gPicture *pic);
	void setValue(bool vl);
	void setToggle(bool vl);

//"Method"
	void resize(long w,long h);
//"Signals"
	void (*onClick)(gControl *sender);
	
//"Private"
	int type;
	bool disable;
	bool isToggle;
	char *bufText;
	bool scaled;
	void performScale();
	
	
};

class gMovieBox : public gControl
{
public:
	gMovieBox(gControl *parent);

//"Properties"
	long foreGround();
	long backGround();
	int getBorder();
	bool playing();
	
	void setForeGround(long vl);
	void setBackGround(long vl);
	void setBorder(int vl);
	void setPlaying(bool vl);
	
//"Methods"
	bool loadMovie(char *buf,long len);
	
//"Private"
	bool pl;
	guint timeout;
	GdkPixbufAnimation *animation;
	GdkPixbufAnimationIter *iter;
		

};

class gPictureBox : public gControl
{
public:
	gPictureBox(gControl *parent); 
	
	long foreGround();
	long backGround();
	int alignment();
	int getBorder();
	bool stretch();
	gPicture* picture();
	
	void setForeGround(long vl);
	void setBackGround(long vl);
	void setAlignment(int vl);
	void setBorder(int vl);
	void setStretch(bool vl);
	void setPicture(gPicture *pic);

//"Private"
	bool stretched;
	GdkPixbuf *pix;
	void redraw();

};

class gColumnView : public gControl
{
public:
	gColumnView(gControl *parent);

//"Properties"
	char* columnText(long ind);
	long columnAlignment(long ind);
	long columnWidth(long ind);
	long columnsCount();
	long count();
	bool expanded(long ind);
	bool header();
	long itemCount(long ind);
	gPicture* itemPicture(long ind);
	char* itemText(long ind,long col);
	char* key(long ind);
	long keyIndex(char *key);
	bool resizable();
	long scrollBars();
	
	void setColumnAlignment(long ind,long vl);
	void setColumnText(long ind,char *vl);
	void setColumnWidth(long ind,long vl);
	void setColumnsCount(long vl);
	void setExpanded(long ind,bool vl);
	void setItemPicture(long ind,gPicture *pic);
	void setItemText(long ind,long col,char *txt);
	void setHeader(bool vl);
	void setResizable(bool vl);
	void setScrollBars(long vl);
	
//"Methods"
	int add(char *key,char *vl,gPicture *pic,char *pos,char *parent);
	void itemClear(long ind);
	void remove(long ind);
	
//"Private"
	bool sort;
	bool colResize;
	long colCount;
};

class gListView : public gControl
{
public:
	gListView(gControl *parent);

//"Properties"
	long backGround();
	long foreGround();
	long count();
	long keyIndex(char *key);	
	long index();
	char* itemKey(long ind);
	bool itemSelected(long ind);
	gPicture* itemPicture(long ind);
	char* itemText(long ind);
	long mode();
	
	void setBackGround(long vl);
	void setForeGround(long vl);
	void setItemPicture(long ind,gPicture *pic);
	void setItemSelected(long ind,bool vl);
	void setItemText(long ind,char *vl);
	void setMode(long vl);
//"Methods"
	int add(char *key,char *vl,gPicture *pic,char *pos);
	void clear();
	void remove(long pos);

//"Signals"
	void (*onSelect)(gListView *sender);

//"Private"
	long selmode;
	bool sort;
};

class gListBox : public gControl
{
public:
	gListBox(gControl *parent);

//"Properties"
	long foreGround();
	long backGround();
	long count();
	long index();
	bool itemSelected(long ind);
	char* itemText(long ind);
	char* list();
	long mode();
	bool sorted();
	char *text();
	
	void setForeGround(long vl);
	void setBackGround(long vl);
	void setIndex(long vl); //TODO
	void setItemSelected(long ind,bool vl);
	void setItemText(long ind,char *txt); //TODO
	void setList(char *vl);
	void setMode(long vl);
	void setSorted(bool vl);
	
//"Methods"
	void add(char *vl,long pos);
	void clear();
	long find(char *ptr);
	void remove(long pos);
//"Events"
	void (*onSelect)(gListBox *data);
	
//"Private"
	long selmode;
	bool sort;
};

class gSpinBox : public gControl
{
public:
	gSpinBox(gControl *parent);
	
//"Properties"
	long backGround();
	long foreGround();
	long maxValue();
	long minValue();
	long step();
	long value();
	bool wrap();
	
	void setBackGround(long vl);
	void setForeGround(long vl);
	void setMaxValue  (long vl);
	void setMinValue  (long vl);
	void setStep      (long vl);
	void setValue     (long vl);
	void setWrap      (bool vl);
	
//"Signals"
	void (*onChange)  (gSpinBox *sender);

};

class gComboBox : public gControl
{
public:
	gComboBox(gControl *parent);
	
	long count();
	long index();
	char* itemText(long ind);
	long length();
	char* list();
	bool readOnly();
	bool sorted();
	char* text();
	
	void setIndex(long vl);
	void setItemText(long ind,char *txt);
	void setList(char *vl);
	void setReadOnly(bool vl);
	void setSorted(bool vl);
	void setText(char *vl);

//"Methods"
	void add(char *vl,long pos);
	void clear();
	long find(char *ptr);
	void remove(long pos);

//"Signals"
	void (*onChange)(gComboBox *sender);	
	
//"Private"
	GtkWidget *entry;
	bool sort;

};

class gTextBox : public gControl
{
public:
	gTextBox(gControl *parent);

//"Properties"	
	int alignment();
	long backGround();
	long foreGround();
	bool hasBorder();
	long length();
	int maxLength();
	bool password();
	int position();
	bool readOnly();
	int selLength();
	int selStart();
	char* selText();
	char* text();
	
	void setAlignment(int vl);
	void setBackGround(long vl);
	void setBorder(bool vl);
	void setForeGround(long vl);
	void setMaxLength(int len);
	void setPassword(bool vl);
	void setPosition(int pos);
	void setReadOnly(bool vl);
	void setSelText(char *txt,int len);
	void setText(char *vl);

//"Methods"
	void insert(char* txt,int len);
	void selClear();
	void select(int start,int len);
	void selectAll();
	
//"Signals"
	void (*onChange)(gTextBox *sender);

};

class gTextArea : public gControl
{
public:
	gTextArea(gControl *parent);
	
//"Properties"
	bool hasBorder();
	long backGround();
	long column();
	long foreGround();
	long length();
	long line();
	long position();
	bool readOnly();
	char* text();
	bool wrap();
	
	void setBackGround(long vl);
	void setBorder(bool vl);
	void setColumn(long vl);
	void setForeGround(long vl);
	void setLine(long vl);
	void setPosition(long vl);
	void setReadOnly(bool vl);
	void setText(char *txt);
	void setWrap(bool vl);
	
//"Methods"
	void copy();
	void cut();
	void ensureVisible();
	void paste();
	void insert(char *txt);
	long toLine(long pos);
	long toColumn(long pos);
	long toPosition(long line,long col); 
	
//"Selection properties"
	long selStart();
	long selEnd();
	char* selText();
	
	void setSelText(char *vl);

//"Selection methods"
	void selDelete();
	void selSelect(long start,long length);
};

class gSlider : public gControl
{
public:
	gSlider(gControl *parent);
	
//"Properties"
	long foreGround();
	long backGround();
	long max();
	long min();
	bool tracking();
	long value();
	bool mark();
	long step();
	long pageStep();
	
	void setForeGround(long vl);
	void setBackGround(long vl);
	void setMax(long vl);
	void setMin(long vl);
	void setTracking(bool vl);
	void setValue(long vl);
	void setMark(bool vl);
	void setStep(long vl);
	void setPageStep(long vl);
	
//"Signals"
	void (*onChange)(gSlider *sender);
	
//"Private"
	void orientation(long w,long h);
	bool bDraw;
	long p_step;
	long p_page;
};

class gScrollBar : public gSlider
{
public:
	gScrollBar(gControl *parent);
	
//"Private"
	void orientation(long w,long h);
};


class gContainer : public gControl
{
public:
	gContainer();
	gContainer(gControl *parent);
	~gContainer();
	
	long arrange();
	bool autoSize();
	long padding();
	long spacing();
	virtual long clientWidth();
	virtual long clientHeight();
	virtual long clientX();
	virtual long clientY();
	
	void setArrange(long vl);
	void setAutoSize(bool vl);
	void setPadding(long vl);
	void setSpacing(long vl);
		
	long childCount();
	gControl* child(long index);
	
//"Private"
	long offX;
	long offY; 
	long pad;
	long spc;
	long arr;
	bool autosz;
	bool locked;
	GtkWidget *radiogroup;
	virtual void performArrange();
};

class gScrollView : public gContainer
{
public:
	gScrollView(gControl *parent);
	
//"Properties"
	long clientX();
	long clientY();
	long clientWidth();
	long clientHeight();
	long scrollX();
	void setScrollX(long vl);
	long scrollY();
	void setScrollY(long vl);
	long scrollBar();
	void setScrollBar(long vl);
	long foreGround();
	long backGround();
	void setForeGround(long vl);
	void setBackGround(long vl);
	int getBorder();
	void setBorder(int vl);
	long width();
	long height();
	void setWidth(long vl);
	void setHeight(long vl);
//"Methods"
	void move(long x,long y);
	void resize(long w,long h);
	void scroll(long x,long y);
	
//"Private"
	void performArrange();

};

class gDrawingArea : public gContainer
{
public:
	gDrawingArea(gControl *parent);
		
	long foreGround();
	long backGround();
	int getBorder();
	void setBorder(int vl);
	
	void setForeGround(long vl);
	void setBackGround(long vl);
	bool cached();
	void setCached(bool vl);

//"Methods"
	void clear();
//"Events"
	void (*onExpose)(gDrawingArea *sender,long x,long y,long w,long h);
	
//"Private"
	GdkPixmap *buffer;
	GdkGC *gc;

};

class gTabStrip : public gContainer
{
public:
	gTabStrip(gControl *parent);
	
//"Properties"
	long count();
	long index();
	long orientation();
	gPicture* picture(long ind);
	void setOrientation(long vl);
	bool tabEnabled(long vl);
	char *text(long ind);
	
	int setCount(long vl);
	void setIndex(long vl);
	void setPicture(long ind,gPicture *pic);
	void setTabEnabled(long ind,bool vl);
	void setText(long ind,char *txt);
//"Private"
	long tabCount;

};

class gFrame : public gContainer
{
public:
	gFrame(gControl *parent,bool title);
		
	long backGround();
	void setBackGround(long vl);
	long foreGround();
	void setForeGround(long vl);
	char* text();
	void setText(char* vl);
	int getBorder();
	void setBorder(int vl);

};


class gMenu
{
public:
	gMenu(gMainWindow *par,bool hidden);
	gMenu(gMenu *par,bool hidden);
	~gMenu();
	
	void *hFree;
	
	static long   winChildCount(gMainWindow *win);
	static gMenu* winChildMenu(gMainWindow *par,long pos);
//"Properties"
	bool checked();
	gMenu* childMenu(long pos);
	long childCount();
	char* shortcut();
	char* text();
	bool visible();
	gPicture* picture();
	
	void setChecked(bool vl);
	void setShortcut(char *txt);
	void setText(char *vl);
	void setVisible(bool vl);
	void setPicture(gPicture *pic);
	
//"Methods"
	void popup();
	void destroy();
	
// "Signals"
	void (*onFinish)(gMenu *sender); // Special
	void (*onClick)(gMenu *sender);

//"Private"
	void *pr;
	GdkPixbuf *img_buf;
	bool chk;
	bool top_level;
	bool stop_signal;
	char *txt;
	char *scut;
	GtkAccelGroup *accel;
	GtkMenu *child;
	GtkMenuItem *menu;
	GtkWidget *lbl;
	GtkWidget *aclbl;
	GtkWidget *img;
	void setImage(GdkPixbuf *buf);
};

class gMainWindow : public gContainer
{
public:
	gMainWindow(bool plug);
	gMainWindow(gControl *parent);
	~gMainWindow();

//"Properties"
	int _border();
	gPicture *icon();
	gPicture *mask();
	long menuCount();
	bool modal();
	const char* text();
	bool topOnly();
	bool skipTaskBar();
	
	void setBorder(int b);
	void setIcon(gPicture *pic);
	void setMask(gPicture *pic);
	void setText(const char *txt);
	void setTopOnly(bool vl);
	void setSkipTaskBar(bool b);

//"Methods"
	void center();
	void showModal();
	void raise();
	
//"Signals"
	void (*onOpen)(gMainWindow *sender);
	void (*onShow)(gMainWindow *sender);
	void (*onHide)(gMainWindow *sender);
	void (*onMove)(gMainWindow *sender);
	void (*onResize)(gMainWindow *sender);
	bool (*onClose)(gMainWindow *sender);

//"Private"
	bool top_only;	
	bool resize_flag;
	GtkAccelGroup *accel;
	GtkStyle *win_style;
	GtkMenuBar *menuBar;
	GtkWidget *esc_button;
//	gint bufW,bufH,bufX,bufY;
};

class gApplication
{
public:
	static void init(int *argc,char ***argv);
	static void iteration();
	
	static long controlCount();
	static long winCount();
	
	static gControl* controlItem(long index);
	static gControl* controlItem(GtkWidget *wid);
	static gMainWindow *winItem(long index);	
	
//
	static void suspendEvents(bool vl);
	static void enableEvents();
	static bool userEvents();
	static bool allEvents();
	
};


#endif
