/***************************************************************************

  color.c

  The SDL component

  Color management

  (c) 2004-2005 Laurent Carlier <lordheavy@infonie.fr>
                Benot Minisini <gambas@users.sourceforge.net>

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 1, or (at your option)
  any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

***************************************************************************/

#define __COLOR_C

#include "main.h"
#include "gambas.h"

#include "SDL_endian.h"

/**************************************************************************

  Color - Set the color for the draw from rgba value

***************************************************************************/

BEGIN_METHOD(COLOR_rgb, GB_INTEGER Red; GB_INTEGER Green; GB_INTEGER Blue; GB_INTEGER Alpha)

  Uint8 red, green, blue, alpha;

  red   = VARG(Red);
  green = VARG(Green);
  blue  = VARG(Blue);
  alpha = VARGOPT(Alpha, 0xFF);

  GB.ReturnInteger((red << 24) | (green << 16) | (blue << 8) | alpha);
  
END_METHOD

BEGIN_METHOD(COLOR_red, GB_INTEGER color)

  GB.ReturnInteger((VARG(color) >> 24) & 0xFF);

END_METHOD

BEGIN_METHOD(COLOR_green, GB_INTEGER color)

  GB.ReturnInteger((VARG(color) >> 16) & 0xFF);

END_METHOD

BEGIN_METHOD(COLOR_blue, GB_INTEGER color)

  GB.ReturnInteger((VARG(color) >> 8) & 0xFF);

END_METHOD

BEGIN_METHOD(COLOR_alpha, GB_INTEGER color)

  GB.ReturnInteger(VARG(color) & 0xFF);

END_METHOD

/**************************************************************************/

GB_DESC Ccolor[] =
{
  GB_DECLARE("Color", 0), GB_NOT_CREATABLE(),

  GB_STATIC_METHOD("RGBA", "i", COLOR_rgb, "(Red)i(Green)i(Blue)i[(Alpha)i]"),

  GB_STATIC_METHOD("Red",   "i", COLOR_red,   "(Color)i"),
  GB_STATIC_METHOD("Green", "i", COLOR_green, "(Color)i"),
  GB_STATIC_METHOD("Blue" , "i", COLOR_blue,  "(Color)i"),
  GB_STATIC_METHOD("Alpha", "i", COLOR_alpha, "(Color)i"),

  GB_END_DECLARE
};
