/***************************************************************************

  game.c

  The SDL component

  Various game utils

  (c) 2004-2005 Laurent Carlier <lordheavy@infonie.fr>
                Benot Minisini <gambas@users.sourceforge.net>

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 1, or (at your option)
  any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

***************************************************************************/

#define __GAME_C

#include "main.h"
#include "gambas.h"

#include "surface.h"

#include "SDL.h"

/**************************************************************************

  Poke - Set/get bytes of a Surface/Screen

***************************************************************************/

BEGIN_METHOD(POKE_writebyte, GB_INTEGER index; GB_INTEGER value; GB_OBJECT surface)

  SDL_Surface *pokesurf = MainScreen;
  Uint8 value  = VARG(value);
  Uint32 index = VARG(index);
  Uint8 *byte = 0;
  
  if (MISSING(surface))
  {
    if (!MainScreen)
    {
      GB.ReturnBoolean(0);
      return;
    }
  }
  else
  {
    CSURFACE *surface = VARG(surface);

    if (GB.CheckObject(surface))
      return;

    pokesurf = GETSURFACE(surface);
  }

  if (index>(pokesurf->h*pokesurf->pitch))
  {
    GB.ReturnBoolean(0);
    return;
  }
  
  byte = pokesurf->pixels;
  
  if (!SDL_MUSTLOCK(pokesurf))
    SDL_LockSurface(pokesurf);
  
  byte[index] = value;
    
  if (!SDL_MUSTLOCK(pokesurf))
    SDL_UnlockSurface(pokesurf);
    
  GB.ReturnBoolean(1);

END_METHOD

BEGIN_METHOD(POKE_readbyte, GB_INTEGER index; GB_OBJECT surface)

  SDL_Surface *pokesurf = MainScreen;
  Uint32 index = VARG(index);
  Uint8 *byte = 0;
  
  if (MISSING(surface))
  {
    if (!MainScreen)
    {
      GB.ReturnInteger(0);
      return;
    }
  }
  else
  {
    CSURFACE *surface = VARG(surface);

    if (GB.CheckObject(surface))
      return;

    pokesurf = GETSURFACE(surface);
  }

  if (index>(pokesurf->h*pokesurf->pitch))
  {
    GB.ReturnInteger(0);
    return;
  }
  
  byte = pokesurf->pixels;
  
  if (!SDL_MUSTLOCK(pokesurf))
    SDL_LockSurface(pokesurf);
  
  GB.ReturnInteger(byte[index]);
    
  if (!SDL_MUSTLOCK(pokesurf))
    SDL_UnlockSurface(pokesurf);
    
END_METHOD

BEGIN_METHOD(POKE_linewidth, GB_OBJECT surface)

  SDL_Surface *pokesurf = MainScreen;
  
  if (MISSING(surface))
  {
    if (!MainScreen)
    {
      GB.ReturnInteger(0);
      return;
    }
  }
  else
  {
    CSURFACE *surface = VARG(surface);

    if (GB.CheckObject(surface))
      return;

    pokesurf = GETSURFACE(surface);
  }

  GB.ReturnInteger(pokesurf->pitch);
   
END_METHOD

BEGIN_METHOD(POKE_pixelsize, GB_OBJECT surface)

  SDL_Surface *pokesurf = MainScreen;
  
  if (MISSING(surface))
  {
    if (!MainScreen)
    {
      GB.ReturnInteger(0);
      return;
    }
  }
  else
  {
    CSURFACE *surface = VARG(surface);

    if (GB.CheckObject(surface))
      return;

    pokesurf = GETSURFACE(surface);
  }

  GB.ReturnInteger(pokesurf->format->BytesPerPixel);
   
END_METHOD

/**************************************************************************/

GB_DESC Cpoke[] =
{
  GB_DECLARE("Poke", 0), GB_NOT_CREATABLE(),

  GB_STATIC_METHOD("WriteByte", "b", POKE_writebyte, "(Index)i(Value)i[(Surface)Surface;]"),
  GB_STATIC_METHOD("ReadByte",  "i", POKE_readbyte,  "(Index)i[(Surface)Surface;]"),
  
  GB_STATIC_METHOD("LineWidth", "i", POKE_linewidth, "[(Surface)Surface;]"),
  GB_STATIC_METHOD("PixelSize", "i", POKE_pixelsize, "[(Surface)Surface;]"),

  GB_END_DECLARE
};
