/***************************************************************************

  key.c

  Gambas extension using SDL

  (c) 2004-2005 Laurent Carlier <lordheavy@infonie.fr>
                Benot Minisini <gambas@sourceforge.net>

  This program is free software; you can redistribute it
   and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 1, or (at your option)
  any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

***************************************************************************/

#define __KEY_C

#include "main.h"
#include "gambas.h"

#include "SDL.h"

int getkey(SDL_KeyboardEvent *key)
{
  return (key->keysym.sym);
}

BEGIN_METHOD(KEY_wait, GB_INTEGER key)

  SDL_Event Event;
  Uint16 key = 0;

  while (1)
  {
    if (SDL_PollEvent(&Event))
    {
      if ((Event.type == (SDL_KEYDOWN)))
      {
        if (MISSING(key))
	{
	  key = Event.key.keysym.sym;
	  break;
	}
	else
	{
	  if (Event.key.keysym.sym == VARG(key))
	  {
	    key = getkey(&Event.key);
	    break;
	  }
	}
      }
    }
  }
  
  GB.ReturnInteger(key);

END_METHOD

BEGIN_METHOD_VOID(KEY_scan)

  SDL_Event Event;
  Uint16 key = 0;

  while (SDL_PollEvent(&Event))
  {
    if ((Event.type == (SDL_KEYDOWN)))
    {
      key = getkey(&Event.key);
      break;
    }
  }
  
  GB.ReturnInteger(key);

END_METHOD

BEGIN_METHOD(KEY_pressed, GB_INTEGER key)

  Uint8 *KeyPressed;
  SDL_Event Event;

  if (MISSING(key))
  {
    while (SDL_PollEvent(&Event))
    {
      if ((Event.type == (SDL_KEYDOWN)))
      {
        GB.ReturnBoolean(1);
        return;
      }
    }
    
    GB.ReturnBoolean(0);
    return;
  }
  
  KeyPressed = SDL_GetKeyState(NULL);
  
  if (KeyPressed[VARG(key)])
    GB.ReturnBoolean(1);
  else
    GB.ReturnBoolean(0);
    
  SDL_PumpEvents();

END_METHOD

BEGIN_METHOD(KEY_text, GB_INTEGER key)

  GB.ReturnNewZeroString((char *) SDL_GetKeyName(VARG(key)));
  
END_METHOD

BEGIN_PROPERTY(KEY_shift)

  GB.ReturnBoolean(SDL_GetModState() & KMOD_SHIFT);
    
END_PROPERTY

BEGIN_PROPERTY(KEY_control)

  GB.ReturnBoolean(SDL_GetModState() & KMOD_CTRL);

END_PROPERTY

BEGIN_PROPERTY(KEY_alt)

  GB.ReturnBoolean(SDL_GetModState() & KMOD_ALT);

END_PROPERTY

BEGIN_PROPERTY(KEY_meta)

  GB.ReturnBoolean(SDL_GetModState() & KMOD_META);

END_PROPERTY

/**************************************************************************/

GB_DESC Ckey[] =
{
  GB_DECLARE("Key", 0), GB_VIRTUAL_CLASS(),

  GB_STATIC_METHOD("Wait",    "i", KEY_wait,    "[(key)i]"),
  GB_STATIC_METHOD("Scan",    "i", KEY_scan,    NULL),
  GB_STATIC_METHOD("Pressed", "b", KEY_pressed, "[(key)i]"),
  GB_STATIC_METHOD("Text",    "s", KEY_text,    "(key)i"),
  
  GB_STATIC_PROPERTY_READ("Shift",   "b", KEY_shift),
  GB_STATIC_PROPERTY_READ("Control", "b", KEY_control),
  GB_STATIC_PROPERTY_READ("Alt",     "b", KEY_alt),
  GB_STATIC_PROPERTY_READ("Meta",    "b", KEY_meta),

  GB_CONSTANT("BackSpace",   "i", SDLK_BACKSPACE),
  GB_CONSTANT("Tab",         "i", SDLK_TAB),
  GB_CONSTANT("Clear",       "i", SDLK_CLEAR),
  GB_CONSTANT("Return",      "i", SDLK_RETURN),
  GB_CONSTANT("Pause",       "i", SDLK_PAUSE),
  GB_CONSTANT("Escape",      "i", SDLK_ESCAPE),
  GB_CONSTANT("Esc",         "i", SDLK_ESCAPE),
  GB_CONSTANT("Space",       "i", SDLK_SPACE),
  GB_CONSTANT("Exclaim",     "i", SDLK_EXCLAIM),
  GB_CONSTANT("QuoteDbl",    "i", SDLK_QUOTEDBL),
  GB_CONSTANT("Hash",        "i", SDLK_HASH),
  GB_CONSTANT("Dollar",      "i", SDLK_DOLLAR),
  GB_CONSTANT("Ampersand",   "i", SDLK_AMPERSAND),
  GB_CONSTANT("Quote",       "i", SDLK_QUOTE),
  GB_CONSTANT("LeftPar",     "i", SDLK_LEFTPAREN),
  GB_CONSTANT("RightPar",    "i", SDLK_RIGHTPAREN),
  GB_CONSTANT("Asterisk",    "i", SDLK_ASTERISK),
  GB_CONSTANT("Plus",        "i", SDLK_PLUS),
  GB_CONSTANT("Comma",       "i", SDLK_COMMA),
  GB_CONSTANT("Minus",       "i", SDLK_MINUS),
  GB_CONSTANT("Period",      "i", SDLK_PERIOD),
  GB_CONSTANT("Slash",       "i", SDLK_SLASH),
  GB_CONSTANT("Colon",       "i", SDLK_COLON),
  GB_CONSTANT("SemiColon",   "i", SDLK_SEMICOLON),
  GB_CONSTANT("Less",        "i", SDLK_LESS),
  GB_CONSTANT("Equals",      "i", SDLK_EQUALS),
  GB_CONSTANT("Greater",     "i", SDLK_GREATER),
  GB_CONSTANT("Question",    "i", SDLK_QUESTION),
  GB_CONSTANT("At",          "i", SDLK_AT),
  GB_CONSTANT("LeftBracket", "i", SDLK_LEFTBRACKET),
  GB_CONSTANT("BackSlash",   "i", SDLK_BACKSLASH),
  GB_CONSTANT("RightBracket","i", SDLK_RIGHTBRACKET),
  GB_CONSTANT("Caret",       "i", SDLK_CARET),
  GB_CONSTANT("UnderScore",  "i", SDLK_UNDERSCORE),
  GB_CONSTANT("BackQuote",   "i", SDLK_BACKQUOTE),
  GB_CONSTANT("Delete",      "i", SDLK_DELETE),

  GB_CONSTANT("Num0", "i", SDLK_0),
  GB_CONSTANT("Num1", "i", SDLK_1),
  GB_CONSTANT("Num2", "i", SDLK_2),
  GB_CONSTANT("Num3", "i", SDLK_3),
  GB_CONSTANT("Num4", "i", SDLK_4),
  GB_CONSTANT("Num5", "i", SDLK_5),
  GB_CONSTANT("Num6", "i", SDLK_6),
  GB_CONSTANT("Num7", "i", SDLK_7),
  GB_CONSTANT("Num8", "i", SDLK_8),
  GB_CONSTANT("Num9", "i", SDLK_9),

  GB_CONSTANT("A", "i", SDLK_a),
  GB_CONSTANT("B", "i", SDLK_b),
  GB_CONSTANT("C", "i", SDLK_c),
  GB_CONSTANT("D", "i", SDLK_d),
  GB_CONSTANT("E", "i", SDLK_e),
  GB_CONSTANT("F", "i", SDLK_f),
  GB_CONSTANT("G", "i", SDLK_g),
  GB_CONSTANT("H", "i", SDLK_h),
  GB_CONSTANT("I", "i", SDLK_i),
  GB_CONSTANT("J", "i", SDLK_j),
  GB_CONSTANT("K", "i", SDLK_k),
  GB_CONSTANT("L", "i", SDLK_l),
  GB_CONSTANT("M", "i", SDLK_m),
  GB_CONSTANT("N", "i", SDLK_n),
  GB_CONSTANT("O", "i", SDLK_o),
  GB_CONSTANT("P", "i", SDLK_p),
  GB_CONSTANT("Q", "i", SDLK_q),
  GB_CONSTANT("R", "i", SDLK_r),
  GB_CONSTANT("S", "i", SDLK_s),
  GB_CONSTANT("T", "i", SDLK_t),
  GB_CONSTANT("U", "i", SDLK_u),
  GB_CONSTANT("V", "i", SDLK_v),
  GB_CONSTANT("W", "i", SDLK_w),
  GB_CONSTANT("X", "i", SDLK_x),
  GB_CONSTANT("Y", "i", SDLK_y),
  GB_CONSTANT("Z", "i", SDLK_z),

  GB_CONSTANT("Left",  "i", SDLK_LEFT),
  GB_CONSTANT("Right", "i", SDLK_RIGHT),
  GB_CONSTANT("Up",    "i", SDLK_UP),
  GB_CONSTANT("Down",  "i", SDLK_DOWN),

  GB_CONSTANT("Insert",   "i", SDLK_INSERT),
  GB_CONSTANT("Home",     "i", SDLK_HOME),
  GB_CONSTANT("End",      "i", SDLK_END),
  GB_CONSTANT("PageUp",   "i", SDLK_PAGEUP),
  GB_CONSTANT("PageDown", "i", SDLK_PAGEDOWN),

  GB_CONSTANT("Kpad0", "i", SDLK_KP0),
  GB_CONSTANT("Kpad1", "i", SDLK_KP1),
  GB_CONSTANT("Kpad2", "i", SDLK_KP2),
  GB_CONSTANT("Kpad3", "i", SDLK_KP3),
  GB_CONSTANT("Kpad4", "i", SDLK_KP4),
  GB_CONSTANT("Kpad5", "i", SDLK_KP5),
  GB_CONSTANT("Kpad6", "i", SDLK_KP6),
  GB_CONSTANT("Kpad7", "i", SDLK_KP7),
  GB_CONSTANT("Kpad8", "i", SDLK_KP8),
  GB_CONSTANT("Kpad9", "i", SDLK_KP9),
  GB_CONSTANT("KpadPeriod",   "i", SDLK_KP_PERIOD),
  GB_CONSTANT("KpadDivide",   "i", SDLK_KP_DIVIDE),
  GB_CONSTANT("KpadMultiply", "i", SDLK_KP_MULTIPLY),
  GB_CONSTANT("KpadMinus",    "i", SDLK_KP_MINUS),
  GB_CONSTANT("KpadPlus",     "i", SDLK_KP_PLUS),
  GB_CONSTANT("KpadEnter",    "i", SDLK_KP_ENTER),
  GB_CONSTANT("KpadEquals",   "i", SDLK_KP_EQUALS),

  GB_CONSTANT("F1",  "i", SDLK_F1),
  GB_CONSTANT("F2",  "i", SDLK_F2),
  GB_CONSTANT("F3",  "i", SDLK_F3),
  GB_CONSTANT("F4",  "i", SDLK_F4),
  GB_CONSTANT("F5",  "i", SDLK_F5),
  GB_CONSTANT("F6",  "i", SDLK_F6),
  GB_CONSTANT("F7",  "i", SDLK_F7),
  GB_CONSTANT("F8",  "i", SDLK_F8),
  GB_CONSTANT("F9",  "i", SDLK_F9),
  GB_CONSTANT("F10", "i", SDLK_F10),
  GB_CONSTANT("F11", "i", SDLK_F11),
  GB_CONSTANT("F12", "i", SDLK_F12),
  GB_CONSTANT("F13", "i", SDLK_F13),
  GB_CONSTANT("F14", "i", SDLK_F14),
  GB_CONSTANT("F15", "i", SDLK_F15),

  GB_CONSTANT("NumLock",    "i", SDLK_NUMLOCK),
  GB_CONSTANT("CapsLock",   "i", SDLK_CAPSLOCK),
  GB_CONSTANT("ScrollLock", "i", SDLK_SCROLLOCK),
  GB_CONSTANT("RightShift", "i", SDLK_RSHIFT),
  GB_CONSTANT("LeftShift",  "i", SDLK_LSHIFT),
  GB_CONSTANT("RightCtrl",  "i", SDLK_RCTRL),
  GB_CONSTANT("LeftCtrl",   "i", SDLK_LCTRL),
  GB_CONSTANT("RightAlt",   "i", SDLK_RALT),
  GB_CONSTANT("LeftAlt",    "i", SDLK_LALT),
  GB_CONSTANT("RightMeta",  "i", SDLK_RMETA),
  GB_CONSTANT("LeftMeta",   "i", SDLK_LMETA),
  GB_CONSTANT("RightWin",   "i", SDLK_RSUPER),
  GB_CONSTANT("LeftWin",    "i", SDLK_LSUPER),
  GB_CONSTANT("ModeShift",  "i", SDLK_MODE),
  GB_CONSTANT("Help",       "i", SDLK_HELP),
  GB_CONSTANT("PrintScr",   "i", SDLK_PRINT),
  GB_CONSTANT("SysReq",     "i", SDLK_SYSREQ),
  GB_CONSTANT("Break",      "i", SDLK_BREAK),
  GB_CONSTANT("Menu",       "i", SDLK_MENU),
  GB_CONSTANT("Power",      "i", SDLK_POWER),
  GB_CONSTANT("Euro",       "i", SDLK_EURO),

  GB_END_DECLARE
};
