/***************************************************************************

  main.c

  The SDL component

  (c) 2004-2005 Laurent Carlier <lordheavy@infonie.fr>
                Benot Minisini <gambas@users.sourceforge.net>

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 1, or (at your option)
  any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

***************************************************************************/

#define __MAIN_C

#include "main.h"
#include "gb_common.h"
#include <string.h>

#include "driver.h"
#include "surface.h"
#include "screen.h"
#include "color.h"
#include "palette.h"
#include "draw.h"
#include "key.h"
#include "mouse.h"
#include "font.h"
#include "timer.h"
#include "game.h"

#include "../gb.sdl.h"

#include "SDL.h"
#include "SDL_ttf.h"

const SDL_VideoInfo *DriverInfo = NULL;
Mscreen *mScreen = NULL;

GB_INTERFACE GB;
SDL_INTERFACE SDL;

static int my_loop()
{
	SDL_Event event;
  
	if (!MainScreen)
		return (0);

	while (!mScreen->WantQuit)
	{
		while (SDL_PollEvent(&event))
		{
		}
	}

	return (mScreen->RetVal);
}

static void my_post()
{
	GB.CheckPost();
}

GB_DESC *GB_CLASSES[] =
{
  Cdriverinfo,
  Cscreen,
  Cdrawing, Ccolor,
  Cpalette, Cpalcolor,
  Cfont,
  Csurface, Csurfacealpha, Csurfacecolor,
  Ckey, Cmouse,
  Ctimer,
  /*Csprite,*/ Cpoke,

  NULL
};

int GB_INIT(void)
{
  GB.GetInterface("gb.sdl", SDL_INTERFACE_VERSION, &SDL);

  if (SDL.Init (SDL_INIT_VIDEO | SDL_INIT_TIMER | SDL_INIT_EVENTTHREAD))
    return FALSE;

  if (TTF_Init()==-1)
  {
    GB.Error("TTF_Init : &1\n", TTF_GetError());
    return FALSE;
  }

  DRIVERINFO_init();
  SDL_EnableUNICODE(1);

  GB.Hook(GB_HOOK_LOOP, (void *)my_loop);
  GB.Hook(GB_HOOK_POST, (void *)my_post);

  /* Here we define default datas about the screen */
  GB.Alloc ((void **) &mScreen, sizeof (Mscreen));
  memset (mScreen, 0, sizeof (Mscreen));

  mScreen->Wname = GB.Application.Name();             /* Default name of the window -> name of the app */
  mScreen->Gred = mScreen->Ggreen = mScreen->Gblue = 1; /* Default gamma of the screen */
  if (!DriverInfo->wm_available)
    mScreen->fullscreen = 1;

  return FALSE;
}

void GB_EXIT(void)
{
  TTF_Quit();
  SDL.Quit();
  GB.Free ((void **) &mScreen);
}
