/***************************************************************************

  mouse.c

  Gambas extension using SDL

  (c) 2004-2005 Laurent Carlier <lordheavy@infonie.fr>
                Benot Minisini <gambas@users.sourceforge.net>
                Bodard Fabien   <gambasfr@wanadoo.fr>

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 1, or (at your option)
  any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

***************************************************************************/

#define __MOUSE_C

#include "main.h"
#include "gambas.h"

#include "SDL.h"

static int rx,ry;

BEGIN_METHOD(MOUSE_move, GB_INTEGER x; GB_INTEGER y)

  SDL_WarpMouse(VARG(x), VARG(y));

END_METHOD

BEGIN_PROPERTY(MOUSE_x)

  SDL_GetMouseState(&rx, NULL);
  GB.ReturnInteger(rx);

END_PROPERTY

BEGIN_PROPERTY(MOUSE_y)

  SDL_GetMouseState(NULL, &ry);
  GB.ReturnInteger(ry);

END_PROPERTY

BEGIN_PROPERTY(MOUSE_left)

  GB.ReturnBoolean(SDL_GetMouseState(NULL, NULL)& SDL_BUTTON(1));

END_PROPERTY

BEGIN_PROPERTY(MOUSE_right)

  GB.ReturnBoolean(SDL_GetMouseState(NULL, NULL)& SDL_BUTTON(3));

END_PROPERTY

BEGIN_PROPERTY(MOUSE_middle)

  GB.ReturnBoolean(SDL_GetMouseState(NULL, NULL)& SDL_BUTTON(2));

END_PROPERTY

BEGIN_PROPERTY(MOUSE_relativex)

  int x;

  SDL_GetMouseState(&x, NULL);
  GB.ReturnInteger(x-rx);
  rx = x;

END_PROPERTY

BEGIN_PROPERTY(MOUSE_relativey)

  int y;

  SDL_GetMouseState(NULL, &y);
  GB.ReturnInteger(y-ry);
  ry = y;

END_PROPERTY

BEGIN_PROPERTY(MOUSE_grab)

  if (READ_PROPERTY)
  {
    GB.ReturnBoolean(SDL_WM_GrabInput(SDL_GRAB_QUERY) && SDL_GRAB_ON);
  }
  else
  {
    if (VPROP(GB_BOOLEAN))
      SDL_WM_GrabInput(SDL_GRAB_ON);
    else
      SDL_WM_GrabInput(SDL_GRAB_OFF);
  }

END_PROPERTY

BEGIN_PROPERTY(MOUSE_show)

  if (READ_PROPERTY)
  {
    GB.ReturnBoolean(SDL_ShowCursor(SDL_GRAB_QUERY) && SDL_ENABLE);
  }
  else
  {
    if (VPROP(GB_BOOLEAN))
      SDL_ShowCursor(SDL_ENABLE);
    else
      SDL_ShowCursor(SDL_DISABLE);
  }

END_PROPERTY

/**************************************************************************/

GB_DESC Cmouse[] =
{
  GB_DECLARE("Mouse", 0), GB_VIRTUAL_CLASS(),

  GB_STATIC_METHOD("Move", NULL, MOUSE_move, "(X)i(Y)i"),

  GB_STATIC_PROPERTY_READ("X", "i", MOUSE_x),
  GB_STATIC_PROPERTY_READ("Y", "i", MOUSE_y),
  GB_STATIC_PROPERTY_READ("RelativeX", "i", MOUSE_relativex),
  GB_STATIC_PROPERTY_READ("RelativeY", "i", MOUSE_relativey),
  GB_STATIC_PROPERTY_READ("Left",   "b", MOUSE_left),
  GB_STATIC_PROPERTY_READ("Right",  "b", MOUSE_right),
  GB_STATIC_PROPERTY_READ("Middle", "b", MOUSE_middle),
  
  GB_STATIC_PROPERTY("Grab", "b", MOUSE_grab),
  GB_STATIC_PROPERTY("Show", "b", MOUSE_show),

  GB_END_DECLARE
};
