/***************************************************************************

  palette.c

  Gambas extension using SDL

  (c) 2004-2005 Laurent Carlier <lordheavy@infonie.fr>
                Benot Minisini <gambas@users.sourceforge.net>
                Fabien Bodard   <gambasfr@wanadoo.fr>

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 1, or (at your option)
  any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

***************************************************************************/

#define __PALETTE_C

#include "main.h"

#include "palette.h"
#include "surface.h"

#include "SDL.h"

/**************************************************************************

  Palette - a class to manage palettes (8 bpp color surfaces/screen)

***************************************************************************/

BEGIN_PROPERTY(PALCOLOR_value)

  if (READ_PROPERTY)
  {
    Uint32 value = (THIS->Color[THIS->index].r << 16) | (THIS->Color[THIS->index].g << 8) |
                   THIS->Color[THIS->index].b;
    GB.ReturnInteger(value);
  }
  else
  {
    Uint32 value = VPROP(GB_INTEGER);
    THIS->Color[THIS->index].r = ((value >> 16) & 0xFF);
    THIS->Color[THIS->index].g = ((value >> 8) & 0xFF);
    THIS->Color[THIS->index].b = (value & 0xFF);
  }
    
END_PROPERTY

BEGIN_METHOD(PALCOLOR_rgb, GB_INTEGER Red; GB_INTEGER Green; GB_INTEGER Blue)

  Uint8 red, green, blue;

  red   = VARG(Red);
  green = VARG(Green);
  blue  = VARG(Blue);

  THIS->Color[THIS->index].r = red;
  THIS->Color[THIS->index].g = green;
  THIS->Color[THIS->index].b = blue;
  
  if ((THIS->index)>(THIS->nbrcolors))
    THIS->nbrcolors = (THIS->index)+1;
  
END_METHOD

BEGIN_PROPERTY(PALCOLOR_red)

  GB.ReturnInteger(THIS->Color[THIS->index].r);
    
END_PROPERTY

BEGIN_PROPERTY(PALCOLOR_green)

  GB.ReturnInteger(THIS->Color[THIS->index].g);
    
END_PROPERTY

BEGIN_PROPERTY(PALCOLOR_blue)

  GB.ReturnInteger(THIS->Color[THIS->index].b);
    
END_PROPERTY

BEGIN_METHOD(PALETTE_get, GB_INTEGER index)

  long index = VARG(index);

  if (index<0 || index>255)
  {
    GB.Error("Palette index must be between 0 and 255");
    return;
  }

  THIS->index = index;
  RETURN_SELF();

END_METHOD

BEGIN_METHOD(PALETTE_getpalette, GB_OBJECT surface)

  SDL_Surface *palsurf = MainScreen;
  int loop;
    
  if (MISSING(surface))
  {
    if (!MainScreen)
    {
      GB.ReturnBoolean(0);
      return;
    }
  }
  else
  {
    CSURFACE *surface = VARG(surface);

    if (GB.CheckObject(surface))
      return;

    palsurf = GETSURFACE(surface);
  }

  if (palsurf->format->palette==NULL)
  {
    GB.ReturnBoolean(0);
    return;
  }
  
  THIS->nbrcolors = palsurf->format->palette->ncolors;
  
  for (loop=0; loop<THIS->nbrcolors; loop++)
    THIS->Color[loop] = palsurf->format->palette->colors[loop];
  
  GB.ReturnBoolean(1);
  
END_METHOD

BEGIN_METHOD(PALETTE_setpalette, GB_OBJECT surface; GB_INTEGER first; GB_INTEGER nbr)

  SDL_Surface *palsurf = MainScreen;
  int flags=0, firstcolor, nbrcolors;
    
  if (MISSING(surface))
  {
    if (!MainScreen)
    {
      GB.ReturnBoolean(0);
      return;
    }
  }
  else
  {
    CSURFACE *surface = VARG(surface);

    if (GB.CheckObject(surface))
      return;

    palsurf = GETSURFACE(surface);
  }

  if (palsurf->format->palette==NULL)
  {
    GB.ReturnBoolean(0);
    return;
  }
  
  firstcolor = VARGOPT(first, 0);
  nbrcolors  = VARGOPT(nbr,   1);
  
  if (firstcolor>255)
  {
    GB.ReturnBoolean(0);
    return;
  }

  if (nbrcolors+firstcolor>256)
    nbrcolors = 256 - firstcolor;
    
  if (THIS->logpal)
    flags = flags | SDL_LOGPAL;
      
  if (THIS->physpal)
    flags = flags | SDL_PHYSPAL;

  SDL_SetPalette(palsurf, flags, THIS->Color, firstcolor, nbrcolors);     
  GB.ReturnBoolean(1);
  
END_METHOD

BEGIN_PROPERTY(PALETTE_size)

  GB.ReturnInteger(THIS->nbrcolors);

END_PROPERTY

BEGIN_PROPERTY(PALETTE_logpal)

  if (READ_PROPERTY)
    GB.ReturnBoolean(THIS->logpal);
  else
    THIS->logpal = VPROP(GB_BOOLEAN);

END_PROPERTY

BEGIN_PROPERTY(PALETTE_physpal)

  if (READ_PROPERTY)
    GB.ReturnBoolean(THIS->physpal);
  else
    THIS->physpal = VPROP(GB_BOOLEAN);

END_PROPERTY

/**************************************************************************/

GB_DESC Cpalcolor[] =
{
  GB_DECLARE(".PaletteColor",0), GB_VIRTUAL_CLASS(),
  
  GB_PROPERTY("Value", "i", PALCOLOR_value),
  
  GB_METHOD("SetRGB", NULL, PALCOLOR_rgb, "(Red)i(Green)i(Blue)i"),
  
  GB_PROPERTY_READ("Red",   "i", PALCOLOR_red),
  GB_PROPERTY_READ("Green", "i", PALCOLOR_green),
  GB_PROPERTY_READ("Blue",  "i", PALCOLOR_blue),
  
  GB_END_DECLARE
};

GB_DESC Cpalette[] =
{
  GB_DECLARE("Palette", sizeof(CPALETTE)),

  GB_METHOD("_get", ".PaletteColor", PALETTE_get, "(Index)i"),
  GB_METHOD("GetPalette", "b", PALETTE_getpalette, "[(Surface)Surface;]"),
  GB_METHOD("SetPalette", "b", PALETTE_setpalette, "[(Surface)Surface;(FirstColor)i(NbrColors)i]"),
  
  GB_PROPERTY_READ("Size", "i", PALETTE_size),
  
  GB_PROPERTY("Logical",  "b", PALETTE_logpal),
  GB_PROPERTY("Physical", "b", PALETTE_physpal),

  GB_END_DECLARE
};
