/***************************************************************************

  screen.c

  The SDL component

  (c) 2004-2005 Laurent Carlier <lordheavy@infonie.fr>
                Benot Minisini <gambas@users.sourceforge.net>
                Bodard Fabien   <gambasfr@wanadoo.fr>
                Daniel Campos Fern�dez <danielcampos@netcourrier.com>

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 1, or (at your option)
  any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

***************************************************************************/

#define __SCREEN_C

#include "main.h"

#include "screen.h"
#include "surface.h"

#include "SDL.h"
#include "SDL_framerate.h"
#include "SDL_syswm.h"

static GB_CLASS CLASS_surface;
static FPSmanager fpsm;
static Uint32 start = 0;
static double fcurrRate = 0;
static int fcount = 1;
static int Rate = 0;

/**************************************************************************

  Screen - class for managing the main screen

***************************************************************************/

BEGIN_METHOD_VOID(SCREEN_init)

  CLASS_surface = GB.FindClass("Surface");
  SDL_initFramerate(&fpsm);
  
END_METHOD

BEGIN_METHOD(SCREEN_show, GB_INTEGER width; GB_INTEGER height; GB_INTEGER depth)

  Uint32 myFlags = 0;
  Uint16 width, height;
  Uint8 depth;

  width  = VARG(width);
  height = VARG(height);
  depth  = VARG(depth);

  /* enable hardware surface if the driver support it :)
     enable HWPALETTE for 8 bpp surfaces */
  myFlags |= (SDL_HWSURFACE | SDL_DOUBLEBUF | SDL_HWPALETTE | SDL_ASYNCBLIT);
  
  if (mScreen->fullscreen)
    myFlags |= SDL_FULLSCREEN;

  MainScreen = SDL_SetVideoMode (width, height, depth, myFlags);
  
  if (!MainScreen)
  {
    GB.Error ("Cannot open the Screen : &1",SDL_GetError());
    return;
  }
  
  SDL_WM_SetCaption(mScreen->Wname, mScreen->Wname);
  
  if (mScreen->fullscreen)
    SDL_SetGamma (mScreen->Gred, mScreen->Ggreen, mScreen->Gblue);

END_METHOD

BEGIN_METHOD(SCREEN_grab, GB_INTEGER x; GB_INTEGER y; GB_INTEGER w; GB_INTEGER h)

  SDL_Surface *grabsurf;
  CSURFACE *surface = 0;
  SDL_Rect rect;

  if (!MainScreen)
    return;

  GB.New((void **)surface, CLASS_surface, NULL, NULL);
  
  
  if (MISSING(x))
  {
    rect.w  = MainScreen->w;
    rect.h  = MainScreen->h;
    rect.x  = rect.y = 0;
  }
  else
  {
    rect.x = VARG(x);
    rect.y = VARG(y);
    
    if ((rect.x>MainScreen->w) ||(rect.y>MainScreen->h))
    {
      GB.Error ("Cannot grab the Screen : value out of range !");
      return;
    }
    
    rect.w  = rect.x + VARG(w);
    rect.h  = rect.y + VARG(h);
    
    if (rect.w>(MainScreen->w - rect.x))
      rect.w = (MainScreen->w - rect.x);
      
    if (rect.h>(MainScreen->h - rect.y))
      rect.h = (MainScreen->h - rect.y);
  }
     
  grabsurf = SDL_CreateRGBSurface (MainScreen->flags, rect.w, rect.h,
             MainScreen->format->BitsPerPixel, 0, 0, 0, 0);

  if (grabsurf == NULL)
  {
    GB.Error ("Cannot grab the Screen : &1",SDL_GetError());
    return;
  }

  if (SDL_BlitSurface(MainScreen, &rect, grabsurf, NULL)<0)
  {
    GB.Error ("Cannot grab the Screen : &1",SDL_GetError());
    return;
  }
  
  surface->id    = grabsurf;
  surface->alpha = grabsurf->format->alpha;
  surface->key   = grabsurf->format->colorkey;
  GB.ReturnObject(surface);
    
END_METHOD

BEGIN_METHOD(SCREEN_save, GB_STRING file)

  if (!MainScreen)
    return;
    
  if (SDL_SaveBMP(MainScreen, STRING(file))<0)
    GB.Error(" Cannot Save : &1",SDL_GetError());
  
END_METHOD

BEGIN_METHOD(SCREEN_close, GB_INTEGER RetVal)

  mScreen->WantQuit = 1;
  mScreen->RetVal   = VARGOPT(RetVal,0);

END_METHOD

BEGIN_METHOD_VOID(SCREEN_refresh)

  if (!MainScreen)
  {
    GB.ReturnInteger(0);
    return;
  }
  
  if (start==0)
    start = SDL_GetTicks();

  SDL_Flip(MainScreen);

  if (Rate)
    SDL_framerateDelay(&fpsm);
    
  if (fcount==100)
  {
    double value = (SDL_GetTicks() - start)/100;
    
    if (value>0)
      fcurrRate = 1000/value;
    
    fcount = 0;
    start = SDL_GetTicks();
  }
  else
   fcount++;
  
  GB.ReturnInteger(fcurrRate);
    
END_METHOD

BEGIN_METHOD(SCREEN_gamma, GB_FLOAT red; GB_FLOAT green; GB_FLOAT blue)

  mScreen->Gred   = VARG(red);
  mScreen->Ggreen = VARG(green);
  mScreen->Gblue  = VARG(blue);

  if (!MainScreen)
    return;

  if (MainScreen->flags & SDL_FULLSCREEN)
    SDL_SetGamma (mScreen->Gred, mScreen->Ggreen, mScreen->Gblue);

END_METHOD

BEGIN_METHOD(SCREEN_clear, GB_INTEGER Red; GB_INTEGER Green; GB_INTEGER Blue)

  Uint32 Color;

  if (!MainScreen)
    return;

  Color = SDL_MapRGB(MainScreen->format, (VARGOPT(Red,0) & 0xFF), (VARGOPT(Green,0) & 0xFF), (VARGOPT(Blue,0) & 0xFF));

  if (SDL_FillRect(MainScreen, NULL, Color) < 0)
    GB.Error("Cannot Clear the screen : &1",SDL_GetError());

END_METHOD

BEGIN_PROPERTY(SCREEN_caption)

  if (READ_PROPERTY)
  {
    GB.ReturnNewZeroString((char *) mScreen->Wname);
    return;
  }
  else
  {
    mScreen->Wname = GB.ToZeroString(PROP(GB_STRING));
    if (MainScreen)
      SDL_WM_SetCaption(mScreen->Wname, mScreen->Wname);
  }

END_PROPERTY

BEGIN_PROPERTY(SCREEN_fullscreen)

  if (READ_PROPERTY)
    GB.ReturnBoolean((int ) mScreen->fullscreen);
  else
  {
    if (DriverInfo->wm_available)
    {
      if ((VPROP(GB_BOOLEAN) && !(mScreen->fullscreen)) ||
         (!VPROP(GB_BOOLEAN) && (mScreen->fullscreen)))
      {
        if (MainScreen)
          SDL_WM_ToggleFullScreen(MainScreen);

	mScreen->fullscreen = !mScreen->fullscreen;
      }
    }
    /* Set the gamma only if fullscreen */
    if (MainScreen)
    {
      if (mScreen->fullscreen)
        SDL_SetGamma (mScreen->Gred, mScreen->Ggreen, mScreen->Gblue);
      else
        SDL_SetGamma (1,1,1);
    }
  }

END_PROPERTY

BEGIN_PROPERTY(SCREEN_width)

  if (MainScreen)
    GB.ReturnInteger((int ) MainScreen->w);
  else
    GB.ReturnInteger(0);

END_PROPERTY

BEGIN_PROPERTY(SCREEN_height)

  if (MainScreen)
    GB.ReturnInteger((int ) MainScreen->h);
  else
    GB.ReturnInteger(0);

END_PROPERTY

BEGIN_PROPERTY(SCREEN_depth)

  if (MainScreen)
    GB.ReturnInteger((Uint8 ) MainScreen->format->BitsPerPixel);
  else
    GB.ReturnInteger(0);

END_PROPERTY

BEGIN_PROPERTY(SCREEN_showed)

  if (MainScreen)
    GB.ReturnBoolean(1);
  else
    GB.ReturnBoolean(0);

END_PROPERTY

BEGIN_METHOD(SCREEN_setframerate, GB_INTEGER rate)

  Rate = VARG(rate);
  
  if (!Rate)
    return;
  
  SDL_setFramerate(&fpsm, Rate);
    
END_METHOD

BEGIN_PROPERTY(SCREEN_id)

  if (MainScreen)
  {
	SDL_SysWMinfo info;
	SDL_VERSION(&info.version);
	SDL_GetWMInfo(&info);
    GB.ReturnInteger(info.info.x11.window);
  }
  else
    GB.ReturnInteger(0);
  
END_PROPERTY

/**************************************************************************/

GB_DESC Cscreen[] =
{
  GB_DECLARE("Screen", 0), GB_NOT_CREATABLE(),

  GB_STATIC_METHOD("_init", NULL, SCREEN_init, NULL),

  GB_STATIC_METHOD("Show",  NULL, SCREEN_show,  "(Width)i(Height)i(Depth)i"),
  GB_STATIC_METHOD("Close", NULL, SCREEN_close, "[(return)i]"),
  
  GB_STATIC_METHOD("Grab",  "Surface", SCREEN_grab, "[(x)i(y)i(w)i(h)i]"),
  GB_STATIC_METHOD("Save",  NULL,      SCREEN_save, "(file)s"),

  GB_STATIC_METHOD("Clear", NULL, SCREEN_clear, "[(red)i(green)i(blue)i]"),
  GB_STATIC_METHOD("Cls",   NULL, SCREEN_clear, "[(red)i(green)i(blue)i]"),

  GB_STATIC_METHOD("Refresh", "i", SCREEN_refresh, NULL),
  GB_STATIC_METHOD("Update",  "i", SCREEN_refresh, NULL),
  
  GB_STATIC_METHOD("SetGamma", NULL, SCREEN_gamma, "(red)f(green)f(blue)f"),

  GB_STATIC_PROPERTY("Caption", "s", SCREEN_caption),
  GB_STATIC_PROPERTY("Text",    "s", SCREEN_caption),
  GB_STATIC_PROPERTY("Title",   "s", SCREEN_caption),

  GB_STATIC_PROPERTY("Fullscreen", "b", SCREEN_fullscreen),

  GB_STATIC_PROPERTY_READ("Width",  "i", SCREEN_width),
  GB_STATIC_PROPERTY_READ("W",      "i", SCREEN_width),
  GB_STATIC_PROPERTY_READ("Height", "i", SCREEN_height),
  GB_STATIC_PROPERTY_READ("H"     , "i", SCREEN_height),
  GB_STATIC_PROPERTY_READ("Depth",  "i", SCREEN_depth),
  GB_STATIC_PROPERTY_READ("D",      "i", SCREEN_depth),
  GB_STATIC_PROPERTY_READ("Showed", "b", SCREEN_showed),
  GB_STATIC_PROPERTY_READ("Id",     "i", SCREEN_id),
  GB_STATIC_PROPERTY_READ("Handle", "i", SCREEN_id),


  GB_STATIC_METHOD("SetFramerate", NULL, SCREEN_setframerate, "(rate)i"),

  GB_END_DECLARE
};
