/***************************************************************************

  timer.c

  The SDL component

  (c) 2004-2005 Laurent Carlier <lordheavy@infonie.fr>
                Benot Minisini <gambas@users.sourceforge.net>

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 1, or (at your option)
  any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

***************************************************************************/

#define __TIMER_C

#include "gambas.h"
#include "main.h"
#include "timer.h"

#include "SDL.h"

DECLARE_EVENT(ctimer);

Uint32 myTimer(Uint32 _interval, void *_timer)
{
  GB.Raise((CTIMER *) _timer, ctimer, 0);

  return _interval;
}

/**************************************************************************

  Timer

***************************************************************************/

BEGIN_METHOD_VOID(CTIMER_new)

  THIS->id = NULL;
  THIS->interval = 1000;

END_METHOD

BEGIN_METHOD_VOID(CTIMER_free)

  if (THIS->id != NULL)
    SDL_RemoveTimer(THIS->id);

END_METHOD

BEGIN_PROPERTY(CTIMER_delay)

  if (READ_PROPERTY)
     GB.ReturnInteger(THIS->interval);
  else
    THIS->interval = VPROP(GB_INTEGER);

END_METHOD

BEGIN_PROPERTY(CTIMER_Enabled)

  int _enable;

  if (READ_PROPERTY)
    GB.ReturnBoolean((int ) THIS->id);
  else
  {
  _enable = VPROP(GB_BOOLEAN);

  if ((THIS->id != NULL && _enable) || (THIS->id == NULL && !_enable))
    return;

  if (_enable)
    THIS->id = SDL_AddTimer((THIS->interval/10)*10, myTimer, THIS);
  else
    {
    SDL_RemoveTimer(THIS->id);
    THIS->id = NULL;
    }
  }

END_PROPERTY

/**************************************************************************/

GB_DESC Ctimer[] =
{
  GB_DECLARE("Timer", sizeof(CTIMER)),

  GB_EVENT("Timer", NULL, NULL, &ctimer),

  GB_METHOD("_new", NULL, CTIMER_new, NULL),
  GB_METHOD("_free", NULL, CTIMER_free, NULL),

  GB_PROPERTY("Delay", "i", CTIMER_delay),
  GB_PROPERTY("Enabled", "b", CTIMER_Enabled),

  GB_END_DECLARE
};
